using System;
using System.Collections.Generic;
using EnPalermo.Lit;
using EnPalermo.Mem;

namespace EnPalermo.Arr
{



    #region Enums

    /// <summary>
    /// Opciones de combinacin.
    /// </summary>
    public enum CombineMode
    {
        /// <summary>
        /// AND: Selecciona elementos que se encuentran tanto a la IZQ como a la DER.
        /// </summary>
        AND,
        /// <summary>
        /// OR: Selecciona elementos que se encuentran a la IZQ o DER.
        /// </summary>
        OR,
        /// <summary>
        /// XOR: Selecciona elementos que se encuentran a la IZQ o DER pero NO en las 2.
        /// </summary>
        XOR,
        /// <summary>
        /// NOT: Selecciona elementos que se encuentran a la IZQ (a) pero NO en la DER (b).
        /// </summary>
        NOT,
    }

    #endregion


    #region Delegados Genericos

    /// <summary>
    /// Delegado evaluador.
    /// </summary>
    /// <typeparam name="T">Tipo.</typeparam>
    /// <param name="x">Elemento x.</param>
    /// <param name="y">Elemento y.</param>
    /// <returns>True si x e y coinciden.</returns>
    public delegate bool Evaluator<T>(T x, T y);

    /// <summary>
    /// Delegado comparador.
    /// </summary>
    /// <typeparam name="T">Tipo.</typeparam>
    /// <param name="x">Elemento x.</param>
    /// <param name="y">Elemento y.</param>
    /// <returns>-1 si x menor a y, 0 si coinciden, 1 si x mayor a y.</returns>
    public delegate int Comparetor<T>(T x, T y);

    /// <summary>
    /// Delegado Condicional.
    /// </summary>
    /// <param name="e">Elemento.</param>
    /// <returns>True si supera la condicin.</returns>
    public delegate bool Conditional<T>(T e);

    #endregion


    /// <summary>
    /// Funciones estticas de comparacin, bsqueda y reemplazo con arrays unidimensionales.
    /// </summary>
    public static class ArrComp
    {
        #region IndexOf

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(byte[] arr, byte f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (byte* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(sbyte[] arr, sbyte f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (sbyte* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(char[] arr, char f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (char* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(short[] arr, short f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (short* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(ushort[] arr, ushort f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (ushort* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(int[] arr, int f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (int* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(uint[] arr, uint f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (uint* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(long[] arr, long f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (long* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(ulong[] arr, ulong f, bool last)
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (ulong* p = arr)
                {
                    if (!last)
                        return Comp.IndexOf(p, l, f);
                    return Comp.LastIndexOf(p, l, f);
                }
            }
        }

 
        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(byte[] arr, int i, int l, byte f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (byte* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }


        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(sbyte[] arr, int i, int l, sbyte f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (sbyte* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(char[] arr, int i, int l, char f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (char* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }
        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(short[] arr, int i, int l, short f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (short* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(ushort[] arr, int i, int l, ushort f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (ushort* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }


        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(int[] arr, int i, int l, int f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (int* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(uint[] arr, int i, int l, uint f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (uint* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(long[] arr, int i, int l, long f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (long* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(ulong[] arr, int i, int l, ulong f, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return -1;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (ulong* p = arr)
                {
                    if (!last)
                        al = Comp.IndexOf(p + i, l, f);
                    else
                        al = Comp.LastIndexOf(p + i, l, f);
                }
            }
            if (al != -1)
                return al + i;
            return -1;
        }

        #endregion
        #region IndexOf Generico

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        /// <typeparam name="T">Tipo (IEquatable).</typeparam>
        public static int IndexOf<T>(T[] arr, T f, bool last) where T : IEquatable<T>
        {
            int l = arr.Length;
            if (l == 0)
                return -1;
            int i;

            if (f == null)
            {
                if (!last)
                {
                    i = 0;
                    do
                    {
                        if (Object.ReferenceEquals(arr[i], null))
                            return i;
                    } while (++i != l);
                }
                else
                {
                    do
                    {
                        if (Object.ReferenceEquals(arr[--l], null))
                            return l;
                    } while (l != 0);
                }
            }
            else if (!last)
            {
                i = 0;
                do
                {
                    if (f.Equals(arr[i]))
                        return i;
                } while (++i != l);
            }
            else
            {
                do
                {
                    if (f.Equals(arr[--l]))
                        return l;
                } while (l != 0);
            }
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <returns>Elemento encontrado o -1.</returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static int IndexOf<T>(T[] arr, T f, bool last, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            int l = arr.Length;
            if (l == 0)
                return -1;
            int i;

            if (f == null)
            {
                if (!last)
                {
                    i = 0;
                    do
                    {
                        if (Object.ReferenceEquals(arr[i], null))
                            return i;
                    } while (++i != l);
                }
                else
                {
                    do
                    {
                        if (Object.ReferenceEquals(arr[--l], null))
                            return l;
                    } while (l != 0);
                }
            }
            else if (!last)
            {
                i = 0;
                do
                {
                    if (comp.Compare(arr[i], f) == 0)
                        return i;
                } while (++i != l);
            }
            else
            {
                do
                {
                    if (comp.Compare(arr[--l], f) == 0)
                        return l;
                } while (l != 0);
            }
            return -1;
        }


        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        /// <typeparam name="T">Tipo (IEquatable).</typeparam>
        public static int IndexOf<T>(T[] arr, int i, int l, T f, bool last) where T : IEquatable<T>
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int la = arr.Length;
            if (l == 0 || i >= la)
                return -1;
            if (l < 0 || (l += i) > la)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = la;
            }
            if (f == null)
            {
                if (!last)
                {
                    do
                    {
                        if (Object.ReferenceEquals(arr[i], null))
                            return i;
                    } while (++i != l);
                }
                else
                {
                    do
                    {
                        if (Object.ReferenceEquals(arr[--l], null))
                            return l;
                    } while (l != i);
                }
            }
            else if (!last)
            {
                do
                {
                    if (f.Equals(arr[i]))
                        return i;
                } while (++i != l);
            }
            else
            {
                do
                {
                    if (f.Equals(arr[--l]))
                        return l;
                } while (l != i);
            }
            return -1;
        }


        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <returns>Elemento encontrado o -1.</returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static int IndexOf<T>(T[] arr, int i, int l, T f, bool last, IComparer<T> comp)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int la = arr.Length;
            if (l == 0 || i >= la)
                return -1;
            if (l < 0 || (l += i) > la)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = la;
            }
            if (comp == null)
                comp = Comparer<T>.Default;
            if (f == null)
            {
                if (!last)
                {
                    do
                    {
                        if (Object.ReferenceEquals(arr[i], null))
                            return i;
                    } while (++i != l);
                }
                else
                {
                    do
                    {
                        if (Object.ReferenceEquals(arr[--l], null))
                            return l;
                    } while (l != i);
                }
            }
            else if (!last)
            {
                do
                {
                    if (comp.Compare(arr[i], f) == 0)
                        return i;
                } while (++i != l);
            }
            else
            {
                do
                {
                    if (comp.Compare(arr[--l], f) == 0)
                        return l;
                } while (l != i);
            }
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <param name="comp">Comparador.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static int IndexOf<T>(T[] arr, int i, int l, T f, bool last, Comparetor<T> comp)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int la = arr.Length;
            if (l == 0 || i >= la)
                return -1;
            if (l < 0 || (l += i) > la)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = la;
            }
            if (!last)
            {
                do
                {
                    if (comp(arr[i], f) == 0)
                        return i;
                } while (++i != l);
            }
            else
            {
                do
                {
                    if (comp(arr[--l], f) == 0)
                        return l;
                } while (l != i);
            }
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <param name="eval">Evaluador.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static int IndexOf<T>(T[] arr, int i, int l, T f, bool last, Evaluator<T> eval)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int la = arr.Length;
            if (l == 0 || i >= la)
                return -1;
            if (l < 0 || (l += i) > la)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = la;
            }

            if (!last)
            {
                do
                {
                    if (eval(arr[i], f))
                        return i;
                } while (++i != l);
            }
            else
            {
                do
                {
                    if (eval(arr[--l], f))
                        return l;
                } while (l != i);
            }
            return -1;
        }

        /// <summary>
        /// Busca en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del elemento.</param>
        /// <param name="eval">Evaluador.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static int IndexOf<T>(T[] arr, int i, int l, T f, bool last, Conditional<T> eval)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int la = arr.Length;
            if (l == 0 || i >= la)
                return -1;
            if (l < 0 || (l += i) > la)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = la;
            }

            if (!last)
            {
                do
                {
                    if (eval(arr[i]))
                        return i;
                } while (++i != l);
            }
            else
            {
                do
                {
                    if (eval(arr[--l]))
                        return l;
                } while (l != i);
            }
            return -1;
        }


        #endregion

        #region Exist

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(byte[] arr, byte f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (byte* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(sbyte[] arr, sbyte f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (sbyte* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(char[] arr, char f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (char* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(short[] arr, short f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (short* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(ushort[] arr, ushort f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (ushort* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(int[] arr, int f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (int* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(uint[] arr, uint f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (uint* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(long[] arr, long f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (long* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(ulong[] arr, ulong f)
        {
            int l = arr.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (ulong* p = arr)
                    return Comp.Exist(p, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(byte[] arr, int i, int l, byte f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (byte* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(sbyte[] arr, int i, int l, sbyte f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (sbyte* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(char[] arr, int i, int l, char f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (char* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(short[] arr, int i, int l, short f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (short* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(ushort[] arr, int i, int l, ushort f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (ushort* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(int[] arr, int i, int l, int f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (int* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(uint[] arr, int i, int l, uint f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (uint* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(long[] arr, int i, int l, long f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (long* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        public static bool Exist(ulong[] arr, int i, int l, ulong f)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int al = arr.Length;
            if (l == 0 || i >= al)
                return false;
            if (l < 0 || l + i > al)
            {
                if (l < -1)
                    throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE_NEGATIVE);
                l = al - i;
            }
            unsafe
            {
                fixed (ulong* p = arr)
                    return Comp.Exist(p + i, l, f);
            }
        }


        #endregion


        #region Exist Generico

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe en el array.</returns>
        /// <typeparam name="T">Tipo (IEquatable).</typeparam>
        public static bool Exist<T>(T[] arr, T f) where T : IEquatable<T>
        {
            return IndexOf(arr, f, false) != -1;
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <returns>True si existe en el array.</returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static bool Exist<T>(T[] arr, T f, IComparer<T> comp)
        {
            return IndexOf(arr, f, false, comp) != -1;
        }


        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <returns>True si existe en el array.</returns>
        /// <typeparam name="T">Tipo (IEquatable).</typeparam>
        public static bool Exist<T>(T[] arr, int i, int l, T f) where T : IEquatable<T>
        {
            return IndexOf(arr, i, l, f, false) != -1;
        }

        /// <summary>
        /// Comprueba si el elemento existe en el en Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="i">Inicio de la bsqueda. No puede ser negativo. Si es igual o mayor al largo de array, retorna -1 sin generar excepcin.</param>
        /// <param name="l">Largo de la bsqueda. Si es -1 o si i+l exceden el largo de array, buscar en el resto del array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <returns>True si existe en el array.</returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static bool Exist<T>(T[] arr, int i, int l, T f, IComparer<T> comp)
        {
            return IndexOf(arr, i, l, f, false, comp) != -1;
        }

        #endregion


        #region Equal

        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(byte[] a1, byte[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (byte* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }

        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(sbyte[] a1, sbyte[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (sbyte* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }


        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(char[] a1, char[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (char* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }



        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(short[] a1, short[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (short* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }


        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(ushort[] a1, ushort[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (ushort* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }


        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(int[] a1, int[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (int* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }


        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(uint[] a1, uint[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (uint* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }



        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(long[] a1, long[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (long* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }

        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(ulong[] a1, ulong[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (ulong* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }

        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(PByte[] a1, PByte[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (PByte* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }

        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(PChar[] a1, PChar[] a2)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            unsafe
            {
                fixed (PChar* p1 = a1, p2 = a2)
                    return Comp.Equal(p1, p2, l);
            }
        }

        #endregion



        #region Combine Generico

        /// <summary>
        /// Genera un nuevo array con el producto de la combinacin de los arrays a y b.
        /// </summary>
        /// <param name="a">Array A (Izquierda).
        /// En el caso de tipos de referencia, no debe contener elementos nulos (no verifica!).</param>
        /// <param name="op">
        /// Operador de combinacin entre a y b.
        /// </param>
        /// <param name="b">Array B (derecha).
        /// En el caso de tipos de referencia, no debe contener elementos nulos (no verifica!).
        /// </param>
        /// <param name="asRef">
        /// Si true, puede modificar y retornar las referencias de los arrays de 
        /// entrada. De este modo, el contenido de a y b dejar de ser confiable luego de llamar a
        /// este mtodo. Este parmetro es ignorado en OR y XOR.
        /// </param>
        /// <returns>
        /// Retorna el array resultante de las comparaciones.
        /// </returns>
        /// <typeparam name="T">Tipo (IEquatable).</typeparam>
        public static T[] Combine<T>(T[] a, CombineMode op, T[] b, bool asRef) where T : IEquatable<T>
        {
            int la = a.Length;
            int lb = b.Length;
            if (la == 0)
            {
                if (lb == 0 || op == CombineMode.AND || op == CombineMode.NOT)
                    return a;
                if (asRef)
                    return b;
                return ArrGeneral.Clone(b);
            }

            if (lb == 0)
            {
                if (op == CombineMode.AND)
                    return b;
                if (asRef)
                    return a;
                return ArrGeneral.Clone(a);
            }
            int tot = 0, x = 0, y;
            T[] tmp;
            T act;

            switch (op)
            {
                case CombineMode.NOT:
                    if (asRef)
                        tmp = a;
                    else
                        tmp = new T[la];
                    do
                    {
                        y = 0;
                        act = a[x];
                        while (!act.Equals(b[y]) && ++y != lb)
                            ;
                        if (y == lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    if (tot == la)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
                case CombineMode.AND:
                    if (la > lb)
                    {
                        tmp = b;
                        b = a;
                        a = tmp;
                        y = la;
                        la = lb;
                        lb = y;
                        if (!asRef)
                            tmp = new T[la];
                    }
                    else if (asRef)
                        tmp = a;
                    else
                        tmp = new T[la];
                    do
                    {
                        y = 0;
                        act = a[x];
                        while (!act.Equals(b[y]) && ++y != lb)
                            ;
                        if (y != lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    if (tot == la)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
                case CombineMode.XOR:
                    tmp = new T[la + lb];
                    do
                    {
                        y = 0;
                        act = a[x];
                        while (!act.Equals(b[y]) && ++y != lb)
                            ;
                        if (y == lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    x = 0;
                    do
                    {
                        y = 0;
                        act = b[x];
                        while (!act.Equals(a[y]) && ++y != la)
                            ;
                        if (y == la)
                            tmp[tot++] = act;
                    } while (++x != lb);
                    if (tot == la + lb)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
            }
            // OR
            tmp = new T[la + lb];
            do
            {
                tmp[tot++] = a[x];
            } while (++x != la);
            x = 0;
            do
            {
                y = 0;
                act = b[x];
                while (!act.Equals(a[y]) && ++y != la)
                    ;
                if (y == la)
                    tmp[tot++] = act;
            } while (++x != lb);
            if (tot == la + lb)
                return tmp;
            return ArrGeneral.Clone(tmp, 0, tot, 0, tot);

        }


        /// <summary>
        /// Genera un nuevo array con el producto de la combinacin de los arrays a y b.
        /// </summary>
        /// <param name="a">Array A (Izquierda).
        /// En el caso de tipos de referencia, no debe contener elementos nulos (no verifica!).</param>
        /// <param name="op">
        /// Operador de combinacin entre a y b.
        /// </param>
        /// <param name="b">Array B (derecha).
        /// En el caso de tipos de referencia, no debe contener elementos nulos (no verifica!).
        /// </param>
        /// <param name="asRef">
        /// Si true, puede modificar y retornar las referencias de los arrays de 
        /// entrada. De este modo, el contenido de a y b dejar de ser confiable luego de llamar a
        /// este mtodo. Este parmetro es ignorado en OR y XOR.
        /// </param>
        /// <param name="eval">Delegado evaluador: bool Evaluator(T x,T y).</param>
        /// <returns>
        /// Retorna el array resultante de las comparaciones.
        /// </returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static T[] Combine<T>(T[] a, CombineMode op, T[] b, bool asRef, Evaluator<T> eval)
        {
            int la = a.Length;
            int lb = b.Length;
            if (la == 0)
            {
                if (lb == 0 || op == CombineMode.AND || op == CombineMode.NOT)
                    return a;
                if (asRef)
                    return b;
                return ArrGeneral.Clone(b);
            }

            if (lb == 0)
            {
                if (op == CombineMode.AND)
                    return b;
                if (asRef)
                    return a;
                return ArrGeneral.Clone(a);
            }
            int tot = 0, x = 0, y;
            T[] tmp;
            T act;

            switch (op)
            {
                case CombineMode.NOT:
                    if (asRef)
                        tmp = a;
                    else
                        tmp = new T[la];
                    do
                    {
                        y = 0;
                        act = a[x];
                        while (!eval(b[y], act) && ++y != lb)
                            ;
                        if (y == lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    if (tot == la)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
                case CombineMode.AND:
                    if (la > lb)
                    {
                        tmp = b;
                        b = a;
                        a = tmp;
                        y = la;
                        la = lb;
                        lb = y;
                        if (!asRef)
                            tmp = new T[la];
                    }
                    else if (asRef)
                        tmp = a;
                    else
                        tmp = new T[la];
                    do
                    {
                        y = 0;
                        act = a[x];
                        while (!eval(b[y], act) && ++y != lb)
                            ;
                        if (y != lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    if (tot == la)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
                case CombineMode.XOR:
                    tmp = new T[la + lb];
                    do
                    {
                        y = 0;
                        act = a[x];
                        while (!eval(b[y], act) && ++y != lb)
                            ;
                        if (y == lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    x = 0;
                    do
                    {
                        y = 0;
                        act = b[x];
                        while (!eval(a[y], act) && ++y != la)
                            ;
                        if (y == la)
                            tmp[tot++] = act;
                    } while (++x != lb);
                    if (tot == la + lb)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
            }
            // OR
            tmp = new T[la + lb];
            do
            {
                tmp[tot++] = a[x];
            } while (++x != la);
            x = 0;
            do
            {
                y = 0;
                act = b[x];
                while (!eval(a[y], act) && ++y != la)
                    ;
                if (y == la)
                    tmp[tot++] = act;
            } while (++x != lb);
            if (tot == la + lb)
                return tmp;
            return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
        }


        /// <summary>
        /// Genera un nuevo array con el producto de la combinacin de los arrays a y b.
        /// </summary>
        /// <param name="a">Array A (Izquierda).
        /// En el caso de tipos de referencia, no debe contener elementos nulos (no verifica!).</param>
        /// <param name="op">
        /// Operador de combinacin entre a y b.
        /// </param>
        /// <param name="b">Array B (derecha).
        /// En el caso de tipos de referencia, no debe contener elementos nulos (no verifica!).
        /// </param>
        /// <param name="asRef">
        /// Si true, puede modificar y retornar las referencias de los arrays de 
        /// entrada. De este modo, el contenido de a y b dejar de ser confiable luego de llamar a
        /// este mtodo. Este parmetro es ignorado en OR y XOR.
        /// </param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <returns>
        /// Retorna el array resultante de las comparaciones (puede estar vacio).
        /// </returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static T[] Combine<T>(T[] a, CombineMode op, T[] b, bool asRef, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            int la = a.Length;
            int lb = b.Length;
            if (la == 0)
            {
                if (lb == 0 || op == CombineMode.AND || op == CombineMode.NOT)
                    return a;
                if (asRef)
                    return b;
                return ArrGeneral.Clone(b);
            }

            if (lb == 0)
            {
                if (op == CombineMode.AND)
                    return b;
                if (asRef)
                    return a;
                return ArrGeneral.Clone(a);
            }
            int tot = 0, x = 0, y;
            T[] tmp;
            T act;

            switch (op)
            {
                case CombineMode.NOT:
                    if (asRef)
                        tmp = a;
                    else
                        tmp = new T[la];
                    do
                    {
                        y = 0;
                        act = a[x];
                        while (comp.Compare(b[y], act) != 0 && ++y != lb)
                            ;
                        if (y == lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    if (tot == la)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
                case CombineMode.AND:
                    if (la > lb)
                    {
                        tmp = b;
                        b = a;
                        a = tmp;

                        y = la;
                        la = lb;
                        lb = y;
                        if (!asRef)
                            tmp = new T[la];
                    }
                    else if (asRef)
                        tmp = a;
                    else
                        tmp = new T[la];
                    do
                    {
                        y = 0;
                        act = a[x];

                        while (comp.Compare(b[y], act) != 0 && ++y != lb) ;

                        if (y != lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    if (tot == la)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
                case CombineMode.XOR:
                    tmp = new T[la + lb];
                    do
                    {
                        y = 0;
                        act = a[x];
                        while (comp.Compare(b[y], act) != 0 && ++y != lb)
                            ;
                        if (y == lb)
                            tmp[tot++] = act;
                    } while (++x != la);
                    x = 0;
                    do
                    {
                        y = 0;
                        act = b[x];
                        while (comp.Compare(a[y], act) != 0 && ++y != la)
                            ;
                        if (y == la)
                            tmp[tot++] = act;
                    } while (++x != lb);
                    if (tot == la + lb)
                        return tmp;
                    return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
            }
            // OR
            tmp = new T[la + lb];
            do
            {
                tmp[tot++] = a[x];
            } while (++x != la);
            x = 0;
            do
            {
                y = 0;
                act = b[x];
                while (comp.Compare(a[y], act) != 0 && ++y != la)
                    ;
                if (y == la)
                    tmp[tot++] = act;
            } while (++x != lb);
            if (tot == la + lb)
                return tmp;
            return ArrGeneral.Clone(tmp, 0, tot, 0, tot);
        }

        #endregion


        #region Eliminar Duplicados Generico

        /// <summary>
        /// Genera una copia del array eliminando duplicados y nulos (solo en tipos de referencia).
        /// </summary>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        /// <param name="arr">Array.</param>
        /// <param name="sort">
        /// Si true, ordena el array (orden ascendente).
        /// Si false, preserva el orden original (mas lento y no necesariamente preserva la primer aparicin del elemento).
        /// </param>
        /// <returns>
        /// Array con los elementos seleccionados. 
        /// </returns>
        public static T[] RemoveDups<T>(T[] arr, bool sort) where T : IComparable<T>
        {
            int l = arr.Length;
            if (l < 2)
            {
                if (l == 1 && !Object.ReferenceEquals(arr[0], null))
                    return new T[1] { arr[0] };
                return new T[0];
            }
            T[] copia = ArrGeneral.Clone(arr);
            int[] index = new int[l];
            int a = 1;
            if (sort)
            {
                ArrSort.Ascending(copia, 0, l);
            }
            else
            {
                do
                {
                    index[a] = a;
                } while (++a != l);
                a = 1;
                ArrSort.Ascending(copia, index, 0, l);
            }
            int elims = 0;
            if (default(T) == null)
            {
                if (Object.ReferenceEquals(copia[0], null))
                {
                    elims = 1;
                    index[0] = -1;
                }
                do
                {
                    if (Object.ReferenceEquals(copia[a], null) || copia[a].CompareTo(copia[a - 1]) == 0)
                    {
                        elims++;
                        index[a] = -1;
                    }

                } while (++a != l);
            }
            else
            {
                do
                {
                    if (copia[a].CompareTo(copia[a - 1]) == 0)
                    {
                        elims++;
                        index[a] = -1;
                    }

                } while (++a != l);
            }

            if (elims == 0)
            {
                if (sort)
                    return copia;
                Array.Copy(arr, 0, copia, 0, l);
                return copia;
            }
            if (elims == l)
                return new T[0];

            T[] ret = new T[l - elims];
            a = 0;
            int b = 0;
            if (sort)
            {
                do
                {
                    if (index[a] != -1)
                    {
                        ret[b++] = copia[a];
                    }
                } while (++a != l);

            }
            else
            {
                ArrSort.Ascending(index, 0, l);
                while (index[++a] == -1)
                    ;
                do
                {

                    ret[b++] = arr[index[a]];
                } while (++a != l);
            }
            return ret;
        }


        /// <summary>
        /// Genera una copia del array eliminando duplicados y nulos (solo en tipos de referencia).
        /// </summary>
        /// <typeparam name="T">Tipo.</typeparam>
        /// <param name="arr">Array.</param>
        /// <param name="sort">
        /// Si true, ordena el array (orden ascendente).
        /// Si false, preserva el orden original (mas lento y no necesariamente preserva la primer aparicin del elemento).
        /// </param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <returns>
        /// Array con los elementos seleccionados. 
        /// </returns>
        public static T[] RemoveDups<T>(T[] arr, bool sort, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            int l = arr.Length;
            if (l < 2)
            {
                if (l == 1 && !Object.ReferenceEquals(arr[0], null))
                    return new T[1] { arr[0] };
                return new T[0];
            }
            T[] copia = ArrGeneral.Clone(arr, 0, l, 0, l);
            int[] index = new int[l];
            int a = 1;
            if (sort)
            {
                ArrSort.Ascending(copia, 0, l, comp);
            }
            else
            {
                do
                {
                    index[a] = a;
                } while (++a != l);
                a = 1;
                ArrSort.Ascending(copia, index, 0, l, comp);
            }
            int elims = 0;
            if (default(T) == null)
            {
                if (Object.ReferenceEquals(copia[0], null))
                {
                    elims = 1;
                    index[0] = -1;
                }
                do
                {
                    if (Object.ReferenceEquals(copia[a], null) || comp.Compare(copia[a], copia[a - 1]) == 0)
                    {
                        elims++;
                        index[a] = -1;
                    }

                } while (++a != l);
            }
            else
            {
                do
                {
                    if (comp.Compare(copia[a], copia[a - 1]) == 0)
                    {
                        elims++;
                        index[a] = -1;
                    }

                } while (++a != l);
            }

            if (elims == 0)
            {
                if (sort)
                    return copia;
                System.Array.Copy(arr, 0, copia, 0, l);
                return copia;
            }
            if (elims == l)
                return new T[0];

            T[] ret = new T[l - elims];
            a = 0;
            int b = 0;

            if (sort)
            {
                do
                {
                    if (index[a] != -1)
                    {
                        ret[b++] = copia[a];
                    }
                } while (++a != l);

            }
            else
            {
                ArrSort.Ascending(index, 0, l);
                while (index[++a] == -1)
                    ;
                do
                {
                    ret[b++] = arr[index[a]];
                } while (++a != l);
            }

            return ret;
        }

        #endregion

        #region Equal Generico

        /// <summary>
        /// Verifica si los Array son iguales.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <returns>True si son iguales.</returns>
        /// <typeparam name="T">Tipo (IEquatable).</typeparam>
        public static bool Equal<T>(T[] a1, T[] a2) where T : IEquatable<T>
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            int a = 0;
            if (default(T) == null)
            {
                do
                {
                    if (Object.ReferenceEquals(a1[a], null))
                    {
                        if (!Object.ReferenceEquals(a2[a], null))
                            return false;
                    }
                    else if (!a1[a].Equals(a2[a]))
                        return false;

                } while (++a != l);
            }
            else
            {
                do
                {
                    if (!a1[a].Equals(a2[a]))
                        return false;
                } while (++a != l);
            }
            return true;
        }

        /// <summary>
        /// Verifica si los Array son iguales de acuerdo al comparador.
        /// </summary>
        /// <param name="a1">Array 1.</param>
        /// <param name="a2">Array 2.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <returns>True si son iguales.</returns>
        /// <typeparam name="T">Tipo.</typeparam>
        public static bool Equal<T>(T[] a1, T[] a2, IComparer<T> comp)
        {
            if (Object.ReferenceEquals(a1, a2))
                return true;
            if (a1 == null || a2 == null)
                return false;
            int l = a1.Length;
            if (l != a2.Length)
                return false;
            if (l == 0)
                return true;
            if (comp == null)
                comp = Comparer<T>.Default;
            int a = 0;
            do
            {
                if (comp.Compare(a1[a], a2[a]) != 0)
                    return false;
            } while (++a != l);
            return true;
        }


        #endregion
        
    }
}
