using System;
using System.Collections.Generic;
using EnPalermo.Lit;
using EnPalermo.Mem;

namespace EnPalermo.Arr
{

    /// <summary>
    /// Funciones estticas relativas a la generacin y operacin con arrays unidimensionales.
    /// </summary>
    public static class ArrGeneral
    {

        #region Arrays vacios y constructor esttico

        /// <summary>
        /// Array de objetos vacio.
        /// </summary>
        public static readonly object[] ArrayEmptyObject;

        /// <summary>
        /// Array de cadenas vacio.
        /// </summary>
        public static readonly string[] ArrayEmptyString;

        /// <summary>
        /// Array de bytes vacio.
        /// </summary>
        public static readonly byte[] ArrayEmptyByte;

        /// <summary>
        /// Array de sbytes vacio.
        /// </summary>
        public static readonly sbyte[] ArrayEmptySByte;

        /// <summary>
        /// Array de shorts vacio.
        /// </summary>
        public static readonly short[] ArrayEmptyShort;

        /// <summary>
        /// Array de ushorts vacio.
        /// </summary>
        public static readonly ushort[] ArrayEmptyUShort;

        /// <summary>
        /// Array de chars vacio.
        /// </summary>
        public static readonly char[] ArrayEmptyChar;

        /// <summary>
        /// Array de ints vacio.
        /// </summary>
        public static readonly int[] ArrayEmptyInt;

        /// <summary>
        /// Array de uints vacio.
        /// </summary>
        public static readonly uint[] ArrayEmptyUInt;

        /// <summary>
        /// Array de longs vacio.
        /// </summary>
        public static readonly long[] ArrayEmptyLong;

        /// <summary>
        /// Array de ulongs vacio.
        /// </summary>
        public static readonly ulong[] ArrayEmptyULong;


        static ArrGeneral() 
        {
            ArrayEmptyObject = new object[0];
            ArrayEmptyString = new string[0];
            ArrayEmptyByte = new byte[0];
            ArrayEmptySByte = new sbyte[0];
            ArrayEmptyShort = new short[0];
            ArrayEmptyUShort = new ushort[0];
            ArrayEmptyChar = new char[0];
            ArrayEmptyInt = new int[0];
            ArrayEmptyUInt = new uint[0];
            ArrayEmptyLong = new long[0];
            ArrayEmptyULong = new ulong[0];
 
        }

        #endregion


        #region Clone Punteros

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static byte[] Clone(byte* arr, int l)
        {
            byte[] ret = new byte[l];
            if (l != 0)
                fixed (byte* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }


        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static sbyte[] Clone(sbyte* arr, int l)
        {
            sbyte[] ret = new sbyte[l];
            if (l != 0)
                fixed (sbyte* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static char[] Clone(char* arr, int l)
        {
            char[] ret = new char[l];
            if (l != 0)
                fixed (char* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static short[] Clone(short* arr, int l)
        {
            short[] ret = new short[l];
            if (l != 0)
                fixed (short* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static ushort[] Clone(ushort* arr, int l)
        {
            ushort[] ret = new ushort[l];
            if (l != 0)
                fixed (ushort* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static int[] Clone(int* arr, int l)
        {
            int[] ret = new int[l];
            if (l != 0)
                fixed (int* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static uint[] Clone(uint* arr, int l)
        {
            uint[] ret = new uint[l];
            if (l != 0)
                fixed (uint* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static long[] Clone(long* arr, int l)
        {
            long[] ret = new long[l];
            if (l != 0)
                fixed (long* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static ulong[] Clone(ulong* arr, int l)
        {
            ulong[] ret = new ulong[l];
            if (l != 0)
                fixed (ulong* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static PByte[] Clone(PByte* arr, int l)
        {
            PByte[] ret = new PByte[l];
            if (l != 0)
                fixed (PByte* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo de la copia.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static PChar[] Clone(PChar* arr, int l)
        {
            PChar[] ret = new PChar[l];
            if (l != 0)
                fixed (PChar* pd = ret)
                    General.Copy(arr, pd, l);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static byte[] Clone(byte* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            byte[] ret = new byte[lTotalDest];
            if (lCop != 0)
                fixed (byte* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static sbyte[] Clone(sbyte* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            sbyte[] ret = new sbyte[lTotalDest];
            if (lCop != 0)
                fixed (sbyte* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static char[] Clone(char* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            char[] ret = new char[lTotalDest];
            if (lCop != 0)
                fixed (char* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static short[] Clone(short* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            short[] ret = new short[lTotalDest];
            if (lCop != 0)
                fixed (short* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static ushort[] Clone(ushort* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            ushort[] ret = new ushort[lTotalDest];
            if (lCop != 0)
                fixed (ushort* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static int[] Clone(int* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int[] ret = new int[lTotalDest];
            if (lCop != 0)
                fixed (int* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static uint[] Clone(uint* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            uint[] ret = new uint[lTotalDest];
            if (lCop != 0)
                fixed (uint* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static long[] Clone(long* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            long[] ret = new long[lTotalDest];
            if (lCop != 0)
                fixed (long* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static ulong[] Clone(ulong* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            ulong[] ret = new ulong[lTotalDest];
            if (lCop != 0)
                fixed (ulong* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static PByte[] Clone(PByte* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            PByte[] ret = new PByte[lTotalDest];
            if (lCop != 0)
                fixed (PByte* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public unsafe static PChar[] Clone(PChar* arr, int lCop, int iDest, int lTotalDest)
        {
            if (iDest < 0 || lTotalDest < 0 || lCop < 0 || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            PChar[] ret = new PChar[lTotalDest];
            if (lCop != 0)
                fixed (PChar* pd = ret)
                    General.Copy(arr, pd + iDest, lCop);
            return ret;
        }

        #endregion

        #region Clone

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static byte[] Clone(byte[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            byte[] ret = new byte[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (byte* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }


        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static sbyte[] Clone(sbyte[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            sbyte[] ret = new sbyte[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (sbyte* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static char[] Clone(char[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            char[] ret = new char[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (char* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static short[] Clone(short[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            short[] ret = new short[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (short* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static ushort[] Clone(ushort[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            ushort[] ret = new ushort[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (ushort* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static int[] Clone(int[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int[] ret = new int[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (int* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }


        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static uint[] Clone(uint[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            uint[] ret = new uint[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (uint* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static long[] Clone(long[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            long[] ret = new long[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (long* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static ulong[] Clone(ulong[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            ulong[] ret = new ulong[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (ulong* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static PChar[] Clone(PChar[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            PChar[] ret = new PChar[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (PChar* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static PByte[] Clone(PByte[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            int l = arr.Length;
            if (iArr < 0 || iDest < 0 || lTotalDest < 0 || lCop < 0 || l < iArr + lCop || lTotalDest < iDest + lCop)
                throw new ArgumentOutOfRangeException("iArr,lCop,iDest,lTotalDest", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            PByte[] ret = new PByte[lTotalDest];
            if (lCop != 0)
            {
                unsafe
                {
                    fixed (PByte* po = arr, pd = ret)
                        General.Copy(po + iArr, pd + iDest, lCop);
                }
            }
            return ret;
        }


        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static byte[] Clone(byte[] arr)
        {
            int l = arr.Length;
            byte[] ret = new byte[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (byte* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static sbyte[] Clone(sbyte[] arr)
        {
            int l = arr.Length;
            sbyte[] ret = new sbyte[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (sbyte* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static char[] Clone(char[] arr)
        {
            int l = arr.Length;
            char[] ret = new char[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (char* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static short[] Clone(short[] arr)
        {
            int l = arr.Length;
            short[] ret = new short[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (short* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static ushort[] Clone(ushort[] arr)
        {
            int l = arr.Length;
            ushort[] ret = new ushort[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (ushort* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static int[] Clone(int[] arr)
        {
            int l = arr.Length;
            int[] ret = new int[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (int* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static uint[] Clone(uint[] arr)
        {
            int l = arr.Length;
            uint[] ret = new uint[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (uint* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static long[] Clone(long[] arr)
        {
            int l = arr.Length;
            long[] ret = new long[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (long* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static ulong[] Clone(ulong[] arr)
        {
            int l = arr.Length;
            ulong[] ret = new ulong[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (ulong* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }


        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static PByte[] Clone(PByte[] arr)
        {
            int l = arr.Length;
            PByte[] ret = new PByte[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (PByte* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }


        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Array copiado o vacio.</returns>
        public static PChar[] Clone(PChar[] arr)
        {
            int l = arr.Length;
            PChar[] ret = new PChar[l];
            if (l != 0)
            {
                unsafe
                {
                    fixed (PChar* po = arr, pd = ret)
                        General.Copy(po, pd, l);
                }
            }
            return ret;
        }


        #endregion

        #region Sum

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static byte[] Sum(params byte[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            byte[] ret = new byte[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (byte* p = ret)
                    {
                        byte* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (byte* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static sbyte[] Sum(params sbyte[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            sbyte[] ret = new sbyte[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (sbyte* p = ret)
                    {
                        sbyte* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (sbyte* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static char[] Sum(params char[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            char[] ret = new char[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (char* p = ret)
                    {
                        char* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (char* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static short[] Sum(params short[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            short[] ret = new short[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (short* p = ret)
                    {
                        short* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (short* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static ushort[] Sum(params ushort[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            ushort[] ret = new ushort[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (ushort* p = ret)
                    {
                        ushort* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (ushort* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static int[] Sum(params int[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            int[] ret = new int[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (int* p = ret)
                    {
                        int* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (int* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static uint[] Sum(params uint[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            uint[] ret = new uint[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (uint* p = ret)
                    {
                        uint* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (uint* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static long[] Sum(params long[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            long[] ret = new long[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (long* p = ret)
                    {
                        long* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (long* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static ulong[] Sum(params ulong[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            ulong[] ret = new ulong[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (ulong* p = ret)
                    {
                        ulong* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (ulong* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static PByte[] Sum(params PByte[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            PByte[] ret = new PByte[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (PByte* p = ret)
                    {
                        PByte* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (PByte* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }


        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <returns>Copia compuesta o vacia si todos los arrays estan vacios.</returns>
        public static PChar[] Sum(params PChar[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            PChar[] ret = new PChar[tot];

            if (tot != 0)
            {
                int l;
                unsafe
                {
                    fixed (PChar* p = ret)
                    {
                        PChar* pd = p;
                        for (a = 0; a < arrays.Length; a++)
                        {
                            l = arrays[a].Length;
                            if (l != 0)
                            {
                                fixed (PChar* po = arrays[a])
                                    General.Copy(po, pd, l);
                                pd += l;
                            }
                        }
                    }
                }
            }
            return ret;
        }


        #endregion

        #region Fill

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(byte[] arr, byte fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (byte* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(byte[] arr, int i, int l, byte fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (byte* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(sbyte[] arr, sbyte fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (sbyte* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(sbyte[] arr, int i, int l, sbyte fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (sbyte* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }


        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(short[] arr, short fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (short* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(short[] arr, int i, int l, short fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (short* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(char[] arr, char fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (char* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(char[] arr, int i, int l, char fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (char* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(ushort[] arr, ushort fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (ushort* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(ushort[] arr, int i, int l, ushort fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (ushort* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(int[] arr, int fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (int* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(int[] arr, int i, int l, int fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (int* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(uint[] arr, uint fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (uint* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(uint[] arr, int i, int l, uint fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (uint* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(long[] arr, long fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (long* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(long[] arr, int i, int l, long fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (long* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(ulong[] arr, ulong fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (ulong* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(ulong[] arr, int i, int l, ulong fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (ulong* p = arr)
                    {
                        if (fill != 0)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(PByte[] arr, PByte fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (PByte* p = arr)
                    {
                        if (fill != PByte.Zero)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(PByte[] arr, int i, int l, PByte fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (PByte* p = arr)
                    {
                        if (fill != PByte.Zero)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(PChar[] arr, PChar fill)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (PChar* p = arr)
                    {
                        if (fill != PChar.Zero)
                            General.Fill(p, l, fill);
                        else
                            General.Clean(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(PChar[] arr, int i, int l, PChar fill)
        {
            int la = arr.Length;
            if (i < 0 || i + l > la)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l != 0)
            {
                unsafe
                {
                    fixed (PChar* p = arr)
                    {
                        if (fill != PChar.Zero)
                            General.Fill(p + i, l, fill);
                        else
                            General.Clean(p + i, l);
                    }
                }
            }
        }

        #endregion

        #region Fill Objetos

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(string[] arr, string fill)
        {
            if (Object.ReferenceEquals(fill, null))
                Array.Clear(arr, 0, arr.Length);
            else
                for (int a = 0; a < arr.Length; a++)
                    arr[a] = fill;
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Fill(string[] arr, int i, int l, string fill)
        {
            if (Object.ReferenceEquals(fill, null))
                Array.Clear(arr, i, l);
            else
            {

                l += i;
                for (int a = i; a < l; a++)
                    arr[i] = fill;
            }
        }

        #endregion


        #region Clone Generico

        /// <summary>
        /// Clona array en base al rango especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="iArr">Inicio de la copia en el array origen.</param>
        /// <param name="iDest">Inicio de la copia en el array destino.</param>
        /// <param name="lCop">Largo de la copia en array origen.</param>
        /// <param name="lTotalDest">Largo total del nuevo array.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        /// <returns>Array copiado o vacio si lTotalDest es 0.</returns>
        public static T[] Clone<T>(T[] arr, int iArr, int lCop, int iDest, int lTotalDest)
        {
            T[] ret = new T[lTotalDest];
            if (lCop != 0)
                Array.Copy(arr, iArr, ret, iDest, lCop);
            return ret;
        }

        /// <summary>
        /// Clona array.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        /// <returns>Array copiado o vacio.</returns>
        public static T[] Clone<T>(T[] arr)
        {
            int l = arr.Length;
            T[] ret = new T[l];
            if (l != 0)
                Array.Copy(arr, 0, ret, 0, l);
            return ret;
        }
        
        #endregion
        
        #region Sum Generico

        /// <summary>
        /// Devuelve un array con la copia compuesta de todos los arrays en parmetro.
        /// </summary>
        /// <param name="arrays">Arrays a combinar.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        /// <returns>Copia compuesta.</returns>
        public static T[] Sum<T>(params T[][] arrays)
        {
            int tot = 0, a;
            for (a = 0; a < arrays.Length; a++)
                tot += arrays[a].Length;
            T[] ret = new T[tot];
            if (tot != 0)
            {
                int l;
                for (a = 0; a < arrays.Length; a++)
                {
                    l = arrays[a].Length;
                    if (l != 0)
                    {
                        Array.Copy(arrays[a], 0, ret, tot, l);
                        tot += l;
                    }
                }
            }
            return ret;
        }

        #endregion
        
        #region Fill Generico

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <typeparam name="T">Tipo (estructura).</typeparam>
        public static void Fill<T>(T[] arr, T fill) where T : struct
        {
            if (fill.Equals(default(T)))
                Array.Clear(arr, 0, arr.Length);
            else
                for (int a = 0; a < arr.Length; a++)
                    arr[a] = fill;
        }

        /// <summary>
        /// Rellena un array con el elemento especificado.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="fill">Relleno.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo (estructura).</typeparam>
        public static void Fill<T>(T[] arr, int i, int l, T fill) where T : struct
        {
            if (fill.Equals(default(T)))
                Array.Clear(arr, i, l);
            else
            {

                l += i;
                for (int a = i; a < l; a++)
                    arr[i] = fill;
            }
        }
        
        #endregion


        #region Strings Generico

        /// <summary>
        /// Genera un array con las representaciones de cadena de cada elemento.
        /// </summary>
        /// <typeparam name="T">Tipo.</typeparam>
        /// <param name="arr">
        /// Array a convertir.
        /// </param>
        /// <returns>
        /// Array de cadenas.
        /// En los tipos de referencia, los elementos nulos son representados como cadenas vacias.
        /// </returns>
        public static string[] Strings<T>(T[] arr)
        {
            string[] ret = new string[arr.Length];
            int a = 0;
            if (default(T) != null)
            {
                while (a < arr.Length)
                    ret[a] = arr[a++].ToString();
            }
            else
            {
                for(;a<arr.Length;a++)
                {
                    ret[a] = arr[a] == null ? Others.Empty : arr[a].ToString();
                }
            }
            return ret;
        }

        /// <summary>
        /// Genera un array con las representaciones de cadena de cada elemento.
        /// </summary>
        /// <typeparam name="T">Tipo (IFormattable).</typeparam>
        /// <param name="arr">
        /// Array a convertir.
        /// </param>
        /// <param name="f">Cadena de formato. Null para usar el formato por defecto.</param>
        /// <param name="p">Proveedor de formato.</param>
        /// <returns>
        /// Array de cadenas.
        /// En los tipos de referencia, los elementos nulos son representados como cadenas vacias.
        /// </returns>
        public static string[] Strings<T>(T[] arr, string f, IFormatProvider p) where T : IFormattable
        {
            string[] ret = new string[arr.Length];
            int a = 0;
            if (default(T) != null)
            {
                while (a < arr.Length)
                    ret[a] = arr[a++].ToString(f, p);
            }
            else
            {
                for(;a<arr.Length;a++)
                {
                    ret[a] = arr[a] == null ? Others.Empty : arr[a].ToString(f, p);
                }
            }
            return ret;
        }

        #endregion

        

    }
}
