using System;
using System.Collections.Generic;
using EnPalermo.Lit;
using EnPalermo.Mem;
using EnPalermo.Uti;

namespace EnPalermo.Arr
{
    /// <summary>
    /// Funciones estticas relativas al ordenamiento de arrays unidimensionales.
    /// </summary>
    public static class ArrSort
    {
        #region Orden Aleatoreo Primitivos

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(byte[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(byte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (byte* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(sbyte[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(sbyte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (sbyte* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(char[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(char[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (char* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(short[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(short[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (short* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }


        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(ushort[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(ushort[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (ushort* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(int[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(int[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (int* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(uint[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(uint[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (uint* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(long[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(long[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (long* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(ulong[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(ulong[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (ulong* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(PByte[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(PByte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (PByte* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Randomize(PChar[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(PChar[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (PChar* p = arr)
                        Sort.Randomize(p + i, l);
                }
        }

        #endregion

        #region Orden Inverso Primitivos

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(byte[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(byte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (byte* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(sbyte[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(sbyte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (sbyte* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(char[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(char[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (char* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(short[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(short[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (short* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }


        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(ushort[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(ushort[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (ushort* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(int[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(int[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (int* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(uint[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(uint[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (uint* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(long[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(long[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (long* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(ulong[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(ulong[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (ulong* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(PByte[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(PByte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (PByte* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Reverse(PChar[] arr)
        {
            Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Invierte el orden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(PChar[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (PChar* p = arr)
                        Sort.Reverse(p + i, l);
                }
        }

        #endregion

        #region Orden Ascendente Primitivos

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(byte[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(byte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (byte* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(sbyte[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(sbyte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (sbyte* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(char[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(char[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (char* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(short[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(short[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (short* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }


        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(ushort[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(ushort[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (ushort* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(int[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(int[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (int* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(uint[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(uint[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (uint* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(long[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(long[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (long* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(ulong[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(ulong[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (ulong* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(PByte[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(PByte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (PByte* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Ascending(PChar[] arr)
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(PChar[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (PChar* p = arr)
                        Sort.Ascending(p + i, l);
                }
        }


        #endregion

        #region Orden Descendente Primitivos

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(byte[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(byte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (byte* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(sbyte[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(sbyte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (sbyte* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(char[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(char[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (char* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(short[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(short[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (short* p = arr)
                        Sort.Descending(p + i, l);
                }
        }


        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(ushort[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(ushort[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (ushort* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(int[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(int[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (int* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(uint[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(uint[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (uint* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(long[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(long[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (long* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(ulong[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(ulong[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (ulong* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(PByte[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(PByte[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (PByte* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Descending(PChar[] arr)
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Orden QuickSort descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(PChar[] arr, int i, int l)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                unsafe
                {
                    fixed (PChar* p = arr)
                        Sort.Descending(p + i, l);
                }
        }

        #endregion

        #region Orden Aleatoreo Generico

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Randomize<T>(T[] arr)
        {
            Randomize(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Randomize<T>(T[] arr, int i, int l)
        {
            int fin = i + l;
            if (i < 0 || fin > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                int j;
                // tipo.
                T temp;
                if (i == 0)
                {
                    do
                    {
                        j = Rand.NextMinor(l);
                        if (j != i)
                        {
                            temp = arr[i];
                            arr[i] = arr[j];
                            arr[j] = temp;
                        }
                    } while (++i != fin);
                }
                else
                {
                    int sum = i;
                    do
                    {
                        j = Rand.NextMinor(l) + sum;
                        if (j != i)
                        {
                            temp = arr[i];
                            arr[i] = arr[j];
                            arr[j] = temp;
                        }
                    } while (++i != fin);
                }
            }
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenarn de acuerdo a las claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Randomize<T, U>(T[] keys, U[] items)
        {
            Randomize(keys, items, 0, keys.Length);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Randomize<T, U>(T[] keys, U[] items, int i, int l)
        {
            int fin = i + l;
            if (i < 0 || fin > keys.Length || fin > items.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                // tipo.
                int j;
                T temp;
                U temp2;
                if (i == 0)
                {
                    do
                    {
                        j = Rand.NextMinor(l);
                        if (j != i)
                        {
                            temp = keys[i];
                            keys[i] = keys[j];
                            keys[j] = temp;
                            temp2 = items[i];
                            items[i] = items[j];
                            items[j] = temp2;
                        }
                    } while (++i != fin);
                }
                else
                {
                    int sum = i;
                    do
                    {
                        j = Rand.NextMinor(l) + sum;
                        if (j != i)
                        {
                            temp = keys[i];
                            keys[i] = keys[j];
                            keys[j] = temp;
                            temp2 = items[i];
                            items[i] = items[j];
                            items[j] = temp2;
                        }
                    } while (++i != fin);
                }
            }
        }

        #endregion

        #region Orden Inverso Generico

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Reverse<T>(T[] arr)
        {
            Array.Reverse(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="i">Inicio.</param>
        /// <param name="arr">Array.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Reverse<T>(T[] arr, int i, int l)
        {
            Array.Reverse(arr, i, l);
        }

        #endregion

        #region Orden Ascendente Generico Privados

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        private static void ascending<T>(T[] arr, int inf, int sup) where T : IComparable<T>
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = arr[(i + j) >> 1], temp;
                do
                {
                    while (x.CompareTo(arr[i]) == 1)
                        i++;
                    while (x.CompareTo(arr[j]) == -1)
                        j--;
                    if (i <= j)
                    {
                        temp = arr[i];
                        arr[i] = arr[j];
                        arr[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(arr, inf, j);
                ascending(arr, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (para objetos).
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        private static void ascendingO<T>(T[] arr, int inf, int sup) where T : IComparable<T>
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = arr[(i + j) >> 1], temp;
                do
                {
                    if (!Object.ReferenceEquals(x, null))
                    {
                        while (x.CompareTo(arr[i]) == 1)
                            i++;
                        while (x.CompareTo(arr[j]) == -1)
                            j--;
                    }
                    else
                    {
                        while (!Object.ReferenceEquals(arr[j], null))
                            j--;
                    }
                    if (i <= j)
                    {
                        temp = arr[i];
                        arr[i] = arr[j];
                        arr[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascendingO(arr, inf, j);
                ascendingO(arr, i, sup);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <param name="comp">Comparador.</param>
        /// <param name="arr">Array.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        private static void ascending<T>(T[] arr, int inf, int sup, Comparetor<T> comp)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = arr[(i + j) >> 1], temp;
                do
                {
                    while (comp(arr[i], x) == -1)
                        i++;
                    while (comp(arr[j], x) == 1)
                        j--;
                    if (i <= j)
                    {
                        temp = arr[i];
                        arr[i] = arr[j];
                        arr[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(arr, inf, j, comp);
                ascending(arr, i, sup, comp);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave (IComparable).</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        private static void ascending<T, U>(T[] keys, U[] items, int inf, int sup) where T : IComparable<T>
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = keys[(i + j) >> 1], temp;
                U temp2;
                do
                {
                    while (x.CompareTo(keys[i]) == 1)
                        i++;
                    while (x.CompareTo(keys[j]) == -1)
                        j--;
                    if (i <= j)
                    {
                        temp = keys[i];
                        keys[i] = keys[j];
                        keys[j] = temp;
                        temp2 = items[i];
                        items[i] = items[j];
                        items[j] = temp2;

                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(keys, items, inf, j);
                ascending(keys, items, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (para objetos).
        /// </summary>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave (IComparable).</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        private static void ascendingO<T, U>(T[] keys, U[] items, int inf, int sup) where T : IComparable<T>
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = keys[(i + j) >> 1], temp;
                U temp2;
                do
                {
                    if (!Object.ReferenceEquals(x, null))
                    {
                        while (x.CompareTo(keys[i]) == 1)
                            i++;
                        while (x.CompareTo(keys[j]) == -1)
                            j--;
                    }
                    else
                    {

                        while (!Object.ReferenceEquals(keys[j], null))
                            j--;
                    }


                    if (i <= j)
                    {
                        temp = keys[i];
                        keys[i] = keys[j];
                        keys[j] = temp;
                        temp2 = items[i];
                        items[i] = items[j];
                        items[j] = temp2;

                        i++;
                        j--;
                    }
                } while (i <= j);
                ascendingO(keys, items, inf, j);
                ascendingO(keys, items, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <param name="comp">Comparador.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        private static void ascending<T, U>(T[] keys, U[] items, int inf, int sup, Comparetor<T> comp)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = keys[(i + j) >> 1], temp;
                U temp2;

                do
                {
                    while (comp(keys[i], x) == -1)
                        i++;
                    while (comp(keys[j], x) == 1)
                        j--;
                    if (i <= j)
                    {
                        temp = keys[i];
                        keys[i] = keys[j];
                        keys[j] = temp;
                        temp2 = items[i];
                        items[i] = items[j];
                        items[j] = temp2;

                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(keys, items, inf, j, comp);
                ascending(keys, items, i, sup, comp);
            }
        }

        #endregion

        #region Orden Ascendente Generico Publicos

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        public static void Ascending<T>(T[] arr) where T : IComparable<T>
        {
            Ascending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        public static void Ascending<T>(T[] arr, int i, int l) where T : IComparable<T>
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                if (default(T) == null)
                    ascendingO(arr, i, l - 1);
                else
                    ascending(arr, i, l - 1);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Ascending<T>(T[] arr, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            Ascending(arr, 0, arr.Length, comp.Compare);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Ascending<T>(T[] arr, int i, int l, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            Ascending(arr, i, l, comp.Compare);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Ascending<T>(T[] arr, Comparetor<T> comp)
        {
            Ascending(arr, 0, arr.Length, comp);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Ascending<T>(T[] arr, int i, int l, Comparetor<T> comp)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                ascending(arr, i, l - 1, comp);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="sc">Modo de comparacin.</param>
        public static void Ascending(string[] arr, StringComparison sc)
        {
            Ascending(arr, 0, arr.Length, sc);
        }



        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="sc">Modo de comparacin.</param>
        public static void Ascending(string[] arr, int i, int l, StringComparison sc)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                Comparetor<string> comp;
                switch (sc)
                {
                    case StringComparison.CurrentCulture:
                        comp = StringComparer.CurrentCulture.Compare;
                        break;
                    case StringComparison.CurrentCultureIgnoreCase:
                        comp = StringComparer.CurrentCultureIgnoreCase.Compare;
                        break;
                    case StringComparison.InvariantCulture:
                        comp = StringComparer.InvariantCulture.Compare;
                        break;
                    case StringComparison.InvariantCultureIgnoreCase:
                        comp = StringComparer.InvariantCultureIgnoreCase.Compare;
                        break;
                    case StringComparison.OrdinalIgnoreCase:
                        comp = StringComparer.OrdinalIgnoreCase.Compare;
                        break;
                    default:
                        comp = StringComparer.Ordinal.Compare;
                        break;
                }
                ascending(arr, i, l - 1, comp);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave (IComparable).</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Ascending<T, U>(T[] keys, U[] items) where T : IComparable<T>
        {
            Ascending(keys, items, 0, keys.Length);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave (IComparable).</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Ascending<T, U>(T[] keys, U[] items, int i, int l) where T : IComparable<T>
        {
            int fin = i + l;
            if (i < 0 || fin > keys.Length || fin > items.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                if (default(T) == null)
                    ascendingO(keys, items, i, l - 1);
                else
                    ascending(keys, items, i, l - 1);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Ascending<T, U>(T[] keys, U[] items, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            ascending(keys, items, 0, keys.Length, comp.Compare);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Ascending<T, U>(T[] keys, U[] items, int i, int l, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            ascending(keys, items, i, l, comp.Compare);
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Ascending<T, U>(T[] keys, U[] items, Comparetor<T> comp)
        {
            Ascending(keys, items, 0, keys.Length, comp);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Ascending<T, U>(T[] keys, U[] items, int i, int l, Comparetor<T> comp)
        {
            int fin = i + l;
            if (i < 0 || fin > keys.Length || fin > items.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                ascending(keys, items, i, l - 1, comp);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="sc">Modo de comparacin.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Ascending<U>(string[] keys, U[] items, StringComparison sc)
        {
            Ascending(keys, items, 0, keys.Length, sc);
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="sc">Modo de comparacin.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Ascending<U>(string[] keys, U[] items, int i, int l, StringComparison sc)
        {
            int fin = i + l;
            if (i < 0 || fin > keys.Length || fin > items.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                Comparetor<string> comp;
                switch (sc)
                {
                    case StringComparison.CurrentCulture:
                        comp = StringComparer.CurrentCulture.Compare;
                        break;
                    case StringComparison.CurrentCultureIgnoreCase:
                        comp = StringComparer.CurrentCultureIgnoreCase.Compare;
                        break;
                    case StringComparison.InvariantCulture:
                        comp = StringComparer.InvariantCulture.Compare;
                        break;
                    case StringComparison.InvariantCultureIgnoreCase:
                        comp = StringComparer.InvariantCultureIgnoreCase.Compare;
                        break;
                    case StringComparison.OrdinalIgnoreCase:
                        comp = StringComparer.OrdinalIgnoreCase.Compare;
                        break;
                    default:
                        comp = StringComparer.Ordinal.Compare;
                        break;
                }
                ascending(keys, items, i, l - 1, comp);
            }
        }

        #endregion

        #region Orden Descendente Generico Privados

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        private static void descending<T>(T[] arr, int inf, int sup) where T : IComparable<T>
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = arr[(i + j) >> 1], temp;
                do
                {
                    while (x.CompareTo(arr[i]) == -1)
                        i++;
                    while (x.CompareTo(arr[j]) == 1)
                        j--;
                    if (i <= j)
                    {
                        temp = arr[i];
                        arr[i] = arr[j];
                        arr[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(arr, inf, j);
                descending(arr, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (para objetos).
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        private static void descendingO<T>(T[] arr, int inf, int sup) where T : IComparable<T>
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = arr[(i + j) >> 1], temp;
                do
                {
                    if (!Object.ReferenceEquals(x, null))
                    {
                        while (x.CompareTo(arr[i]) == -1)
                            i++;
                        while (x.CompareTo(arr[j]) == 1)
                            j--;
                    }
                    else
                    {
                        while (!Object.ReferenceEquals(arr[j], null))
                            j--;
                    }
                    if (i <= j)
                    {
                        temp = arr[i];
                        arr[i] = arr[j];
                        arr[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descendingO(arr, inf, j);
                descendingO(arr, i, sup);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <param name="comp">Comparador.</param>
        /// <param name="arr">Array.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        private static void descending<T>(T[] arr, int inf, int sup, Comparetor<T> comp)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = arr[(i + j) >> 1], temp;
                do
                {
                    while (comp(arr[i], x) == 1)
                        i++;
                    while (comp(arr[j], x) == -1)
                        j--;
                    if (i <= j)
                    {
                        temp = arr[i];
                        arr[i] = arr[j];
                        arr[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(arr, inf, j, comp);
                descending(arr, i, sup, comp);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave (IComparable).</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        private static void descending<T, U>(T[] keys, U[] items, int inf, int sup) where T : IComparable<T>
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = keys[(i + j) >> 1], temp;
                U temp2;
                do
                {
                    while (x.CompareTo(keys[i]) == -1)
                        i++;
                    while (x.CompareTo(keys[j]) == 1)
                        j--;
                    if (i <= j)
                    {
                        temp = keys[i];
                        keys[i] = keys[j];
                        keys[j] = temp;
                        temp2 = items[i];
                        items[i] = items[j];
                        items[j] = temp2;

                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(keys, items, inf, j);
                descending(keys, items, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (para objetos).
        /// </summary>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave (IComparable).</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        private static void descendingO<T, U>(T[] keys, U[] items, int inf, int sup) where T : IComparable<T>
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = keys[(i + j) >> 1], temp;
                U temp2;
                do
                {
                    if (!Object.ReferenceEquals(x, null))
                    {
                        while (x.CompareTo(keys[i]) == -1)
                            i++;
                        while (x.CompareTo(keys[j]) == 1)
                            j--;
                    }
                    else
                    {

                        while (!Object.ReferenceEquals(keys[j], null))
                            j--;
                    }


                    if (i <= j)
                    {
                        temp = keys[i];
                        keys[i] = keys[j];
                        keys[j] = temp;
                        temp2 = items[i];
                        items[i] = items[j];
                        items[j] = temp2;

                        i++;
                        j--;
                    }
                } while (i <= j);
                descendingO(keys, items, inf, j);
                descendingO(keys, items, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        /// <param name="comp">Comparador.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        private static void descending<T, U>(T[] keys, U[] items, int inf, int sup, Comparetor<T> comp)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                T x = keys[(i + j) >> 1], temp;
                U temp2;

                do
                {
                    while (comp(keys[i], x) == 1)
                        i++;
                    while (comp(keys[j], x) == -1)
                        j--;
                    if (i <= j)
                    {
                        temp = keys[i];
                        keys[i] = keys[j];
                        keys[j] = temp;
                        temp2 = items[i];
                        items[i] = items[j];
                        items[j] = temp2;

                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(keys, items, inf, j, comp);
                descending(keys, items, i, sup, comp);
            }
        }

        #endregion

        #region Orden Descendente Generico Publicos

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        public static void Descending<T>(T[] arr) where T : IComparable<T>
        {
            Descending(arr, 0, arr.Length);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo (IComparable).</typeparam>
        public static void Descending<T>(T[] arr, int i, int l) where T : IComparable<T>
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                if (default(T) == null)
                    descendingO(arr, i, l - 1);
                else
                    descending(arr, i, l - 1);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Descending<T>(T[] arr, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            Descending(arr, 0, arr.Length, comp.Compare);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Descending<T>(T[] arr, int i, int l, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            Descending(arr, i, l, comp.Compare);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Descending<T>(T[] arr, Comparetor<T> comp)
        {
            Descending(arr, 0, arr.Length, comp);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <typeparam name="T">Tipo.</typeparam>
        public static void Descending<T>(T[] arr, int i, int l, Comparetor<T> comp)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                descending(arr, i, l - 1, comp);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="sc">Modo de comparacin.</param>
        public static void Descending(string[] arr, StringComparison sc)
        {
            Descending(arr, 0, arr.Length, sc);
        }



        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="sc">Modo de comparacin.</param>
        public static void Descending(string[] arr, int i, int l, StringComparison sc)
        {
            if (i < 0 || i + l > arr.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                Comparetor<string> comp;
                switch (sc)
                {
                    case StringComparison.CurrentCulture:
                        comp = StringComparer.CurrentCulture.Compare;
                        break;
                    case StringComparison.CurrentCultureIgnoreCase:
                        comp = StringComparer.CurrentCultureIgnoreCase.Compare;
                        break;
                    case StringComparison.InvariantCulture:
                        comp = StringComparer.InvariantCulture.Compare;
                        break;
                    case StringComparison.InvariantCultureIgnoreCase:
                        comp = StringComparer.InvariantCultureIgnoreCase.Compare;
                        break;
                    case StringComparison.OrdinalIgnoreCase:
                        comp = StringComparer.OrdinalIgnoreCase.Compare;
                        break;
                    default:
                        comp = StringComparer.Ordinal.Compare;
                        break;
                }
                descending(arr, i, l - 1, comp);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave (IComparable).</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Descending<T, U>(T[] keys, U[] items) where T : IComparable<T>
        {
            Descending(keys, items, 0, keys.Length);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave (IComparable).</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Descending<T, U>(T[] keys, U[] items, int i, int l) where T : IComparable<T>
        {
            int fin = i + l;
            if (i < 0 || fin > keys.Length || fin > items.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                if (default(T) == null)
                    descendingO(keys, items, i, l - 1);
                else
                    descending(keys, items, i, l - 1);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Descending<T, U>(T[] keys, U[] items, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            descending(keys, items, 0, keys.Length, comp.Compare);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Descending<T, U>(T[] keys, U[] items, int i, int l, IComparer<T> comp)
        {
            if (comp == null)
                comp = Comparer<T>.Default;
            descending(keys, items, i, l, comp.Compare);
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Descending<T, U>(T[] keys, U[] items, Comparetor<T> comp)
        {
            Descending(keys, items, 0, keys.Length, comp);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="comp">Comparador (null para obtener el comparador por defecto).</param>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="T">Tipo de clave.</typeparam>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Descending<T, U>(T[] keys, U[] items, int i, int l, Comparetor<T> comp)
        {
            int fin = i + l;
            if (i < 0 || fin > keys.Length || fin > items.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
                descending(keys, items, i, l - 1, comp);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="sc">Modo de comparacin.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Descending<U>(string[] keys, U[] items, StringComparison sc)
        {
            Descending(keys, items, 0, keys.Length, sc);
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="i">Inicio.</param>
        /// <param name="l">Largo.</param>
        /// <param name="sc">Modo de comparacin.</param>
        /// <param name="keys">Claves.</param>
        /// <param name="items">
        /// Elementos que se ordenaran de acuerdo a las claves.
        /// Debe ser del mismo largo o mayor que claves.
        /// </param>
        /// <typeparam name="U">Tipo de elemento.</typeparam>
        public static void Descending<U>(string[] keys, U[] items, int i, int l, StringComparison sc)
        {
            int fin = i + l;
            if (i < 0 || fin > keys.Length || fin > items.Length)
                throw new ArgumentOutOfRangeException("i,l", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l > 1)
            {
                Comparetor<string> comp;
                switch (sc)
                {
                    case StringComparison.CurrentCulture:
                        comp = StringComparer.CurrentCulture.Compare;
                        break;
                    case StringComparison.CurrentCultureIgnoreCase:
                        comp = StringComparer.CurrentCultureIgnoreCase.Compare;
                        break;
                    case StringComparison.InvariantCulture:
                        comp = StringComparer.InvariantCulture.Compare;
                        break;
                    case StringComparison.InvariantCultureIgnoreCase:
                        comp = StringComparer.InvariantCultureIgnoreCase.Compare;
                        break;
                    case StringComparison.OrdinalIgnoreCase:
                        comp = StringComparer.OrdinalIgnoreCase.Compare;
                        break;
                    default:
                        comp = StringComparer.Ordinal.Compare;
                        break;
                }
                descending(keys, items, i, l - 1, comp);
            }
        }

        #endregion
    }
}
