using System;
using System.Runtime.InteropServices;
using EnPalermo.Txt;
using EnPalermo.Lit;

namespace EnPalermo.Mem
{
	/// <summary>
	/// Representa una zona de memoria basada en un puntero Char.
	/// Tambin provee un enumerador simple que se autoconsume.
	/// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public unsafe struct PChar:IComparable<PChar>,IEquatable<PChar>
	{

		#region Campos

		/// <summary>
		/// Puntero.
		/// </summary>
		public char* Pointer;

		/// <summary>
		/// Largo.
		/// </summary>
		public int Length;

        /// <summary>
        /// PChar cero.
        /// </summary>
        public static readonly PChar Zero=new PChar();

		#endregion

		#region Constructores

		/// <summary>
		/// Inicializa la estructura.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		public PChar(char * p, int l)
		{
			Pointer=p;Length=l;
		}
        
		/// <summary>
		/// Inicializa la estructura en base a otro PChar.
		/// </summary>
		/// <param name="pChar">PChar.</param>
		/// <param name="i">Inicio.</param>
		/// <param name="l">Largo.</param>
		public PChar(PChar pChar,int i, int l)
		{
			Pointer=pChar.Pointer+i;Length=l;
		}

        /// <summary>
        /// Remueve comillas y efectua Trim con el filtro de todos los carcteres en blanco a un puntero de cadena.
        /// </summary>
        /// <param name="p">Puntero de cadena.</param>
        /// <param name="l">Largo del puntero.</param>
        /// <param name="afterTrimR">Luego del procesamiento de comillas, efecta trim derecho.</param>
        /// <param name="afterTrimL">Luego del procesamiento de comillas, efecta trim izquierdo.</param>
        /// <param name="mode">Opciones de eliminacin de comillas.</param>
        public PChar(char * p, int l,RemoveQuotesMode mode, bool afterTrimL, bool afterTrimR)
        {
            start:
            if (l<2)
            {
                if (l==1 && (afterTrimL || afterTrimR))
                {
                    switch(*p)
                    {
                        case ' ':
                        case '\t':
                        case '\n':
                        case '\r':
                        case '\xa0':
                        case '\x85':
                        case '\0':
                        case '\v':
                        case '\f':
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            l=0;
                            break;
                    }
                }
                Pointer=p;
                Length=l;
                return;
            }
            PChar ret;            
            switch(mode)
            {
                case RemoveQuotesMode.RemoveOne:
                case RemoveQuotesMode.RemoveAll:
                    ret=new PChar(p,l,true,true);
                    if (ret.Length<2)
                        goto end;
                    break;
                case RemoveQuotesMode.RemoveOneNoTrim:
                case RemoveQuotesMode.RemoveAllNoTrim:
                    ret=new PChar(p,l);
                    break;
                default:
                    if (afterTrimL || afterTrimR)
                    {
                        ret=new PChar(p,l,afterTrimL,afterTrimR);
                        goto end;
                    }
                    Pointer=p;
                    Length=l;
                    return;
            }
            switch(*ret.Pointer)
            {
                case '"':
                    if (ret.Pointer[ret.Length-1]=='"') 
                        break;
                    goto default;
                case '\'':
                    if (ret.Pointer[ret.Length-1]=='\'') 
                        break;
                    goto default;

                case '\xab':
                    if (ret.Pointer[ret.Length-1]=='\xbb') 
                        break;
                    goto default;
                case '\u2018':
                    if (ret.Pointer[ret.Length-1]=='\u2019')  
                        break;
                    goto default;
                case '\u201c':
                    if (ret.Pointer[ret.Length-1]=='\u201d') 
                        break;
                    goto default;
                case '\u2039':
                    if (ret.Pointer[ret.Length-1]=='\u203a') 
                        break;
                    goto default;
                default:
                    if ((afterTrimL || afterTrimR) && ((int)mode&4)==0)
                        ret=new PChar(ret.Pointer,ret.Length,afterTrimL,afterTrimR);
                    goto end;
            }
            if (((int)mode&2)!=0)
            {
                p=ret.Pointer+1;
                l=ret.Length-2;
                goto start;
            }
            ret.Pointer++;
            ret.Length-=2;
            if (ret.Length!=0 && (afterTrimL || afterTrimR))
                ret=new PChar(ret.Pointer,ret.Length,afterTrimL,afterTrimR);
           
            end:
            Pointer=ret.Pointer;
            Length=ret.Length;
        }

		/// <summary>
		/// Efectua Trim con el filtro de todos los carcteres en blanco a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero de cadena.</param>
		/// <param name="l">Largo del puntero.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		public PChar(char * p, int l,bool left, bool right)
		{
            if (l>0)
            {
                if (left)
                {
                reswitch1:
                    switch(*p)
                    {
                        case ' ':
                        case '\t':
                        case '\n':
                        case '\r':
                        case '\xa0':
                        case '\x85':
                        case '\0':
                        case '\v':
                        case '\f':
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            p++;
                            if (--l!=0)
                                goto reswitch1;
                            goto end;
                    }
                    if (l==1)
                        goto end;
                }
                if (right)
                {
                reswitch2:
                    switch(p[l-1])
                    {
                        case ' ':
                        case '\t':
                        case '\n':
                        case '\r':
                        case '\xa0':
                        case '\x85':
                        case '\0':
                        case '\v':
                        case '\f':
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            if (--l!=0)
                                goto reswitch2;
                            break;
                    }                
                }
            }
            end:
			Pointer=p;
			Length=l;
		}


		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero de cadena.</param>
		/// <param name="l">Largo del puntero.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
		/// <param name="crlf">Incluye en filtro \r y \n.</param>
		/// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
		/// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
		/// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
		/// <param name="tab">Incluye en filtro el taburador (\t).</param>
		public PChar(char * p, int l,bool left, bool right, bool sp, bool lat1, bool uni,bool tab, bool crlf, bool control)
		{
            if (l>0)
            {
                if (left)
                {
                reswitch1:
                    switch(*p)
                    {
                        case ' ':
                            if (sp)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }                               
                            break;
                        case '\t':
                            if (tab)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }                               
                            break;
                        case '\n':
                        case '\r':
                            if (crlf)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }                               
                            break;
                        case '\xa0':
                        case '\x85':
                            if (lat1)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }                               
                            break;
                        case '\0':
                        case '\v':
                        case '\f':
                            if (control)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }                               
                            break;
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            if (uni)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }                               
                            break;
                    }
                    if (l==1)
                        goto end;
                }
                if (right)
                {
                reswitch2:
                    switch(p[l-1])
                    {
                        case ' ':
                            if (!sp || --l==0)
                                break;
                            goto reswitch2;                            
                        case '\t':
                            if (!tab || --l==0)
                                break;
                            goto reswitch2;                            
                        case '\n':
                        case '\r':
                            if (!crlf || --l==0)
                                break;
                            goto reswitch2;                            
                        case '\xa0':
                        case '\x85':
                            if (!lat1 || --l==0)
                                break;
                            goto reswitch2;                            
                        case '\0':
                        case '\v':
                        case '\f':
                            if (!control || --l==0)
                                break;
                            goto reswitch2;                            
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            if (!uni || --l==0)
                                break;
                            goto reswitch2;                            
                    }                
                }
            }
            end:
            Pointer=p;
            Length=l;
		}



		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero de cadena.</param>
		/// <param name="l">Largo del puntero.</param>
		/// <param name="pTrim">Puntero de carcteres de filtro.</param>
		/// <param name="lTrim">Largo del puntero de carcteres de filtro.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		public PChar (char * p, int l,char * pTrim, int lTrim, bool left, bool right)
		{
			if(l>0)
			{
				if(left)
				{
					int a=0;
					while(Comp.Exist(pTrim,lTrim,p[a]) && ++a!=l);
					if (a==l) l=0;
					else
					{
						p+=a;
						l-=a;
						if(right && l!=1)
						{
							while(Comp.Exist(pTrim,lTrim,p[--l]) && l!=0);
							l++;
						}
					}
				}
				else if (right)
				{
					while(Comp.Exist(pTrim,lTrim,p[--l]) && l!=-1);
					l++;
				}
			}
			Pointer=p;
			Length=l;
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero de cadena.</param>
		/// <param name="l">Largo del puntero.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="cTrim">Carcteres para Trim.</param>
		public PChar (char * p, int l,bool left, bool right, char[] cTrim)
		{
			int ltrim=cTrim.Length;
			if (ltrim!=0)
			{
				fixed(char* ptrim=cTrim)
				{
					PChar pc=new PChar(p,l,ptrim,ltrim,left,right);
					Pointer=pc.Pointer;
					Length=pc.Length;
				} 
			}
			else
			{
				Pointer=p;
				Length=l;
			}
		}



        /// <summary>
        /// Efectua Trim a un puntero de cadena.
        /// </summary>
        /// <param name="p">Puntero de cadena.</param>
        /// <param name="l">Largo del puntero.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        public PChar (char * p, int l, CharGroup group,char * pch, int lch, bool left, bool right)
        {
            if(l>0)
            {
                if (group==CharGroup.AllBlanks)
                {
                    PChar pc=new PChar(p,l,left,right);
                    Pointer=pc.Pointer;
                    Length=pc.Length;
                    return;
                }
                if (group==CharGroup.Custom)
                {
                    PChar pc=new PChar(p,l,pch,lch,left,right);
                    Pointer=pc.Pointer;
                    Length=pc.Length;
                    return;
                }

                if(left)
                {
                    int a=0;
                    while(Texts.InGroup(p[a],group,pch,lch) && ++a!=l);
                    if (a==l) l=0;
                    else
                    {
                        p+=a;
                        l-=a;
                        if(right && l!=1)
                        {
                            while(Texts.InGroup(p[--l],group,pch,lch) && l!=0);
                            l++;
                        }
                    }
                }
                else if (right)
                {
                    while(Texts.InGroup(p[--l],group,pch,lch) && l!=-1);
                    l++;
                }
            }
            Pointer=p;
            Length=l;
        }

        /// <summary>
        /// Efectua Trim a un puntero de cadena.
        /// </summary>
        /// <param name="p">Puntero de cadena.</param>
        /// <param name="l">Largo del puntero.</param>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="custom">Array de carcteres en categora custom.</param>
        public PChar (char * p, int l,bool left, bool right, CharGroup group, char[] custom)
        {
            int lch=custom==null?0:custom.Length;
            PChar pc;
            if (lch!=0)
                fixed(char* pch=custom)
                    pc=new PChar(p,l,group,pch,lch,left,right);
            else
                pc=new PChar(p,l,group,null,0,left,right);
            Pointer=pc.Pointer;
            Length=pc.Length;
        }




		#endregion

		#region Enumeradores

		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// actual y proximo separador encontrado.
		/// </summary>
		/// <param name="c">Separador.</param>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PChar Foward(char c,bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			if (ignoreEmpty)
			{
				while(Pointer[a]==c && ++a!=Length);
				if(a==Length)
				{
					Pointer=null;
					Length=0;
					return this;
				}
				if (a!=0)
				{
					Pointer+=a;
					Length-=a;
				}
				if (Length==1)
				{
					ret=this;
					Pointer=null;
					Length=0;
					return ret;
				}
				a=1;
			}
			while(Pointer[a]!=c && ++a!=Length);

			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// del char siguiente al ltimo separador del bloque y el final del bloque.
		/// </summary>
		/// <param name="c">Separador.</param>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PChar Backward(char c, bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			if (ignoreEmpty)
			{
				while(--a!=-1 && Pointer[a]==c);
				if(a<1)
				{
					if (a==-1)
					{
						Pointer=null;
						Length=0;
						return this;
					}
					ret=new PChar(Pointer,1);
					Pointer=null;
					Length=0;
					return ret;
				}
				Length=a+1;
			}
			while(--a!=-1 && Pointer[a]!=c);
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// actual y proximo separador encontrado.
		/// </summary>
		/// <param name="p">Puntero de separadores.</param>
		/// <param name="l">Largo del puntero de separadores.</param>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PChar Foward(char * p, int l, bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			if (ignoreEmpty)
			{
				while(Comp.Exist(p,l,Pointer[a]) && ++a!=Length);
				if(a==Length)
				{
					Pointer=null;
					Length=0;
					return this;
				}
				if (a!=0)
				{
					Pointer+=a;
					Length-=a;
				}
				if (Length==1)
				{
					ret=this;
					Pointer=null;
					Length=0;
					return ret;
				}
				a=1;
			}
			while(!Comp.Exist(p,l,Pointer[a]) && ++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// del char siguiente al ltimo separador del bloque y el final del bloque.
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <param name="p">Puntero de separadores.</param>
		/// <param name="l">Largo del puntero de separadores.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PChar Backward(char * p, int l, bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			if (ignoreEmpty)
			{
				while(--a!=-1 && Comp.Exist(p,l,Pointer[a]));
				if(a<1)
				{
					if (a==-1)
					{
						Pointer=null;
						Length=0;
						return this;
					}
					ret=new PChar(Pointer,1);
					Pointer=null;
					Length=0;
					return ret;
				}
				Length=a+1;
			}
			while(--a!=-1 && !Comp.Exist(p,l,Pointer[a]));
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}

        /// <summary>
        /// Enumerador que devuelve el PChar del bloque entre la posicin
        /// actual y proximo separador encontrado.
        /// </summary>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
        /// <returns>
        /// - Si el separador es detectado:
        ///   PChar con Length==Ubicacin separador.
        ///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
        /// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
        ///   PChar actual. El puntero se setea en null y Length se setea en 0.
        /// - Si el puntero es nulo:
        ///   PChar con puntero nulo.
        ///   El puntero nulo permite indicar que el enumerador se ha consumido.
        /// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
        ///   siendo el ltimo retorno de largo 0 y puntero null.
        /// </returns>
        public PChar Foward(CharGroup group, char * pch, int lch, bool ignoreEmpty)
        {
            if(Pointer==null) return this;
            PChar ret;
            if(Length<1) 
            {
                ret=this;
                Pointer=null;
                Length=0;
                return ret;
            }
            int a=0;
            if (ignoreEmpty)
            {
                while(Texts.InGroup(Pointer[a],group,pch,lch) && ++a!=Length);
                if(a==Length)
                {
                    Pointer=null;
                    Length=0;
                    return this;
                }
                if (a!=0)
                {
                    Pointer+=a;
                    Length-=a;
                }
                if (Length==1)
                {
                    ret=this;
                    Pointer=null;
                    Length=0;
                    return ret;
                }
                a=1;
            }
            while(!Texts.InGroup(Pointer[a],group,pch,lch) && ++a!=Length);
            if(a==Length)
            {
                ret=this;
                Pointer=null;
                Length=0;
                return ret;
            }
            ret=new PChar(Pointer,a);
            Pointer+=++a;
            Length-=a;
            return ret;
        }


        /// <summary>
        /// Enumerador que devuelve el PChar del bloque entre la posicin
        /// del char siguiente al ltimo separador del bloque y el final del bloque.
        /// </summary>
        /// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <returns>
        /// - Si el separador es detectado:
        ///   PChar con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
        ///   El puntero no se modifica y el Length se setea en Ubicacin separador.
        /// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
        ///   PChar actual. El puntero se setea en null y Length se setea en 0.
        /// - Si el puntero es nulo:
        ///   PChar con puntero nulo.
        ///   El puntero nulo permite indicar que el enumerador se ha consumido.
        /// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
        ///   siendo el ltimo retorno de largo 0 y puntero null.
        /// </returns>
        public PChar Backward(CharGroup group, char * pch, int lch, bool ignoreEmpty)
        {
            if(Pointer==null) return this;
            PChar ret;
            if(Length<1) 
            {
                ret=this;
                Pointer=null;
                Length=0;
                return ret;
            }
            int a=Length;
            if (ignoreEmpty)
            {
                while(--a!=-1 && Texts.InGroup(Pointer[a],group,pch,lch));
                if(a<1)
                {
                    if (a==-1)
                    {
                        Pointer=null;
                        Length=0;
                        return this;
                    }
                    ret=new PChar(Pointer,1);
                    Pointer=null;
                    Length=0;
                    return ret;
                }
                Length=a+1;
            }
            while(--a!=-1 && !Texts.InGroup(Pointer[a],group,pch,lch));
            if(a==-1)
            {
                ret=this;
                Pointer=null;
                Length=0;
                return ret;
            }
            ret=new PChar(Pointer+(a+1),Length-a-1);
            Length=a;
            return ret;
        }

        /// <summary>
        /// Enumerador que devuelve el PChar del bloque entre la posicin
        /// actual y proximo CR y/o LF encontrado.
        /// Se efectua trim derecho a cada bloque, y, si este es vacio, se ignora.
        /// </summary>
        /// <param name="doubleLine">Es considerado parrafo solo cuando es seguido de un segundo
        ///  salto de lnea. Los bloques no considerados parrafos se unen.
        ///  </param>
        /// <returns>
        /// - Si el separador es detectado y es considerado parrafo (luego de trim derecho a los bloques):
        ///   PChar con el parrafo.
        ///   El puntero se setea al inicio del siguiente parrafo y Length se setea en el resto del PChar 
        ///   (que puede ser cero).
        /// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
        ///   Pchar actual luego de trim derecho. El puntero se setea en null y Length se setea en 0.
        /// - Si el puntero es nulo:
        ///   PChar con puntero nulo.
        ///   El puntero nulo o Length cero permite indicar que el enumerador se ha consumido.
        /// </returns>
        public PChar FowardParagraph(bool doubleLine)
        {
            if(Pointer==null)
                return this;
            PChar ret;
            if(Length<1) 
            {
                ret=this;
                Pointer=null;
                Length=0;
                return ret;
            }

            int a,rest=0;
            otra:
                a=0;
            do
            {
                if (Pointer[a]=='\r')
                {
                    if (a!=Length-1 && Pointer[a+1]=='\n')
                        rest=2;
                    else
                        rest=1;
                    break;
                }
                else if (Pointer[a]=='\n')
                {
                    rest=1;
                    break;
                }
            }while(++a!=Length);
            ret=new PChar(Pointer,a,false,true);
            if (a==Length)
            {
                Length=0;
                Pointer=null;
                return ret;
            }
            Length-=(a+=rest);
            if (Length==0)
            {
                Pointer=null;
                return ret;
            }
            Pointer+=a;
            if (ret.Length==0)
                goto otra;
            if (!doubleLine)
                return ret;
            PChar ret2;
            otra2:
                a=0;
            do
            {
                if (Pointer[a]=='\r')
                {
                    if (a!=Length-1 && Pointer[a+1]=='\n')
                        rest=2;
                    else
                        rest=1;
                    break;
                }
                else if (Pointer[a]=='\n')
                {
                    rest=1;
                    break;
                }
            }while(++a!=Length);
            ret2=new PChar(Pointer,a,false,true);
            if (a==Length)
            {
                Length=0;
                Pointer=null;
                goto fin;

            }
            Length-=(a+=rest);
            if (Length==0)
            {
                Pointer=null;
                goto fin;
            }
            Pointer+=a;
            if (ret2.Length==0)
                return ret;
            ret.Length=(int)(ret2.Pointer-ret.Pointer)+ret2.Length;
            goto otra2;
            fin:
                if (ret2.Length!=0)
                    ret.Length=(int)(ret2.Pointer-ret.Pointer)+ret2.Length;
            return ret;
        }

		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// actual y proximo CR y/o LF encontrado.
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PChar FowardCrLf(bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;

			if (ignoreEmpty)
			{
				while((Pointer[a]=='\r' || Pointer[a]=='\n') && ++a!=Length);
				if(a==Length)
				{
					Pointer=null;
					Length=0;
					return this;
				}
				if (a!=0)
				{
					Pointer+=a;
					Length-=a;
				}
				if (Length==1)
				{
					ret=this;
					Pointer=null;
					Length=0;
					return ret;
				}
				a=1;
			}
			do
			{
				switch(Pointer[a])
				{
					case '\r':
						ret=new PChar(Pointer,a);
						if(++a!=Length && Pointer[a]=='\n') ++a;
						Pointer+=a;
						Length-=a;
						return ret;
					case '\n':
						ret=new PChar(Pointer,a);
						Pointer+=++a;
						Length-=a;
						return ret;
				}
			}while(++a<Length);
			ret=this;
			Pointer=null;
			Length=0;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// del char siguiente al ltimo CR y/o LF del bloque y el final del bloque.
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PChar BackwardCrLf(bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;

			if (ignoreEmpty)
			{
				while(--a!=-1 && (Pointer[a]=='\n' || Pointer[a]=='\r'));
				if(a<1)
				{
					if (a==-1)
					{
						Pointer=null;
						Length=0;
						return this;
					}
					ret=new PChar(Pointer,1);
					Pointer=null;
					Length=0;
					return ret;
				}
				Length=a+1;
			}

			while(--a!=-1)
			{
				switch(Pointer[a])
				{
					case '\n':
						ret=new PChar(Pointer+(a+1),Length-a-1);
						if(a!=0 && Pointer[a-1]=='\r') --a;
						Length=a;
						return ret;
					case '\r':
						ret=new PChar(Pointer+(a+1),Length-a-1);
						Length=a;
						return ret;
				}
			}
			ret=this;
			Pointer=null;
			Length=0;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// actual y proximo caracter en blanco encontrado.
		/// </summary>
		/// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
		/// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
		/// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
		/// <param name="tab">Incluye en filtro el taburador (\t).</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PChar FowardBlank(bool sp,bool lat1, bool uni,bool tab, bool crlf, bool control)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			do
			{
				switch(Pointer[a])
				{
					case ' ':
						if (sp) break;
						continue;
					case '\t':
						if (tab) break;
						continue;
					case '\n':
					case '\r':
						if (crlf) break;
						continue;
					case '\xa0':
					case '\x85':
						if (lat1) break;
						continue;
					case '\0':
					case '\v':
					case '\f':
						if (control) break;
						continue;
					case '\u1680':
					case '\u2000':
					case '\u2001':
					case '\u2002':
					case '\u2003':
					case '\u2004':
					case '\u2005':
					case '\u2006':
					case '\u2007':
					case '\u2008':
					case '\u2009':
					case '\u200a':
					case '\u200b':
					case '\u2028':
					case '\u2029':
					case '\u202f':
					case '\u3000':
					case '\ufeff':
						if (uni) break;
						continue;
					default:
						continue;
				}
				break;
			}while(++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// del char siguiente al ltimo caracter en blanco del bloque y el final del bloque.
		/// </summary>
		/// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
		/// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
		/// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
		/// <param name="tab">Incluye en filtro el taburador (\t).</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PChar BackwardBlank(bool sp,bool lat1, bool uni,bool tab, bool crlf, bool control)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			while(--a!=-1)
			{
				switch(Pointer[a])
				{
					case ' ':
						if (sp) break;
						continue;
					case '\t':
						if (tab) break;
						continue;
					case '\n':
					case '\r':
						if (crlf) break;
						continue;
					case '\xa0':
					case '\x85':
						if (lat1) break;
						continue;
					case '\0':
					case '\v':
					case '\f':
						if (control) break;
						continue;
					case '\u1680':
					case '\u2000':
					case '\u2001':
					case '\u2002':
					case '\u2003':
					case '\u2004':
					case '\u2005':
					case '\u2006':
					case '\u2007':
					case '\u2008':
					case '\u2009':
					case '\u200a':
					case '\u200b':
					case '\u2028':
					case '\u2029':
					case '\u202f':
					case '\u3000':
					case '\ufeff':
						if (uni) break;
						continue;
					default:
						continue;
				}
				break;
			}
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PChar del bloque considerado como palabra.
		/// </summary>
        /// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PChar FowardWord(bool ignoreEmpty)
		{
			if(Pointer==null) 
                return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			char c;
            
            otra:
            
			do
			{
				if(Pointer[a]<128)
				{
					c=Pointer[a];
					if((c>='a' && c<='z') || (c>='A' && c<='Z') || (c>='0' && c<='9')) 
                        continue;
					if (a!=0 && a+1!=Length)
					{
						switch(c)
						{
							case '\'':
							case '-':
							case '_':
							case '&':
							case '#':
							case '~':
							case '^':
							case '`':
								if (Texts.IsLetterNumber(Pointer[a+1]))
								{
									a++;
									continue;
								}
								break;
						}
					}
					break;
				}
				System.Globalization.UnicodeCategory uc=Texts.GetUnicodeCategory(Pointer[a]);
				if (Texts.IsLetterNumber(uc)) 
                    continue;
				if (a!=0 && a+1!=Length && 
					(uc==System.Globalization.UnicodeCategory.ModifierSymbol || 
					uc==System.Globalization.UnicodeCategory.ConnectorPunctuation) &&
                    Texts.IsLetterNumber(Pointer[a + 1]))
				{
					a++;
					continue;
				}
				break;
			}while(++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
            if (ignoreEmpty && a==0)
            {
                Length--;
                if(Length==0)
                {
                    ret=this;
                    Pointer=null;
                    return ret;
                }
                Pointer++;
                goto otra;
            }
			ret=new PChar(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}

        /// <summary>
        /// Enumerador que devuelve el PChar del bloque considerado como una parte de nombre de archivo
        /// e ignorando los separadores consecutivos.
        /// Se considera un bloque no vacio a las cadenas ".", ".." y todas las que contengan carcteres
        /// luego de trim derecho de espacios y puntos.
        /// Para obtener el PChar con un nombre de archivo verificado, usar la funcin
        /// Sys.FileHelpers.GetFileNameBlock().
        /// </summary>
        /// <returns>
        /// - Si el separador es detectado:
        ///   PChar con Length==Ubicacin separador.
        ///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
        /// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
        ///   PChar actual. El puntero se setea en null y Length se setea en 0.
        /// - Si el puntero es nulo:
        ///   PChar con puntero nulo.
        ///   El puntero nulo permite indicar que el enumerador se ha consumido.
        /// </returns>
        public PChar FowardFNPart()
        {
            if(Pointer==null) 
                return this;
            int a, l;
            char*p;
            otra:
            if(Length<1) 
            {
                PChar ret=this;
                Pointer=null;
                Length=0;
                return ret;
            }
            if (*Pointer=='\\' || *Pointer=='/')
            {
                Pointer++;
                Length--;
                goto otra;
            }
            a=0;
            l=Length;
            while(++a!=l && Pointer[a]!='\\' && Pointer[a]!='/');
            p=Pointer;
            if (a!=l)
            {
                Pointer+=a+1;
                Length-=a+1;
            }
            else
            {
                Pointer+=a;
                Length=0;
            }
            while(--a!=-1)
            {
                switch(p[a])
                {
                    case ' ':
                    case '\xa0':
                    case '\x85':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        continue;
                    case '.':
                        if (a!=0 && (a!=1 || *p!='.'))
                            continue;
                        break;
                }
                break;
            }
            if (a==-1)
                goto otra;
            return new PChar(p,a+1);
        }

		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// actual y proximo caracter en blanco encontrado.
		/// </summary>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PChar FowardBlank()
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			do
			{
				switch(Pointer[a])
				{
					case ' ':
					case '\t':
					case '\n':
					case '\r':
					case '\xa0':
					case '\x85':
					case '\0':
					case '\v':
					case '\f':
					case '\u1680':
					case '\u2000':
					case '\u2001':
					case '\u2002':
					case '\u2003':
					case '\u2004':
					case '\u2005':
					case '\u2006':
					case '\u2007':
					case '\u2008':
					case '\u2009':
					case '\u200a':
					case '\u200b':
					case '\u2028':
					case '\u2029':
					case '\u202f':
					case '\u3000':
					case '\ufeff':
						break;
					default:
						continue;
				}
				break;
			}while(++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// actual y proximo caracter en blanco encontrado + el caracter que se especifique.
		/// </summary>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		/// <param name="c">Caracter adicional a los carcteres en blanco.</param>
		public PChar FowardBlankAndChar(char c)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			do
			{
				switch(Pointer[a])
				{
					case ' ':
					case '\t':
					case '\n':
					case '\r':
					case '\xa0':
					case '\x85':
					case '\0':
					case '\v':
					case '\f':
					case '\u1680':
					case '\u2000':
					case '\u2001':
					case '\u2002':
					case '\u2003':
					case '\u2004':
					case '\u2005':
					case '\u2006':
					case '\u2007':
					case '\u2008':
					case '\u2009':
					case '\u200a':
					case '\u200b':
					case '\u2028':
					case '\u2029':
					case '\u202f':
					case '\u3000':
					case '\ufeff':
						break;
					default:
						if (Pointer[a]==c) break;
						continue;
				}
				break;
			}while(++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// del char siguiente al ltimo caracter en blanco del bloque y el final del bloque.
		/// </summary>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PChar BackwardBlank()
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			while(--a!=-1)
			{
				switch(Pointer[a])
				{
					case ' ':
					case '\t':
					case '\n':
					case '\r':
					case '\xa0':
					case '\x85':
					case '\0':
					case '\v':
					case '\f':
					case '\u1680':
					case '\u2000':
					case '\u2001':
					case '\u2002':
					case '\u2003':
					case '\u2004':
					case '\u2005':
					case '\u2006':
					case '\u2007':
					case '\u2008':
					case '\u2009':
					case '\u200a':
					case '\u200b':
					case '\u2028':
					case '\u2029':
					case '\u202f':
					case '\u3000':
					case '\ufeff':
						break;
					default:
						continue;
				}
				break;
			}
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// actual y proximo separador encontrado (determinado por cualquier elemento no-numerico).
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PChar FowardNumber(bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			if (ignoreEmpty)
			{
				while((Pointer[a]>'9' || Pointer[a]<'0') && ++a!=Length);
				if(a==Length)
				{
					Pointer=null;
					Length=0;
					return this;
				}
				if(a!=0)
				{
					Pointer+=a;
					Length-=a;
				}
				if (Length==1)
				{
					ret=this;
					Pointer=null;
					Length=0;
					return ret;
				}
				a=1;
			}
			while(Pointer[a]<='9' && Pointer[a]>='0' && ++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PChar del bloque entre la posicin
		/// del char siguiente al ltimo separador encontrado (determinado por cualquier elemento no-numerico).
		/// y el final del bloque.
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PChar con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PChar actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PChar con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PChar BackwardNumber(bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PChar ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			if (ignoreEmpty)
			{
				while(--a!=-1 && (Pointer[a]>57 || Pointer[a]<48));
				if(a<1)
				{
					if (a==-1)
					{
						Pointer=null;
						Length=0;
						return this;
					}
					ret=new PChar(Pointer,1);
					Pointer=null;
					Length=0;
					return ret;
				}
				Length=a+1;
			}
			while(--a!=-1 && Pointer[a]<='9' && Pointer[a]>='0');
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PChar(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}

		#endregion

		#region Metodos
		
		/// <summary>
		/// Efectua trim de carcteres en blanco a derecha e izquierda.
		/// </summary>
		/// <returns>Pchar en base a trim (el PChar actual no es modificado).</returns>
		public PChar Trim()
		{
            return new PChar(Pointer,Length,true,true);
		}

		/// <summary>
		/// Efectua Trim con el filtro de todos los carcteres en blanco a un puntero de cadena.
		/// </summary>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <returns>Pchar en base a trim (el PChar actual no es modificado).</returns>
		public PChar Trim(bool left, bool right)
		{
			return new PChar(Pointer,Length,left,right);
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
		/// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
		/// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
		/// <param name="tab">Incluye en filtro el taburador (\t).</param>
		/// <returns>Pchar en base a trim (el PChar actual no es modificado).</returns>
		public PChar Trim(bool left, bool right, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control)
		{
            return new PChar(Pointer,Length,left,right,sp,lat1,uni,tab,crlf,control);
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="pTrim">Puntero de carcteres de filtro.</param>
		/// <param name="lTrim">Largo del puntero de carcteres de filtro.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <returns>Pchar en base a trim (el PChar actual no es modificado).</returns>
		public PChar Trim(char * pTrim, int lTrim, bool left, bool right)
		{
			return new PChar(Pointer,Length,pTrim,lTrim,left,right);
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="cTrim">Carcteres para Trim.</param>
		/// <returns>Pchar en base a trim (el PChar actual no es modificado).</returns>
		public PChar Trim(bool left, bool right, char[] cTrim)
		{
			int ltrim=cTrim.Length;
			if (ltrim!=0)
			{
                fixed(char* ptrim=cTrim)
                {
                    return new PChar(Pointer,Length,ptrim,ltrim,left,right);
                } 

			}
			else return this;
		}


		/// <summary>
		/// Efectua un reconocimiento simple de texto encomillado:
		/// - Efectua Trim al bloque.
		/// - Si encuentra comillas coincidentes al comienzo y al final del bloque resultante, las descarta.
		/// </summary>
        /// <param name="afterTrimR">Luego del procesamiento de comillas, efecta trim derecho.</param>
        /// <param name="afterTrimL">Luego del procesamiento de comillas, efecta trim izquierdo.</param>
        /// <param name="mode">Opciones de eliminacin de comillas.</param>
        /// <returns>PChar con el bloque de texto vlido para extraccin (el PChar actual no es modificado).</returns>
		public PChar RemoveQuotes(RemoveQuotesMode mode, bool afterTrimL, bool afterTrimR)
		{
            return new PChar(Pointer,Length,mode,afterTrimL,afterTrimR);
        }

		#endregion

		#region Operadores

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PChar a.</param>
		/// <param name="b">PChar b.</param>
		/// <returns>True si son iguales.</returns>
		public static bool operator ==(PChar a, PChar b)
		{
            return a.CompareTo(b) == 0;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PChar a.</param>
		/// <param name="b">PChar b.</param>
		/// <returns>True si son diferentes.</returns>
		public static bool operator !=(PChar a, PChar b)
		{
            return a.CompareTo(b) != 0;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PChar a.</param>
		/// <param name="b">PChar b.</param>
		/// <returns>True si A es mayor a B.</returns>
		public static bool operator >(PChar a, PChar b)
		{
            return a.CompareTo(b) == 1;
		}
		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PChar a.</param>
		/// <param name="b">PChar b.</param>
		/// <returns>True si A es menor a B.</returns>
		public static bool operator <(PChar a, PChar b)
		{
            return a.CompareTo(b) == -1;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PChar a.</param>
		/// <param name="b">PChar b.</param>
		/// <returns>True si A es mayor o igual a B.</returns>
		public static bool operator >=(PChar a, PChar b)
		{
            return a.CompareTo(b) != -1;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PChar a.</param>
		/// <param name="b">PChar b.</param>
		/// <returns>True si A es menor o igual a B.</returns>
		public static bool operator <=(PChar a, PChar b)
		{
            return a.CompareTo(b) != 1;
		}

		#endregion

		#region Metodos de Object

        /// <summary>
        /// Obtiene cdigo hash en base a FNV32.
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return (int)EnPalermo.Uti.Crypto.Fnv32((byte*)this.Pointer, this.Length<<1);
        }


		/// <summary>
		/// True si son iguales.
		/// </summary>
		/// <param name="obj">Objeto.</param>
		/// <returns>True si son iguales.</returns>
		public override bool Equals(object obj)
		{
			if (obj is PChar)
			{
                return this.Equals((PChar)obj);
			}
			return false;
		}

		/// <summary>
		/// True si son iguales.
		/// </summary>
		/// <param name="b">PChar.</param>
		/// <returns>True si son iguales.</returns>
		public bool Equals(PChar b)
		{
            return this.CompareTo(b) == 0;

		}



        /// <summary>
        /// Genera una cadena mutable (o vacia si el largo es cero) en base al rango de memoria.
        /// </summary>
        /// <returns>Cadena mutable o vacia si el largo es cero.</returns>
        public override string ToString()
        {
            if (Length<1) return Others.Empty;
            return Mutables.Create(Pointer,Length);
        }
        /// <summary>
        /// Genera una cadena mutable (o vacia si el largo es cero) en base al rango de memoria.
        /// </summary>
        /// <param name="length">Largo de la cadena (no verifica rango).</param>
        /// <returns>Cadena mutable o vacia si el largo es cero.</returns>
        public string ToString(int length)
        {
            if (length<1) return Others.Empty;
            return Mutables.Create(Pointer,length);
        }
        /// <summary>
        /// Genera una cadena mutable (o vacia si el largo es cero) en base al rango de memoria.
        /// </summary>
        /// <param name="length">Largo de la cadena (no verifica rango).</param>
        /// <param name="index">Inicio (no verifica rango).</param>
        /// <returns>Cadena mutable o vacia si el largo es cero.</returns>
        public string ToString(int index, int length)
        {
            if (length<1) return Others.Empty;
            return Mutables.Create(Pointer+index,length);
        }

		#endregion

        #region IComparable<PChar> Members

        /// <summary>
        /// Compara este objeto con otro en base al contenido.
        /// </summary>
        /// <param name="other">Otro objeto.</param>
        /// <returns>-1 si menor, 0 si igual y 1 si mayor.</returns>
        public int CompareTo(PChar other)
        {
            if (this.Pointer == other.Pointer)
            {
                if (this.Length == other.Length)
                    return 0;
                if (this.Length > other.Length)
                    return 1;
                return -1;
            }
            int comp;
            if (this.Length >= other.Length)
            {
                comp = Mem.Comp.Compare(this.Pointer, other.Pointer,other.Length);
                if (comp == -1)
                    return (this.Length > other.Length) ? 1 : 0;
                return this.Pointer[comp] > other.Pointer[comp] ? 1 : -1;
            }
            comp = Mem.Comp.Compare(this.Pointer, other.Pointer, this.Length);
            if (comp == -1 || this.Pointer[comp] < other.Pointer[comp])
                return -1;
            return 1;
        }

        #endregion
    }
}
