using System;
using EnPalermo.Uti;

namespace EnPalermo.Mem
{
    /// <summary>
    /// Funciones estticas de tipo inseguro (ordenamiento).
    /// </summary>
    public unsafe static class Sort
    {

        #region Randomize

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(byte* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j;

            // tipo del puntero.
            byte temp;

            do
            {
                j = Rand.NextMinor(l);
                if (j != a)
                {
                    temp = p[a];
                    p[a] = p[j];
                    p[j] = temp;
                }
            } while (++a != l);
        }


        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(char* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j;

            // tipo del puntero.
            char temp;

            do
            {
                j = Rand.NextMinor(l);
                if (j != a)
                {
                    temp = p[a];
                    p[a] = p[j];
                    p[j] = temp;
                }
            } while (++a != l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(int* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j;

            // tipo del puntero.
            int temp;

            do
            {
                j = Rand.NextMinor(l);
                if (j != a)
                {
                    temp = p[a];
                    p[a] = p[j];
                    p[j] = temp;
                }
            } while (++a != l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(long* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j;

            // tipo del puntero.
            long temp;

            do
            {
                j = Rand.NextMinor(l);
                if (j != a)
                {
                    temp = p[a];
                    p[a] = p[j];
                    p[j] = temp;
                }
            } while (++a != l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(PByte* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j;

            // tipo del puntero.
            PByte temp;

            do
            {
                j = Rand.NextMinor(l);
                if (j != a)
                {
                    temp = p[a];
                    p[a] = p[j];
                    p[j] = temp;
                }
            } while (++a != l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(sbyte* p, int l)
        {
            Randomize((byte*)p, l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(ushort* p, int l)
        {
            Randomize((char*)p, l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(short* p, int l)
        {
            Randomize((char*)p, l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(uint* p, int l)
        {
            Randomize((int*)p, l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(ulong* p, int l)
        {
            Randomize((long*)p, l);
        }

        /// <summary>
        /// Algoritmo de desorden.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Randomize(PChar* p, int l)
        {
            Randomize((PByte*)p, l);
        }

        #endregion

        #region Reverse

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(byte* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j = l >> 1;

            // tipo del puntero.
            byte temp;

            do
            {
                temp = p[a];
                p[a] = p[--l];
                p[l] = temp;
            } while (++a != j);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(char* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j = l >> 1;

            // tipo del puntero.
            char temp;

            do
            {
                temp = p[a];
                p[a] = p[--l];
                p[l] = temp;
            } while (++a != j);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(int* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j = l >> 1;

            // tipo del puntero.
            int temp;

            do
            {
                temp = p[a];
                p[a] = p[--l];
                p[l] = temp;
            } while (++a != j);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(long* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j = l >> 1;

            // tipo del puntero.
            long temp;

            do
            {
                temp = p[a];
                p[a] = p[--l];
                p[l] = temp;
            } while (++a != j);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(PByte* p, int l)
        {
            if (l < 2)
                return;
            int a = 0;
            int j = l >> 1;

            // tipo del puntero.
            PByte temp;

            do
            {
                temp = p[a];
                p[a] = p[--l];
                p[l] = temp;
            } while (++a != j);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(sbyte* p, int l)
        {
            Reverse((byte*)p, l);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(ushort* p, int l)
        {
            Reverse((char*)p, l);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(short* p, int l)
        {
            Reverse((char*)p, l);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(uint* p, int l)
        {
            Reverse((int*)p, l);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(ulong* p, int l)
        {
            Reverse((long*)p, l);
        }

        /// <summary>
        /// Algoritmo de inversin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Reverse(PChar* p, int l)
        {
            Reverse((PByte*)p, l);
        }

        #endregion

        #region Ascending

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(byte* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(byte* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                byte x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }

        
        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(sbyte* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(sbyte* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                sbyte x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(ushort* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(ushort* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                ushort x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(short* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(short* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                short x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(char* p, int l)
        {
            if (l > 1)
                ascending((ushort*)p, 0, l - 1);
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(uint* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(uint* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                uint x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(int* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(int* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                int x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(ulong* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(ulong* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                ulong x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(long* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(long* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                long x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(PChar* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(PChar* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                PChar x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Ascending(PByte* p, int l)
        {
            if (l > 1)
                ascending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Ascendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void ascending(PByte* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                PByte x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] < x)
                        i++;
                    while (p[j] > x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                ascending(p, inf, j);
                ascending(p, i, sup);
            }
        }

        #endregion

        #region Descending

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(byte* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(byte* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                byte x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(sbyte* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(sbyte* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                sbyte x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(ushort* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(ushort* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                ushort x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(short* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(short* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                short x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(char* p, int l)
        {
            if (l > 1)
                descending((ushort*)p, 0, l - 1);
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(uint* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(uint* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                uint x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }


        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(int* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(int* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                int x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(ulong* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(ulong* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                ulong x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(long* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(long* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                long x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(PChar* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(PChar* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                PChar x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Descending(PByte* p, int l)
        {
            if (l > 1)
                descending(p, 0, l - 1);
        }

        /// <summary>
        /// Algoritmo de ordenamiento QuickSort Descendente (iteractivo).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="inf">Limite inferior, para uso iteractivo. 0 para iniciar el ordenamiento.</param>
        /// <param name="sup">Limite superior. Largo-1 para iniciar el ordenamiento.</param>
        private static void descending(PByte* p, int inf, int sup)
        {
            if (inf < sup)
            {
                int i = inf, j = sup;
                // Mismo Tipo
                PByte x = p[(i + j) >> 1], temp;
                do
                {
                    while (p[i] > x)
                        i++;
                    while (p[j] < x)
                        j--;
                    if (i <= j)
                    {
                        temp = p[i];
                        p[i] = p[j];
                        p[j] = temp;
                        i++;
                        j--;
                    }
                } while (i <= j);
                descending(p, inf, j);
                descending(p, i, sup);
            }
        }


        #endregion

    }
}
