using System;
using System.Text;
using System.Security;
using System.Runtime.InteropServices;

namespace EnPalermo.Sys
{
	/// <summary>
	/// Funciones, constantes y estructuras de la Api de Windows para uso general.
	/// </summary>
    [SuppressUnmanagedCodeSecurity]
    public unsafe static class Api
    {

        #region Cadenas de DLL's y Constantes

        /// <summary>
        /// kernel32.dll
        /// </summary>
        public const string KERNEL32 = "kernel32.dll";

        /// <summary>
        /// iphlpapi32.dll
        /// </summary>
        public const string IPHLPAPI = "iphlpapi.dll";

        /// <summary>
        /// user32.dll
        /// </summary>
        public const string USER32 = "user32.dll";

        /// <summary>
        /// advapi32.dll
        /// </summary>
        public const string ADVAPI32 = "advapi32.dll";

        /// <summary>
        /// secur32.dll
        /// </summary>
        public const string SECUR32 = "secur32.dll";

        /// <summary>
        /// Constante MAX_PATH.
        /// </summary>
        public const int MAX_PATH=260;

        #endregion

        #region Funciones Generales

        /// <summary>
        /// Modifica ErrorMode.
        /// </summary>
        /// <param name="newMode">Usar 1 para desactivar mensajes.</param>
        /// <returns>ErrorMode anterior.</returns>
        [DllImport(KERNEL32)]
        public static extern int SetErrorMode(int newMode);


        /// <summary>
        /// Cierra handles.
        /// </summary>
        /// <param name="handle">Handle a cerrar. No usarlos con los handles creados con FindFirstFile.</param>
        /// <returns>True si ok.</returns>
        [DllImport(KERNEL32)]
        public static extern bool CloseHandle(void* handle);

        #endregion

        #region Entorno

        /// <summary>
        /// The GetSystemDefaultLCID function retrieves the system default locale identifier. 
        /// </summary>
        /// <returns>The return value is the system default locale identifier.</returns>
        [DllImport(Api.KERNEL32)]
        public static extern int GetSystemDefaultLCID();

        /// <summary>
        /// The GetUserDefaultLCID function retrieves the user defaultlocale identifier. 
        /// </summary>
        /// <returns>The return value is the user defaultlocale identifier.</returns>
        [DllImport(Api.KERNEL32)]
        public static extern int GetUserDefaultLCID();

        
        /// <summary>
        /// The SetEnvironmentVariable function sets the contents of the specified environment variable for the current process.
        /// </summary>
        /// <param name="lpName">Pointer to a null-terminated string that specifies the name of the environment variable. The operating system creates the environment variable if it does not exist and lpValue is not NULL.</param>
        /// <param name="lpValue">Pointer to a null-terminated string that specifies the contents of the environment variable. An environment variable has a maximum size limit of 32,767 bytes, including the trailing null terminator. </param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool SetEnvironmentVariable(string lpName, string lpValue);

        /// <summary>
        /// The GetWindowsDirectory function retrieves the path of the Windows directory. The Windows directory contains such files as applications, initialization files, and help files.
        /// </summary>
        /// <param name="sb">Pointer to the buffer to receive the null-terminated string containing the path. This path does not end with a backslash unless the Windows directory is the root directory. For example, if the Windows directory is named Windows on drive C, the path of the Windows directory retrieved by this function is C:\Windows. If the system was installed in the root directory of drive C, the path retrieved is C:\.</param>
        /// <param name="length">Maximum size of the buffer specified by the lpBuffer parameter, in TCHARs. This value should be set to MAX_PATH.</param>
        /// <returns>If the function succeeds, the return value is the length of the string copied to the buffer, in TCHARs, not including the terminating null character. If the length is greater than the size of the buffer, the return value is the size of the buffer required to hold the path. If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int GetWindowsDirectory(StringBuilder sb, int length);

        /// <summary>
        /// The ExpandEnvironmentStrings function expands environment-variable strings and replaces them with their defined values.
        /// </summary>
        /// <param name="lpSrc">
        /// Pointer to a null-terminated string that contains environment-variable strings of the form: %variableName%. For each such reference, the %variableName% portion is replaced with the current value of that environment variable.
        /// The replacement rules are the same as those used by the command interpreter. Case is ignored when looking up the environment-variable name. If the name is not found, the %variableName% portion is left undisturbed.
        /// </param>
        /// <param name="lpDst">Pointer to a buffer to receive a copy of the source buffer after all environment-variable name substitutions have been performed.</param>
        /// <param name="nSize">Maximum number of TCHARs that can be stored in the buffer pointed to by the lpDst parameter. When using ANSI strings, the buffer size should be the string length, plus terminating null character, plus one. When using Unicode strings, the buffer size should be the string length plus the terminating null character.</param>
        /// <returns>If the function succeeds, the return value is the number of TCHARs stored in the destination buffer, including the terminating null character. If the destination buffer is too small to hold the expanded string, the return value is the required buffer size, in TCHARs. If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int ExpandEnvironmentStrings(string lpSrc, StringBuilder lpDst, int nSize);

        /// <summary>
        /// The GetEnvironmentVariable function retrieves the contents of the specified variable from the environment block of the calling process. The contents are in the form of a null-terminated string of characters.
        /// </summary>
        /// <param name="lpName">Pointer to a null-terminated string that specifies the name of the environment variable.</param>
        /// <param name="lpValue">Pointer to a buffer that receives the contents of the specified environment variable. An environment variable has a maximum size limit of 32,767 characters, including the null-terminating character. </param>
        /// <param name="size">Size of the buffer pointed to by the lpBuffer parameter, in TCHARs.</param>
        /// <returns>If the function succeeds, the return value is the number of TCHARs stored into the buffer pointed to by lpBuffer, not including the terminating null character. If lpBuffer is not large enough to hold the data, the return value is the buffer size, in TCHARs, required to hold the value string and its terminating null character. If the function fails, the return value is zero. If the specified environment variable was not found in the environment block, GetLastError returns ERROR_ENVVAR_NOT_FOUND.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int GetEnvironmentVariable(string lpName, StringBuilder lpValue, int size);

        /// <summary>
        /// The GetSystemDirectory function retrieves the path of the system directory. The system directory contains system such files such as dynamic-link libraries, drivers, and font files.
        /// </summary>
        /// <param name="sb">Pointer to the buffer to receive the null-terminated string containing the path. This path does not end with a backslash unless the system directory is the root directory. For example, if the system directory is named Windows\System on drive C, the path of the system directory retrieved by this function is C:\Windows\System.</param>
        /// <param name="length">Maximum size of the buffer, in TCHARs. This value should be set to at least MAX_PATH+1 to allow sufficient space for the path and the null terminator.</param>
        /// <returns>If the function succeeds, the return value is the length, in TCHARs, of the string copied to the buffer, not including the terminating null character. If the length is greater than the size of the buffer, the return value is the size of the buffer required to hold the path.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int GetSystemDirectory(StringBuilder sb, int length);
        
        /// <summary>
        /// The GetTempPath function retrieves the path of the directory designated for temporary files.
        /// </summary>
        /// <param name="bufferLen">Size of the string buffer identified by lpBuffer, in TCHARs.</param>
        /// <param name="buffer">Pointer to a string buffer that receives the null-terminated string specifying the temporary file path. The returned string ends with a backslash, for example, C:\TEMP\.</param>
        /// <returns>If the function succeeds, the return value is the length, in TCHARs, of the string copied to lpBuffer, not including the terminating null character. If the return value is greater than nBufferLength, the return value is the size of the buffer required to hold the path.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto)]
        public static extern uint GetTempPath(int bufferLen, StringBuilder buffer);

        /// <summary>
        /// The GetUserName function retrieves the user name of the current thread. This is the name of the user currently logged onto the system.
        /// </summary>
        /// <param name="lpBuffer">Pointer to the buffer to receive the null-terminated string containing the user's logon name. If this buffer is not large enough to contain the entire user name, the function fails. A buffer size of (UNLEN + 1) characters will hold the maximum length user name including the terminating null character. UNLEN is defined in Lmcons.h.</param>
        /// <param name="nSize"> On input, this variable specifies the size of the lpBuffer buffer, in TCHARs. On output, the variable receives the number of TCHARs copied to the buffer, including the terminating null character. If lpBuffer is too small, the function fails and GetLastError returns ERROR_MORE_DATA. This parameter receives the required buffer size, including the terminating null character.</param>
        /// <returns>If the function succeeds, the return value is a nonzero value, and the variable pointed to by nSize contains the number of TCHARs copied to the buffer specified by lpBuffer, including the terminating null character.</returns>
        [DllImport(ADVAPI32, CharSet = CharSet.Auto)]
        public static extern bool GetUserName(StringBuilder lpBuffer, ref int nSize);

        /// <summary>
        /// The GetUserNameEx function retrieves the name of the user or other security principal associated with the calling thread. You can specify the format of the returned name. If the thread is impersonating a client, GetUserNameEx returns the name of the client.
        /// </summary>
        /// <param name="format">Format of the name. This parameter is a value from the EXTENDED_NAME_FORMAT enumeration type. It cannot be NameUnknown.</param>
        /// <param name="domainName">Pointer to a buffer that receives the name in the specified format. The buffer must include space for the terminating null character. </param>
        /// <param name="domainNameLen">On input, this variable specifies the size of the lpNameBuffer buffer, in TCHARs. If the function is successful, the variable receives the number of TCHARs copied to the buffer, not including the terminating null character.
        /// If lpNameBuffer is too small, the function fails and GetLastError returns ERROR_MORE_DATA. This parameter receives the required buffer size, including the terminating null character.</param>
        /// <returns>
        /// If the function succeeds, the return value is a nonzero value.
        /// If the function fails, the return value is zero. To get extended error information, call GetLastError. If the user name is not available in the specified format, the error is ERROR_NONE_MAPPED. If the domain controller is not available to perform the lookup, the error is ERROR_NO_SUCH_DOMAIN.</returns>
        [DllImport(Api.SECUR32, CharSet = CharSet.Unicode, SetLastError = true)]
        public static extern byte GetUserNameEx(int format, StringBuilder domainName, ref int domainNameLen);

        /// <summary>
        /// The GetTickCount function retrieves the number of milliseconds that have elapsed since the system was started. It is limited to the resolution of the system timer. To obtain the system timer resolution, use the GetSystemTimeAdjustment function.
        /// </summary>
        /// <returns>The return value is the number of milliseconds that have elapsed since the system was started.</returns>
        [DllImport(Api.KERNEL32)]
        public static extern uint GetTickCount();


        #endregion

        #region Archivos

        /// <summary>
        /// The GetShortPathName function retrieves the short path form of a specified input path.
        /// </summary>
        /// <param name="lpszLongPath">Pointer to a null-terminated path string. The function retrieves the short form of this path.</param>
        /// <param name="lpszShortPath">Pointer to a buffer to receive the null-terminated short form of the path specified by lpszLongPath.</param>
        /// <param name="cchBuffer">Size of the buffer pointed to by lpszShortPath, in TCHARs.</param>
        /// <returns>If the function succeeds, the return value is the length, in TCHARs, of the string copied to lpszShortPath, not including the terminating null character. If the lpszShortPath buffer is too small to contain the path, the return value is the size of the buffer, in TCHARs, required to hold the path. Therefore, if the return value is greater than cchBuffer, call the function again with a buffer that is large enough to hold the path. If the function fails for any other reason, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto)]
        public static extern int GetShortPathName(string lpszLongPath, StringBuilder lpszShortPath, int cchBuffer);

        /// <summary>
        /// The CreateDirectory function creates a new directory. If the underlying file system supports security on files and directories, the function applies a specified security descriptor to the new directory.
        /// </summary>
        /// <param name="path">Pointer to a null-terminated string that specifies the path of the directory to be created. There is a default string size limit for paths of 248 characters. This limit is related to how the CreateDirectory function parses paths.</param>
        /// <param name="lpSecurityAttributes">Pointer to a SECURITY_ATTRIBUTES structure. The lpSecurityDescriptor member of the structure specifies a security descriptor for the new directory. If lpSecurityAttributes is NULL, the directory gets a default security descriptor. The ACLs in the default security descriptor for a directory are inherited from its parent directory. </param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto)]
        public static extern bool CreateDirectory(string path, void* lpSecurityAttributes);

        /// <summary>
        /// The CopyFile function copies an existing file to a new file. The CopyFileEx function provides two additional capabilities. CopyFileEx can call a specified callback function each time a portion of the copy operation is completed, and CopyFileEx can be canceled during the copy operation.
        /// </summary>
        /// <param name="src">Pointer to a null-terminated string that specifies the name of an existing file. </param>
        /// <param name="dst">Pointer to a null-terminated string that specifies the name of the new file.</param>
        /// <param name="failIfExists">If this parameter is TRUE and the new file specified by lpNewFileName already exists, the function fails. If this parameter is FALSE and the new file already exists, the function overwrites the existing file and succeeds.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool CopyFile(string src, string dst, bool failIfExists);

        /// <summary>
        /// The GetFileAttributesEx function retrieves attributes for a specified file or directory.
        /// </summary>
        /// <param name="name">Pointer to a null-terminated string that specifies a file or directory.</param>
        /// <param name="fileInfoLevel">Class of attribute information to retrieve. This parameter can be one of the following values.</param>
        /// <param name="lpFileInformation">Pointer to a buffer that receives the attribute information. The type of attribute information stored into this buffer is determined by the value of fInfoLevelId.</param>
        /// <returns></returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool GetFileAttributesEx(string name, int fileInfoLevel, FileAttributeData* lpFileInformation);

        /// <summary>
        /// The SetFileAttributes function sets a file's attributes.
        /// </summary>
        /// <param name="name">Pointer to a string that specifies the name of the file whose attributes are to be set.</param>
        /// <param name="attr">File attributes to set for the file. This parameter can be one or more of the following values. However, all other values override FILE_ATTRIBUTE_NORMAL.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool SetFileAttributes(string name, System.IO.FileAttributes attr);

        /// <summary>
        /// The SetFileTime function sets the date and time that a file was created, last accessed, or last modified.
        /// </summary>
        /// <param name="hFile">Handle to the file for which to set the dates and times. The file handle must have been created with the FILE_WRITE_ATTRIBUTES access right. For more information, see File Security and Access Rights. </param>
        /// <param name="creationTime">Pointer to a FILETIME structure that contains the date and time the file was created. This parameter can be NULL if the application does not need to set this information.</param>
        /// <param name="lastAccessTime">Pointer to a FILETIME structure that contains the date and time the file was last accessed. The last access time includes the last time the file was written to, read from, or (in the case of executable files) run. This parameter can be NULL if the application does not need to set this information.</param>
        /// <param name="lastWriteTime">Pointer to a FILETIME structure that contains the date and time the file was last written to. This parameter can be NULL if the application does not want to set this information.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, SetLastError = true)]
        public static extern bool SetFileTime(IntPtr hFile, FileTime* creationTime, FileTime* lastAccessTime, FileTime* lastWriteTime);

        
        /// <summary>
        /// The GetFullPathName function retrieves the full path and file name of a specified file.
        /// </summary>
        /// <param name="path">Pointer to a null-terminated string that specifies a valid file name. This string can use either short (the 8.3 form) or long file names.</param>
        /// <param name="numBufferChars">Size of the buffer to receive the null-terminated string for the drive and path, in TCHARs.</param>
        /// <param name="buffer">Pointer to a buffer that receives the null-terminated string for the drive and path. </param>
        /// <param name="mustBeZero">Pointer to a buffer that receives the address (in lpBuffer) of the final file name component in the path.</param>
        /// <returns>If the function succeeds, the return value is the length of the string copied to lpBuffer, not including the terminating null character, in TCHARs.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto)]
        public static extern int GetFullPathName(char* path, int numBufferChars, char* buffer, IntPtr mustBeZero);


        /// <summary>
        /// The MoveFile function moves an existing file or a directory, including its children.
        /// </summary>
        /// <param name="src">Pointer to a null-terminated string that names an existing file or directory.</param>
        /// <param name="dst">Pointer to a null-terminated string that specifies the new name of a file or directory. The new name must not already exist. A new file may be on a different file system or drive. A new directory must be on the same drive. </param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool MoveFile(string src, string dst);

        /// <summary>
        /// The RemoveDirectory function deletes an existing empty directory.
        /// </summary>
        /// <param name="path">Pointer to a null-terminated string that specifies the path of the directory to be removed. The path must specify an empty directory, and the calling process must have delete access to the directory.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool RemoveDirectory(string path);


        /// <summary>
        /// Obtiene atributos de archivo o directorio.
        /// </summary>
        /// <param name="path">Path.</param>
        /// <returns>Attibutos (convertible a System.IO.FileAttributes) o Int32.MinValue (si el path no existe).</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = false)]
        public static extern int GetFileAttributes(string path);

        /// <summary>
        /// Obtiene atributos de archivo o directorio.
        /// </summary>
        /// <param name="path">Path.</param>
        /// <returns>Attibutos (convertible a System.IO.FileAttributes) o Int32.MinValue (si el path no existe).</returns>
        [DllImport(Api.KERNEL32, SetLastError = false, ExactSpelling=true)]
        public static extern int GetFileAttributesW(char* path);

        /// <summary>
        /// Obtiene atributos de archivo o directorio.
        /// </summary>
        /// <param name="path">Path.</param>
        /// <returns>Attibutos (convertible a System.IO.FileAttributes) o Int32.MinValue (si el path no existe).</returns>
        [DllImport(Api.KERNEL32, SetLastError = false, ExactSpelling = true)]
        public static extern int GetFileAttributesA(byte* path);
   
        /// <summary>
        /// The GetCurrentDirectory function retrieves the current directory for the current process.
        /// </summary>
        /// <param name="nBufferLength">Length of the buffer for the current directory string, in TCHARs. The buffer length must include room for a terminating null character.</param>
        /// <param name="lpBuffer">Pointer to the buffer that receives the current directory string. This null-terminated string specifies the absolute path to the current directory.</param>
        /// <returns>If the function succeeds, the return value specifies the number of characters written to the buffer, not including the terminating null character.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Unicode, ExactSpelling = true)]
        public static extern int GetCurrentDirectoryW(int nBufferLength, char* lpBuffer);

        /// <summary>
        /// The GetCurrentDirectory function retrieves the current directory for the current process.
        /// </summary>
        /// <param name="nBufferLength">Length of the buffer for the current directory string, in TCHARs. The buffer length must include room for a terminating null character.</param>
        /// <param name="lpBuffer">Pointer to the buffer that receives the current directory string. This null-terminated string specifies the absolute path to the current directory.</param>
        /// <returns>If the function succeeds, the return value specifies the number of characters written to the buffer, not including the terminating null character.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Ansi, ExactSpelling = true)]
        public static extern int GetCurrentDirectoryA(int nBufferLength, byte* lpBuffer);

        /// <summary>
        /// The SetCurrentDirectory function changes the current directory for the current process.
        /// </summary>
        /// <param name="path">Pointer to a null-terminated string that specifies the path to the new current directory. This parameter may be a relative path or a full path. In either case, the full path of the specified directory is calculated and stored as the current directory. </param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto)]
        public static extern bool SetCurrentDirectory(string path);

        /// <summary>
        /// The GetLongPathName function converts the specified path to its long form. If no long path is found, this function simply returns the specified name.</summary>
        /// <param name="path">Pointer to a null-terminated path to be converted.</param>
        /// <param name="longPathBuffer">Pointer to the buffer to receive the long path. You can use the same buffer you used for the lpszShortPath parameter.</param>
        /// <param name="bufferLength">Size of the buffer, in TCHARs.</param>
        /// <returns>If the function succeeds, the return value is the length of the string copied to the lpszLongPath parameter, in TCHARs. This length does not include the terminating null character. If the lpszLongPath buffer is too small to contain the path, the return value is the size of the buffer, in TCHARs, required to hold the path. Therefore, if the return value is greater than cchBuffer, call the function again with a buffer that is large enough to hold the path.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Ansi)]
        public static extern int GetLongPathName(char* path, char* longPathBuffer, int bufferLength);

        /// <summary>
        /// The DeleteFile function deletes an existing file.
        /// </summary>
        /// <param name="path">Pointer to a null-terminated string that specifies the file to be deleted. </param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool DeleteFile(string path);

        #endregion

        #region Procesos

        /// <summary>
        /// The GetCommandLine function retrieves the command-line string for the current process.
        /// </summary>
        /// <returns>The return value is a pointer to the command-line string for the current process.</returns>
        [DllImport(KERNEL32, CharSet = CharSet.Auto)]
        public static extern string GetCommandLine();

        /// <summary>
        /// The GetModuleFileName function retrieves the fully qualified path for the specified module.
        /// </summary>
        /// <param name="hModule">Handle to the module whose path is being requested. If this parameter is NULL, GetModuleFileName retrieves the path for the current module.</param>
        /// <param name="buffer">Pointer to a buffer that receives a null-terminated string that specifies the fully-qualified path of the module. If the length of the path exceeds the size specified by the nSize parameter, the function succeeds and the string is truncated to nSize characters and null terminated.
        /// The path can have the prefix "\\?\", depending on how the module was loaded. For more information, see Naming a File.
        /// </param>
        /// <param name="length">Size of the lpFilename buffer, in TCHARs.</param>
        /// <returns>If the function succeeds, the return value is the length of the string copied to the buffer, in TCHARs. If the buffer is too small to hold the module name, the string is truncated to nSize, and the function returns nSize.</returns>
        [DllImport(KERNEL32, CharSet = CharSet.Auto)]
        public static extern int GetModuleFileName(HandleRef hModule, StringBuilder buffer, int length);

        /// <summary>
        /// The OpenProcess function opens an existing process object.
        /// </summary>
        /// <param name="access">Access to the process object. This access right is checked against any security descriptor for the process. This parameter can be one or more of the process access rights.</param>
        /// <param name="inherit">If this parameter is TRUE, the handle is inheritable. If the parameter is FALSE, the handle cannot be inherited.</param>
        /// <param name="processId">Identifier of the process to open.</param>
        /// <returns>If the function succeeds, the return value is an open handle to the specified process. If the function fails, the return value is NULL. To get extended error information, call GetLastError. </returns>
        [DllImport(KERNEL32, CharSet = CharSet.Auto)]
        public static extern IntPtr OpenProcess(int access, bool inherit, int processId);

        /// <summary>
        /// The SetPriorityClass function sets the priority class for the specified process. This value together with the priority value of each thread of the process determines each thread's base priority level.
        /// </summary>
        /// <param name="handle">Handle to the process.</param>
        /// <param name="priorityClass">Priority class for the process.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(KERNEL32, CharSet = CharSet.Auto)]
        public static extern bool SetPriorityClass(IntPtr handle, int priorityClass);

        /// <summary>
        /// The GetCurrentProcess function retrieves a pseudo handle for the current process.
        /// </summary>
        /// <returns>The return value is a pseudo handle to the current process.</returns>
        [DllImport(KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern IntPtr GetCurrentProcess();

        /// <summary>
        /// The GetCurrentProcessId function retrieves the process identifier of the calling process.
        /// </summary>
        /// <returns>The return value is the process identifier of the calling process.</returns>
        [DllImport(KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern uint GetCurrentProcessId();

        #endregion

        #region Consola

        /// <summary>
        /// The GetConsoleWindow function retrieves the window handle used by the console associated with the calling process.
        /// </summary>
        /// <returns>The return value is a handle to the window used by the console associated with the calling process or NULL if there is no such associated console.</returns>
        [DllImport(KERNEL32, CharSet = CharSet.Auto)]
        public static extern int GetConsoleWindow();

        /// <summary>
        /// The SetConsoleTitle function sets the title bar string for the current console window.
        /// </summary>
        /// <param name="title">Pointer to a null-terminated string that contains the string to be displayed in the title bar of the console window. The total size must be less than 64K.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(KERNEL32, CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool SetConsoleTitle(string title);

        /// <summary>
        /// The GetConsoleCP function retrieves the input code page used by the console associated with the calling process. A console uses its input code page to translate keyboard input into the corresponding character value.
        /// </summary>
        /// <returns>The return value is a code that identifies the code page.</returns>
        [DllImport(Api.KERNEL32)]
        public static extern uint GetConsoleCP();

        /// <summary>
        /// The GetConsoleMode function retrieves the current input mode of a console's input buffer or the current output mode of a console screen buffer.
        /// </summary>
        /// <param name="hConsoleHandle">Handle to a console input buffer or a console screen buffer. The handle must have the GENERIC_READ access right. For more information, see Console Buffer Security and Access Rights.</param>
        /// <param name="mode">Pointer to a variable that receives the current mode of the specified buffer.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(KERNEL32, SetLastError = true)]
        public static extern bool GetConsoleMode(IntPtr hConsoleHandle, out int mode);

        #endregion
	}

}
