using System;
using System.Security;
using System.Runtime.InteropServices;


namespace EnPalermo.Sys
{
	/// <summary>
	/// Funciones para interactuar con el PC-Speaker de la computadora local.
	/// </summary>
	[SuppressUnmanagedCodeSecurity]
    public static class Beeps
    {

        /// <summary>
        /// The Beep function generates simple tones on the speaker. The function is synchronous; it does not return control to its caller until the sound finishes.
        /// </summary>
        /// <param name="frecuency">Frequency of the sound, in hertz. This parameter must be in the range 37 through 32,767 (0x25 through 0x7FFF).</param>
        /// <param name="duration">Duration of the sound, in milliseconds.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, SetLastError=false, ExactSpelling = true)]
        public static extern bool Beep(int frecuency, int duration);

		/// <summary>
		/// Constructor Esttico.
		/// </summary>
		static Beeps()
		{
			MelodyDown=new short[]{330, 400, 440, 400, 523, 400, 494, 400, 440, 400, 523, 400, 440, 400, 494, 400, 440, 400, 349, 400, 392, 400, 330, 1200};
			MelodyUp=new short[]{587, 200, 784, 200, 784, 100, 784, 100, 784, 200, 784, 200, 988, 200, 1174, 200, 988, 100, 880, 100, 784, 200, 880, 200, 880, 200, 880, 200, 880, 200, 740, 200, 880, 200, 740, 100, 659, 100, 587, 100, 587, 100, 784, 200, 784, 200, 784, 200, 784, 200, 988, 200, 1174, 200, 988, 100, 880, 100, 784, 200, 880, 200, 880, 200, 587, 200, 740, 200, 784, 100, 784, 100, 784, 100, 784, 100, 784, 100, 784, 100, 784, 200};
		}

		/// <summary>
		/// Melodia Down (Neg).
		/// </summary>
		public static readonly short[] MelodyDown;
		
		/// <summary>
		/// Melodia Up (Pos).
		/// </summary>
		public static readonly short[] MelodyUp;

		/// <summary>
		/// Beep.
		/// </summary>
		/// <remarks>Un beep de 800 hz y tres cuartos de segundo.</remarks>
		public static void Beep()
		{
			Beep(800, 750, 1, false);
		}
		/// <summary>
		/// Beep.
		/// </summary>
		/// <param name="count">Cantidad de Beeps.</param>
		/// <param name="increase">Aumento progresivo de frecuencia y duracin.</param>
		public static void Beep(int count, bool increase)
		{
			if (increase)
			    Beep(200, 300, count, true);
			else 
                Beep(800, 500, count, false);

		}

		/// <summary>
		/// Ejecuta una meloda con el speaker.
		/// </summary>
		/// <param name="melody">Meloda consistente en pares frecuencia - duracin</param>
		public static void Beep(short[] melody)
		{
			int ml=melody.Length;
			if (ml<2)
                return;
            int a=0,f,d;
            do
            {
                f=melody[a];
                if (f < 37)
                    continue;
                d=melody[a+1];
                if (d < 5)
                    continue;
                if (!Beep(f, d))
                    return;
            }while((a+=2)<ml);
		}

        /// <summary>
        /// Ejecuta una meloda con el speaker.
        /// </summary>
        /// <param name="melody">Meloda consistente en pares frecuencia - duracin</param>
        public static void Beep(int[] melody)
        {
            int ml=melody.Length;
            if (ml<2)
                return;
            int a=0,f,d;
            do
            {
                f=melody[a];
                if (f < 37 || f > 32767)
                    continue;
                d=melody[a+1];
                if (d < 5 || d > 10000)
                    continue;
                if (!Beep(f, d))
                    return;
            }while((a+=2)<ml);
        }


		/// <summary>
		/// Beep.
		/// </summary>
		/// <param name="frecuency">Frecuencia inicial.</param>
		/// <param name="duration">Duracin inicial.</param>
		/// <param name="count">Cantidad.</param>
		/// <param name="increase">Aumento progresivo de frecuencia y duracin.</param>
		public static void Beep(int frecuency, int duration, int count, bool increase)
		{
            if (count < 1) 
                return;
            int a=0;
            do
            {
                if (!Beep(frecuency, duration))
                    return;
                if (increase)
                {
                    if (frecuency < 2500)
                    {
                        frecuency = (frecuency * 3) >>1;
                        if (duration < 1000)
                            duration = (duration * 3) >>1;
                    }
                    else if (duration < 1000)
                        duration = (duration * 3) >>1;
                    else
                        increase=false;
                }
            }while(++a!=count);
		}
	}
}
