using System;
using System.Runtime.InteropServices;

namespace EnPalermo.Sys
{
    /// <summary>
    /// Estructura WIN32_FILE_ATTRIBUTE_DATA.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 4)]
    public struct FileAttributeData
    {
        /// <summary>
        /// Atributos (System.IO.FileAttributes).
        /// </summary>
        public System.IO.FileAttributes Attributes;

        /// <summary>
        /// Fecha de creacin.
        /// </summary>
        public FileTime Creation;

        /// <summary>
        /// Fecha de ltimo acceso.
        /// </summary>
        public FileTime LastAccess;

        /// <summary>
        /// Fecha de modificacin.
        /// </summary>
        public FileTime LastWrite;

        /// <summary>
        /// Tamao (alto).
        /// </summary>
        private uint sizeHigh;

        /// <summary>
        /// Tamao (bajo).
        /// </summary>
        private uint sizeLow;

        /// <summary>
        /// Tamao.
        /// </summary>
        public long Size
        {
            get
            {
                if (sizeHigh!=0U)
                    return (((long)sizeHigh) << 32) + (long)sizeLow;
                return (long)sizeLow;
            }
        }

        /// <summary>
        /// Tamao en UInt. Si el tamao supera o iguala los 4GB, retornar UInt32.MaxValue.
        /// </summary>
        public uint SizeUInt
        {
            get
            {
                return sizeLow;
            }
        }

        /// <summary>
        /// Tamao en Int. Si el tamao supera o iguala los 2GB, retornar Int32.MaxValue.
        /// </summary>
        public int SizeInt
        {
            get
            {
                if ((sizeLow&(1U<<31))!=0)
                    return Int32.MaxValue;
                return (int)sizeLow;
            }
        }

    }
}
