using System;
using System.Security;
using System.Runtime.InteropServices;
using EnPalermo.Lit;
using EnPalermo.Txt;

namespace EnPalermo.Sys
{
	/// <summary>
	/// Mtodos y propiedades con informacin variada del entorno.
	/// </summary>
    [SuppressUnmanagedCodeSecurity]
    public static class Info
    {
        #region ComputerName

        /// <summary>
        /// The GetComputerName function retrieves the NetBIOS name of the local computer. This name is established at system startup, when the system reads it from the registry.
        /// If the local computer is a node in a cluster, GetComputerName returns the name of the cluster virtual server.
        /// GetComputerName retrieves only the NetBIOS name of the local computer. To retrieve the DNS host name, DNS domain name, or the fully qualified DNS name, call the GetComputerNameEx function. Additional information is provided by the IADsADSystemInfo interface.
        /// </summary>
        /// <param name="nameBuffer">Pointer to a buffer that receives a null-terminated string containing the computer name or the cluster virtual server name. The buffer size should be large enough to contain MAX_COMPUTERNAME_LENGTH + 1 characters.</param>
        /// <param name="bufferSize">On input, specifies the size of the buffer, in TCHARs. On output, the number of TCHARs copied to the destination buffer, not including the terminating null character.</param>
        /// <returns>If the function fails, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, ExactSpelling=true)]
        private static unsafe extern int GetComputerNameA(byte* nameBuffer, int* bufferSize);


        private static string computerName=null;


        /// <summary>
        /// Recupera el nombre NETBIOS de la computadora.
        /// La definicion del nombre se efecta en la primer llamada. En caso de que exista algn error, se devolver
        /// una cadena vacia.
        /// </summary>
        public static string ComputerName
        {
            get
            {
                if (computerName==null)
                {
                    unsafe
                    {
                        byte* name=stackalloc byte[256];
                        int size=256;
                        if (GetComputerNameA(name,&size)!=0)
                            computerName=Mutables.Create(CodePage.AnsiDefault,name,size);
                        else
                            computerName=Others.Empty;
                    }
                }
                return computerName;
            }
        }

        /// <summary>
        /// Recupera el nombre NETBIOS de la computadora precedida por doble slash (@"\\"). 
        /// La definicion del nombre se efecta en la primer llamada. En caso de que exista algn error, se devolver
        /// una cadena vacia.
        /// </summary>
        public static string ComputerNameSlash
        {
            get
            {
                string cn=ComputerName;
                int l=cn.Length;
                if (l!=0)
                {
                    unsafe
                    {
                        string r=Mutables.Create(l+2);
                        fixed(char*p=r,pcn=cn)
                        {
                            *(int*)p=6029404;
                            int a=0;
                            do
                            {
                                p[a+2]=pcn[a];
                            }
                            while(++a!=l);
                        }
                        return r;
                    }
                }
                return cn;
            }
        }

        /// <summary>
        /// Devuelve true si la computadora en parametro coincide con la actual, o es null, o cadena vacia, o ".", o "\\.".
        /// </summary>
        /// <param name="computer">Computadora.</param>
        /// <returns>True si la computadora en parametro coincide con la actual, o es null, o cadena vacia o ".", o "\\.".</returns>
        public static bool IsMyComputer(string computer)
        {
            int l=computer==null?0:computer.Length;
            if (l==0)
                return true;
            unsafe
            {
                fixed(char*p=computer)
                {
                    if (l==1 && *p=='.')
                        return true;
                    if (l>2 && *p=='\\' && p[1]=='\\')
                    {
                        if (l==3 && computer[2]=='.')
                            return true;
                        string cn=ComputerName;
                        if(cn.Length==(l-=2))
                            fixed(char*pcn=cn)
                                return TxtComp.EqualNCInvariant(p+2,pcn,false,l);
                    }
                    else
                    {
                        string cn=ComputerName;
                        if(cn.Length==l)
                            fixed(char*pcn=cn)
                                return TxtComp.EqualNCInvariant(p,pcn,false,l);
                    }
                    return false;
                }
            
            }
            
        }

        #endregion


	}
}
