using System;
using EnPalermo.Lit;

namespace EnPalermo.Uti
{
	/// <summary>
	/// Funciones estticas para calculos varios.
	/// </summary>
    public static class Calc
    {

        /// <summary>
        /// Obtiene el multiplo de mult igual o inferior al valor.
        /// La funcin no esta diseada para operar ni verificar nmeros negativos ni por supuesto con mult == 0.
        /// </summary>
        /// <param name="n">Valor.</param>
        /// <param name="mult">Multiplicador.</param>
        /// <param name="pos">Si true y el valor final es menor a 1, devuelve mult.</param>
        /// <returns>Valor que es multiplo de mult.</returns>
        public static int MultMinor(int n, int mult, bool pos)
        {
            int ret=(n/mult)*mult;
            if (pos && ret<1)
                return mult;
            return ret;
        }

        /// <summary>
        /// Obtiene el multiplo de mult igual o superior al valor.
        /// La funcin no esta diseada para operar ni verificar nmeros negativos ni por supuesto con mult == 0.
        /// </summary>
        /// <param name="n">Valor.</param>
        /// <param name="mult">Multiplicador.</param>
        /// <param name="pos">Si true y el valor final es menor a 1, devuelve mult.</param>
        /// <returns>Valor que es multiplo de mult.</returns>
        public static int MultMajor(int n, int mult, bool pos)
        {
            int ret=(n/mult)*mult;
            if (pos && ret<1)
                return mult;
            if (ret!=n)
                return ret+mult;
            return ret;
        }

        /// <summary>
        /// Retorna el log base 2 menor o igual al del entero especificado.
        /// </summary>
        /// <param name="n">Entero.</param>
        /// <returns>-1 si es menor a 1 o el log base 2 del entero especificado.</returns>
        public static int LogBase2(int n)
        {
            int a = 30;
            while (n < 1 << a) --a;
            return a;
        }

        /// <summary>
        /// Retorna el log base 2 menor o igual al del entero especificado.
        /// </summary>
        /// <param name="n">Entero.</param>
        /// <returns>-1 si es 0 o el log base 2 del entero especificado.</returns>
        public static int LogBase2(uint n)
        {
            if (n == 0) return -1;
            int a = 31;
            while (n < 1u << a) --a;
            return a;
        }

        /// <summary>
        /// Retorna el log base 2 menor o igual al del entero especificado.
        /// </summary>
        /// <param name="n">Entero.</param>
        /// <returns>-1 si es menor a 1 o el log base 2 del entero especificado.</returns>
        public static int LogBase2(long n)
        {
            int a = 62;
            while (n < 1L << a) --a;
            return a;
        }

        /// <summary>
        /// Retorna el log base 2 menor o igual al del entero especificado.
        /// </summary>
        /// <param name="n">Entero.</param>
        /// <returns>-1 si es 0 o el log base 2 del entero especificado.</returns>
        public static int LogBase2(ulong n)
        {
            if (n == 0) return -1;
            int a = 63;
            while (n < 1ul << a) --a;
            return a;
        }

        /// <summary>
        /// Retorna el log base 2 menor o igual al del entero especificado.
        /// </summary>
        /// <param name="n">Entero.</param>
        /// <returns>-1 si es menor a 1 o el log base 2 del entero especificado.</returns>
        public static int LogBase2(short n)
        {
            int a = 14;
            while (n < 1 << a) --a;
            return a;
        }

        /// <summary>
        /// Retorna el log base 2 menor o igual al del entero especificado.
        /// </summary>
        /// <param name="n">Entero.</param>
        /// <returns>-1 si es 0 o el log base 2 del entero especificado.</returns>
        public static int LogBase2(ushort n)
        {
            if (n == 0) return -1;
            int a = 15;
            while (n < 1 << a) --a;
            return a;
        }

        /// <summary>
        /// Retorna el log base 2 menor o igual al del entero especificado.
        /// </summary>
        /// <param name="n">Entero.</param>
        /// <returns>-1 si es menor a 1 o el log base 2 del entero especificado.</returns>
        public static int LogBase2(sbyte n)
        {
            int a = 6;
            while (n < 1 << a) --a;
            return a;
        }


        /// <summary>
        /// Retorna el log base 2 menor o igual al del entero especificado.
        /// </summary>
        /// <param name="n">Entero.</param>
        /// <returns>-1 si es 0 o el log base 2 del entero especificado.</returns>
        public static int LogBase2(byte n)
        {
            if (n == 0) return -1;
            int a = 7;
            while (n < 1 << a) --a;
            return a;
        }

        /// <summary>
        /// Si el nmero es impar, le suma 1.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Nmero par igual o superior.</returns>
        public static int AddIfOdd(int n)
        {
            return (n&1)!=0?n+1:n;
        }

        /// <summary>
        /// Si el nmero es par, le suma 1.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Nmero impar igual o superior.</returns>
        public static int AddIfEven(int n)
        {
            return (n&1)==0?n+1:n;
        }

        /// <summary>
        /// Obtiene factor de escala proporcional.
        /// </summary>
        /// <param name="width">Ancho original (debe ser mayor a cero).</param>
        /// <param name="height">Alto original (debe ser mayor a cero).</param>
        /// <param name="widthScale">Ancho Mximo. Si es cero o su factor implica superar al heightScale (siempre y cuando este no sea cero), se ajusta usando heightScale. A la salida se establece en el width con escala, asegurndo un mnimo de 1.</param>
        /// <param name="heightScale">Alto Mximo. Si widthScale e heightScale son cero, genera excepcin. A la salida se establece en height con escala, asegurndo un mnimo de 1.</param>
        /// <returns>Factor de escala (multiplicativo).</returns>
        public static double Scale(int width, int height, ref int widthScale, ref int heightScale)
        {
            if (width<1)
                throw new ArgumentOutOfRangeException("width",width,Errors.MINOR_ONE);
            if (height<1)
                throw new ArgumentOutOfRangeException("height",height,Errors.MINOR_ONE);
            double s;
            if (widthScale!=0)
            {
                if (widthScale<0)
                    throw new ArgumentOutOfRangeException("widthScale",widthScale,Errors.NEGATIVE);
                s=(double)widthScale/(double)width;
                int hs=Convert.ToInt32((double)height*s);
                if (heightScale!=0 && hs>heightScale)
                {
                    if (heightScale<0)
                        throw new ArgumentOutOfRangeException("heightScale",heightScale,Errors.NEGATIVE);
                    s=(double)heightScale/(double)height;
                    widthScale=Convert.ToInt32((double)width*s);
                    if (widthScale<1)
                        widthScale=1;
                }
                else if (hs<1)
                    heightScale=1;
                else
                    heightScale=hs;
            }
            else if (heightScale!=0)
            {
                if (heightScale<0)
                    throw new ArgumentOutOfRangeException("heightScale",heightScale,Errors.NEGATIVE);
                s=(double)heightScale/(double)height;
                widthScale=Convert.ToInt32((double)width*s);
                if (widthScale<1)
                    widthScale=1;
            }
            else
                throw new ArgumentException("widthScale,heightScale",Errors.INVALID_COMBINATION);

            return s;
        }

        /// <summary>
        /// Efecta clculo de scala.
        /// </summary>
        /// <param name="x">Valor a calcular (no puede ser negativo).</param>
        /// <param name="scale">Escala (multiplicativa). Debe ser mayor a cero.</param>
        /// <param name="noCero">Si true, se asegura un retorno minimo de 1. Si false, asegura un retorno mnimo de cero.</param>
        /// <param name="round">Si true efecta redondeo. Si false, sumatoria. En todo caso, si la diferencia es menor a 0.01, actua solo en modo redondeo.</param>
        /// <returns>Valor calculado.</returns>
        public static int Scale(int x, double scale, bool noCero, bool round)
        {
            if (x<1)
            {
                if (x==0)
                    return noCero?1:0;
                throw new ArgumentOutOfRangeException("x",x,Errors.NEGATIVE);
            }
            if (scale<0d)
                throw new ArgumentOutOfRangeException("scale",scale,Errors.NEGATIVE);
            scale*=(double)x;
            x=(int)scale;
            scale-=(double)x;
            if ((scale>=0.01d && (!round || scale>=0.50d)) || (noCero && x==0))
                return x+1;
            return x;
        }



    }
}
