using System;
using EnPalermo.Lit;

namespace EnPalermo.Uti
{
    /// <summary>
    /// Clase esttica para la generacin de numeros aleatoreos.
    /// La reinicializacin peridica del generador es automtica.
    /// </summary>
    public static class Rand
    {
        /// <summary>
        /// Constante para la resolucin de rangos.
        /// </summary>
        private const double REAL_UNIT_INT = 1.0 / (double)int.MaxValue;

        /// <summary>
        /// Constante para la resolucin de rangos mayor o igual a 0 y menor a 1.
        /// </summary>
        private const double REAL_UNIT_INT1 = 1.0 / ((double)int.MaxValue + 1.0);

        /// <summary>
        /// Constante para la resolucin de rangos.
        /// </summary>
        private const double REAL_UNIT_UINT = 1.0 / (double)uint.MaxValue;

        /// <summary>
        /// Constante para la resolucin de rangos mayor o igual a 0 y menor a 1.
        /// </summary>
        private const double REAL_UNIT_UINT1 = 1.0 / ((double)uint.MaxValue+1.0);

        /// <summary>
        /// Variable 1.
        /// </summary>
        private static uint x = (uint)Environment.TickCount;
        /// <summary>
        /// Variable 2.
        /// </summary>
        private static uint w = 273326509;
        /// <summary>
        /// Variable 3.
        /// </summary>
        private static uint y = 842502087;
        /// <summary>
        /// Variable 4.
        /// </summary>
        private static uint z = 3579807591;
        /// <summary>
        /// Testigo.
        /// </summary>
        private static uint uInit = x;

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo.
        /// Esta propiedad es la base de la generacin de nmeros.
        /// </summary>
        public static uint NextUInt
        {
            get
            {
                uint t = (x ^ (x << 11));
                t = (w ^ (w >> 19)) ^ (t ^ (t >> 8));
                if ((t & 127) == 0)
                {
                    uint tick = (uint)Environment.TickCount;
                    if (tick != uInit)
                    {
                        y = 842502087 ^ t;
                        z = 3579807591;
                        w = 273326509 ^ (tick - uInit);
                        x = tick;
                        uInit = tick;
                        return t;
                    }
                }
                x = y;
                y = z;
                z = w;
                w = t;
                return t;
            }
        }


        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo.
        /// </summary>
        public static int NextInt
        {
            get
            {
                return (int)NextUInt;
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango positivo del tipo.
        /// </summary>
        public static int NextIntPos
        {
            get
            {
                return (int)(NextUInt & (uint)Int32.MaxValue);
            }
        }


        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo.
        /// </summary>
        public static ulong NextULong
        {
            get
            {
                return ((ulong)NextUInt << 32) | (ulong)NextUInt;
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo.
        /// </summary>
        public static long NextLong
        {
            get
            {
                return (long)NextULong;
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango positivo del tipo.
        /// </summary>
        public static long NextLongPos
        {
            get
            {
                return (long)(NextULong & (ulong)Int64.MaxValue);
            }
        }


        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo.
        /// </summary>
        public static ushort NextUShort
        {
            get
            {
                return (ushort)NextUInt;
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo.
        /// </summary>
        public static short NextShort
        {
            get
            {
                return (short)NextUInt;
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango positivo del tipo.
        /// </summary>
        public static short NextShortPos
        {
            get
            {
                return (short)(NextUInt & 32767);
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo.
        /// </summary>
        public static byte NextByte
        {
            get
            {
                return (byte)NextUInt;
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo.
        /// </summary>
        public static sbyte NextSByte
        {
            get
            {
                return (sbyte)NextUInt;
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango positivo del tipo.
        /// </summary>
        public static sbyte NextSBytePos
        {
            get
            {
                return (sbyte)(NextUInt & 127);
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo excluyendo NaNs e infinitos.
        /// </summary>
        public static float NextSingle
        {
            get
            {
                uint tmp;
                otra:
                tmp = NextUInt;
                if ((tmp & 0x7f800000) == 0x7f800000)
                    goto otra;

                unsafe
                {
                    return *(float*)&tmp;
                }
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango positivo del tipo excluyendo NaNs e infinitos.
        /// </summary>
        public static float NextSinglePos
        {
            get
            {
                uint tmp;
                otra:
                tmp = NextUInt;
                if ((tmp & 0x7f800000) == 0x7f800000)
                    goto otra;
                tmp &= 0x7fffffff;
                unsafe
                {
                    return *(float*)&tmp;
                }
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango del tipo excluyendo NaNs e infinitos.
        /// </summary>
        public static double NextDouble
        {
            get
            {
                unsafe
                {
                    uint* p = stackalloc uint[2];
                    otra:
                    p[1] = NextUInt;
                    if ((p[1] & 0x7ff00000) == 0x7ff00000)
                        goto otra;
                    *p = NextUInt;
                    return *(double*)p;
                }
            }
        }

        /// <summary>
        /// Obtiene un valor aleatoreo abarcando todo el rango positivo del tipo excluyendo NaNs e infinitos.
        /// </summary>
        public static double NextDoublePos
        {
            get
            {
                unsafe
                {
                    uint* p = stackalloc uint[2];
                    otra:
                    p[1] = NextUInt;
                    if ((p[1] & 0x7ff00000) == 0x7ff00000)
                        goto otra;
                    p[1] &= 0x7fffffff;
                    *p = NextUInt;
                    return *(double*)p;
                }
            }
        }

        /// <summary>
        /// Retorna true o false.
        /// </summary>
        public static bool NextBool
        {
            get
            {
                return (NextUInt & 1) != 0;
            }
        }

        /// <summary>
        /// Devuelve double mayor o igual a 0 y menor a 1
        /// con una resolucin basada en el rango positivo de Int32.
        /// </summary>
        public static double DoubleMenor1
        {
            get
            {
                return REAL_UNIT_INT1 * NextIntPos;
            }
        }

        /// <summary>
        /// Devuelve un valor mayor o igual a 0 y menor o igual a max.
        /// </summary>
        /// <param name="max">
        /// Valor mximo. A diferencia de System.Random, es incluido en el rango.
        /// Debe ser mayor o igual a 0.</param>
        /// <returns>Valor mayor o igual a 0 y menor o igual a max.</returns>
        public static int Next(int max)
        {
            if (max < 0)
                throw new ArgumentOutOfRangeException("max", max, Errors.NEGATIVE);
            double val = (REAL_UNIT_INT * NextIntPos) * max;
            int ret = (int)val;
            if (val - ret >= 0.5)
                return ret + 1;
            return ret;
        }

        /// <summary>
        /// Devuelve un valor mayor o igual a 0 y menor a max.
        /// </summary>
        /// <param name="max">
        /// Valor mximo. A igual que System.Random, no es incluido en el rango.
        /// Debe ser mayor o igual a 0.</param>
        /// <returns>Valor mayor o igual a 0 y menor a max.</returns>
        public static int NextMinor(int max)
        {
            if (max < 0)
                throw new ArgumentOutOfRangeException("max", max, Errors.NEGATIVE);
            return (int)((REAL_UNIT_INT1 * NextIntPos) * max);
        }

        /// <summary>
        /// Devuelve un valor mayor o igual a 0 y menor a max.
        /// </summary>
        /// <param name="max">
        /// Valor mximo. A igual que System.Random, no es incluido en el rango.
        /// Debe ser mayor o igual a 0.</param>
        /// <returns>Valor mayor o igual a 0 y menor a max.</returns>
        public static uint NextMinor(uint max)
        {
            return (uint)((REAL_UNIT_UINT1 * NextUInt) * max);
        }

        /// <summary>
        /// Devuelve un valor mayor o igual a min y menor a max.
        /// </summary>
        /// <param name="min">Valor minimo.</param>
        /// <param name="max">
        /// Valor mximo. Al igual que System.Random, no es incluido en el rango.
        /// Debe ser mayor o igual a min.</param>
        /// <returns>Valor mayor o igual a min y menor a max.</returns>
        public static int NextMinor(int min, int max)
        {
            if (max < min)
                throw new ArgumentOutOfRangeException("max", max, Errors.MIN_MAJOR_MAX);
            max -= min;
            if (max < 0)
                return (int)(NextMinor((uint)max) + (long)min);
            return NextMinor(max) + min;
        }

        /// <summary>
        /// Devuelve un valor mayor o igual a min y menor a max.
        /// </summary>
        /// <param name="min">Valor minimo.</param>
        /// <param name="max">
        /// Valor mximo. Al igual que System.Random, no es incluido en el rango.
        /// Debe ser mayor o igual a min.</param>
        /// <returns>Valor mayor o igual a min y menor a max.</returns>
        public static uint NextMinor(uint min, uint max)
        {
            if (max < min)
                throw new ArgumentOutOfRangeException("max", max, Errors.MIN_MAJOR_MAX);
            max -= min;
            return NextMinor(max) + min;
        }

        /// <summary>
        /// Devuelve un valor mayor o igual a 0 y menor o igual a max.
        /// </summary>
        /// <param name="max">
        /// Valor mximo. A diferencia de System.Random, es incluido en el rango.</param>
        /// <returns>Valor mayor o igual a 0 y menor o igual a max.</returns>
        public static uint Next(uint max)
        {
            double val = (REAL_UNIT_UINT * NextUInt) * max;
            uint ret = (uint)val;
            if (val - ret >= 0.5)
                return ret + 1;
            return ret;
        }

        /// <summary>
        /// Devuelve un valor mayor o igual a min y menor o igual a max.
        /// </summary>
        /// <param name="min">Valor minimo.</param>
        /// <param name="max">
        /// Valor mximo. A diferencia de System.Random, es incluido en el rango.
        /// Debe ser mayor o igual a min.</param>
        /// <returns>Valor mayor o igual a min y menor o igual a max.</returns>
        public static int Next(int min, int max)
        {
            if (max < min)
                throw new ArgumentOutOfRangeException("max", max, Errors.MIN_MAJOR_MAX);
            max -= min;
            if (max < 0)
                return (int)(Next((uint)max) + (long)min);
            return Next(max) + min;
        }

        /// <summary>
        /// Devuelve un valor mayor o igual a min y menor o igual a max.
        /// </summary>
        /// <param name="min">Valor minimo.</param>
        /// <param name="max">
        /// Valor mximo. A diferencia de System.Random, es incluido en el rango.
        /// Debe ser mayor o igual a min.</param>
        /// <returns>Valor mayor o igual a min y menor o igual a max.</returns>
        public static uint Next(uint min, uint max)
        {
            if (max < min)
                throw new ArgumentOutOfRangeException("max", max, Errors.MIN_MAJOR_MAX);
            max -= min;
            return Next(max) + min;
        }

        /// <summary>
        /// Rellena puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static unsafe void Fill(byte* p, int l)
        {
            if (l > 0)
            {
                int li = l >> 2;
                if (li != 0)
                {
                    uint* pi = (uint*)p;
                    uint* pl = pi + li;
                    do
                    {
                        *pi = NextUInt;
                    } while (++pi != pl);
                    l &= 3;
                    if (l == 0)
                        return;
                    p = (byte*)pl;
                }
                switch (l)
                {
                    case 1:
                        *p = NextByte;
                        break;
                    case 2:
                        *(ushort*)p = NextUShort;
                        break;
                    default:
                        *(ushort*)p = NextUShort;
                        p[2] = NextByte;
                        break;
                }
            }
        }

        /// <summary>
        /// Rellena puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static unsafe void Fill(ushort* p, int l)
        {
            if (l > 0)
            {
                int li = l >> 1;
                if (li != 0)
                {
                    uint* pi = (uint*)p;
                    uint* pl = pi + li;
                    do
                    {
                        *pi = NextUInt;
                    } while (++pi != pl);
                    if ((l & 1) != 0)
                        *(ushort*)pl = NextUShort;
                }
                else
                    *p = NextUShort;
            }
        }

        /// <summary>
        /// Rellena puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static unsafe void Fill(uint* p, int l)
        {
            if (l > 0)
            {
                uint* pl = p + l;
                do
                {
                    *p = NextUInt;
                } while (++p != pl);
            }
        }

        /// <summary>
        /// Rellena puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static unsafe void Fill(ulong* p, int l)
        {
            if (l > 0)
            {
                uint* pi = (uint*)p;
                uint* pl = (uint*)(p + l);
                do
                {
                    *pi = NextUInt;
                } while (++pi != pl);
            }
        }

        /// <summary>
        /// Rellena Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Fill(byte[] arr)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (byte* p = arr)
                    {
                        Fill(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Fill(sbyte[] arr)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (sbyte* p = arr)
                    {
                        Fill((byte*)p, l);
                    }
                }
            }
        }


        /// <summary>
        /// Rellena Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Fill(short[] arr)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (short* p = arr)
                    {
                        Fill((ushort*)p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Fill(ushort[] arr)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (ushort* p = arr)
                    {
                        Fill(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Fill(int[] arr)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (int* p = arr)
                    {
                        Fill((uint*)p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Fill(uint[] arr)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (uint* p = arr)
                    {
                        Fill(p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Fill(long[] arr)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (long* p = arr)
                    {
                        Fill((ulong*)p, l);
                    }
                }
            }
        }

        /// <summary>
        /// Rellena Array.
        /// </summary>
        /// <param name="arr">Array.</param>
        public static void Fill(ulong[] arr)
        {
            int l = arr.Length;
            if (l != 0)
            {
                unsafe
                {
                    fixed (ulong* p = arr)
                    {
                        Fill(p, l);
                    }
                }
            }
        }


        internal static int SiguienteMenor(int l)
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }
}
