using System;
using EnPalermo.Lit;
using EnPalermo.Col;

namespace EnPalermo.Mem
{
    /// <summary>
    /// Funciones estticas de tipo inseguro (comparacin, bsqueda y reemplazo).
    /// </summary>
    public unsafe static class Comp
    {
        #region IndexOf

        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(byte* p, int l, byte f)
        {
            int a = 0;
            while (a < l) 
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }


        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(sbyte* p, int l, sbyte f)
        {
            int a = 0;
            while (a < l)
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(ushort* p, int l, ushort f)
        {
            int a = 0;
            while (a < l)
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(char* p, int l, char f)
        {
            int a = 0;
            while (a < l)
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(short* p, int l, short f)
        {
            int a = 0;
            while (a < l)
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(uint* p, int l, uint f)
        {
            int a = 0;
            while (a < l)
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }


        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(int* p, int l, int f)
        {
            int a = 0;
            while (a < l)
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(ulong* p, int l, ulong f)
        {
            int a = 0;
            while (a < l)
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }


        /// <summary>
        /// Busca un elemento en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(long* p, int l, long f)
        {
            int a = 0;
            while (a < l)
            {
                if (p[a] == f)
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Busca una cadena de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(byte* p, int l, byte* pf, int lf, bool noCopStack)
        {
            if (lf < 2 || lf >= l)
            {
                if (lf == 1) return IndexOf(p, l, *pf);
                if (lf == l && lf > 0 && Comp.Equal(p, pf, l)) return 0;
                return -1;
            }


            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }
            int a = 0;            
            switch (lf)
            {
                case 2:
                    l--;
                    do
                    {
                        if (*(short*)(p + a) == *(short*)pf) return a;
                    } while (++a != l);
                    return -1;
                case 3:
                    l -= 2;
                    do
                    {
                        if (*(short*)(p + a) == *(short*)pf && p[a + 2] == pf[2]) return a;
                    } while (++a != l);
                    return -1;
                case 4:
                    l -= 3;
                    do
                    {
                        if (*(int*)(p + a) == *(int*)pf) return a;
                    } while (++a != l);
                    return -1;
                case 5:
                    l -= 4;
                    do
                    {
                        if (*(int*)(p + a) == *(int*)pf && p[a + 4] == pf[4]) return a;
                    } while (++a != l);
                    return -1;
                case 6:
                    l -= 5;
                    do
                    {
                        if (*(int*)(p + a) == *(int*)pf && *(short*)(p + (a + 4)) == *(short*)(pf + 4)) return a;
                    } while (++a != l);
                    return -1;
                case 7:
                    l -= 6;
                    do
                    {
                        if (*(int*)(p + a) == *(int*)pf && *(short*)(p + (a + 4)) == *(short*)(pf + 4) && p[a + 6] == pf[6]) return a;
                    } while (++a != l);
                    return -1;
                case 8:
                    l -= 7;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf) return a;
                    } while (++a != l);
                    return -1;
                case 9:
                    l -= 8;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && p[a + 8] == pf[8]) return a;
                    } while (++a != l);
                    return -1;
                case 10:
                    l -= 9;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(short*)(p + (a + 8)) == *(short*)(pf + 8)) return a;
                    } while (++a != l);
                    return -1;
                case 11:
                    l -= 10;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(short*)(p + (a + 8)) == *(short*)(pf + 8) && p[a + 10] == pf[10]) return a;
                    } while (++a != l);
                    return -1;
                case 12:
                    l -= 11;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(int*)(p + (a + 8)) == *(int*)(pf + 8)) return a;
                    } while (++a != l);
                    return -1;
                case 13:
                    l -= 12;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(int*)(p + (a + 8)) == *(int*)(pf + 8) && p[a + 12] == pf[12]) return a;
                    } while (++a != l);
                    return -1;
                case 14:
                    l -= 13;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(int*)(p + (a + 8)) == *(int*)(pf + 8) && *(short*)(p + (a + 12)) == *(short*)(pf + 12)) return a;
                    } while (++a != l);
                    return -1;
                case 15:
                    l -= 14;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(int*)(p + (a + 8)) == *(int*)(pf + 8) && *(short*)(p + (a + 12)) == *(short*)(pf + 12) && p[a + 14] == pf[14]) return a;
                    } while (++a != l);
                    return -1;
                case 16:
                    l -= 15;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(long*)(p + (a + 8)) == *(long*)(pf + 8)) return a;
                    } while (++a != l);
                    return -1;
            }
            l -= lf - 1;
            do
            {
                if (*(long*)(p + a) == *(long*)pf && *(long*)(p + (a + 8)) == *(long*)(pf + 8) && Comp.Equal(p+(a+16),pf+16, lf-16)) return a;
            } while (++a != l);
            return -1;
        }

        /// <summary>
        /// Busca una cadena de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(char* p, int l, char* pf, int lf, bool noCopStack)
        {
            if (lf < 2 || lf >= l)
            {
                if (lf == 1) return IndexOf(p, l, *pf);
                if (lf == l && lf > 0 && Comp.Equal(p, pf, l)) return 0;
                return -1;
            }
            
            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }
            int a = 0;
            switch (lf)
            {
                case 2:
                    l--;
                    do
                    {
                        if (*(int*)(p + a) == *(int*)pf) return a;
                    } while (++a != l);
                    return -1;
                case 3:
                    l -= 2;
                    do
                    {
                        if (*(int*)(p + a) == *(int*)pf && p[a + 2] == pf[2]) return a;
                    } while (++a != l);
                    return -1;
                case 4:
                    l -= 3;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf) return a;
                    } while (++a != l);
                    return -1;
                case 5:
                    l -= 4;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && p[a + 4] == pf[4]) return a;
                    } while (++a != l);
                    return -1;
                case 6:
                    l -= 5;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(int*)(p + (a + 4)) == *(int*)(pf + 4)) return a;
                    } while (++a != l);
                    return -1;
                case 7:
                    l -= 6;
                    do
                    {
                        if (*(long*)(p + a) == *(long*)pf && *(int*)(p + (a + 4)) == *(int*)(pf + 4) && p[a + 6] == pf[6]) return a;
                    } while (++a != l);
                    return -1;
                case 8:
                    l -= 7;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf && 
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4)
                            ) return a;
                    } while (++a != l);
                    return -1;
                case 9:
                    l -= 8;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf &&
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4) &&
                            p[a + 8] == pf[8]
                            ) return a;
                    } while (++a != l);
                    return -1;
                case 10:
                    l -= 9;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf &&
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4) &&
                            *(int*)(p + (a + 8)) == *(int*)(pf + 8) 
                            ) return a;
                    } while (++a != l);
                    return -1;
                case 11:
                    l -= 10;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf &&
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4) &&
                            *(int*)(p + (a + 8)) == *(int*)(pf + 8) &&
                            p[a + 10] == pf[10]
                            ) return a;
                    } while (++a != l);
                    return -1;
                case 12:
                    l -= 11;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf &&
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (a + 8)) == *(long*)(pf + 8) 
                            ) return a;
                    } while (++a != l);
                    return -1;
                case 13:
                    l -= 12;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf &&
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (a + 8)) == *(long*)(pf + 8) &&
                            p[a + 12] == pf[12]
                            ) return a;
                    } while (++a != l);
                    return -1;
                case 14:
                    l -= 13;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf &&
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (a + 8)) == *(long*)(pf + 8) &&
                            *(int*)(p + (a + 12)) == *(int*)(pf + 12) 
                            ) return a;
                    } while (++a != l);
                    return -1;
                case 15:
                    l -= 14;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf && 
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (a + 8)) == *(long*)(pf + 8) &&
                            *(int*)(p + (a + 12)) == *(int*)(pf + 12) && 
                            p[a + 14] == pf[14]
                            ) return a;
                    } while (++a != l);
                    return -1;
                case 16:
                    l -= 15;
                    do
                    {
                        if (
                            *(long*)(p + a) == *(long*)pf && 
                            *(long*)(p + (a + 4)) == *(long*)(pf + 4) && 
                            *(long*)(p + (a + 8)) == *(long*)(pf + 8) && 
                            *(long*)(p + (a + 12)) == *(long*)(pf + 12)
                            ) return a;
                    } while (++a != l);
                    return -1;
            }
            l -= lf - 1;
            do
            {
                if (
                    *(long*)(p + a) == *(long*)pf &&
                    *(long*)(p + (a + 4)) == *(long*)(pf + 4) &&
                    *(long*)(p + (a + 8)) == *(long*)(pf + 8) &&
                    *(long*)(p + (a + 12)) == *(long*)(pf + 12) &&
                    Comp.Equal(p + (a + 16), pf + 16, lf - 16)
                    ) return a;
            } while (++a != l);
            return -1;
        }

        /// <summary>
        /// Busca una cadena de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(sbyte* p, int l, sbyte* pf, int lf, bool noCopStack)
        {
            return IndexOf((byte*)p, l, (byte*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca una cadena de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(ushort* p, int l, ushort* pf, int lf,bool noCopStack)
        {
            return IndexOf((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca una cadena de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOf(short* p, int l, short* pf, int lf,bool noCopStack)
        {
            return IndexOf((char*)p, l, (char*)pf, lf, noCopStack);
        }

        #endregion

        #region LastIndexOf

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(byte* p, int l, byte f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }


        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(sbyte* p, int l, sbyte f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(ushort* p, int l, ushort f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(char* p, int l, char f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(short* p, int l, short f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(uint* p, int l, uint f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(int* p, int l, int f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(ulong* p, int l, ulong f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }



        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(long* p, int l, long f)
        {
            while (--l > -1) if (p[l] == f) return l;
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(byte* p, int l, byte* pf, int lf, bool noCopStack)
        {
            if (lf < 2 || lf >= l)
            {
                if (lf == 1) return LastIndexOf(p, l, *pf);
                if (lf == l && lf > 0 && Comp.Equal(p, pf, l)) return 0;
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }
            switch (lf)
            {
                case 2:
                    l--;
                    while (--l != -1)
                    {
                        if (*(short*)(p + l) == *(short*)pf) return l;
                    } 
                    return -1;
                case 3:
                    l -= 2;
                    while (--l != -1)
                    {
                        if (*(short*)(p + l) == *(short*)pf && p[l + 2] == pf[2]) return l;
                    } 
                    return -1;
                case 4:
                    l -= 3;
                    while (--l != -1)
                    {
                        if (*(int*)(p + l) == *(int*)pf) return l;
                    } 
                    return -1;
                case 5:
                    l -= 4;
                    while (--l != -1)
                    {
                        if (*(int*)(p + l) == *(int*)pf && p[l + 4] == pf[4]) return l;
                    } 
                    return -1;
                case 6:
                    l -= 5;
                    while (--l != -1)
                    {
                        if (*(int*)(p + l) == *(int*)pf && *(short*)(p + (l + 4)) == *(short*)(pf + 4)) return l;
                    } 
                    return -1;
                case 7:
                    l -= 6;
                    while (--l != -1)
                    {
                        if (*(int*)(p + l) == *(int*)pf && *(short*)(p + (l + 4)) == *(short*)(pf + 4) && p[l + 6] == pf[6]) return l;
                    } 
                    return -1;
                case 8:
                    l -= 7;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf) return l;
                    } 
                    return -1;
                case 9:
                    l -= 8;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && p[l + 8] == pf[8]) return l;
                    } 
                    return -1;
                case 10:
                    l -= 9;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(short*)(p + (l + 8)) == *(short*)(pf + 8)) return l;
                    } 
                    return -1;
                case 11:
                    l -= 10;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(short*)(p + (l + 8)) == *(short*)(pf + 8) && p[l + 10] == pf[10]) return l;
                    } 
                    return -1;
                case 12:
                    l -= 11;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(int*)(p + (l + 8)) == *(int*)(pf + 8)) return l;
                    } 
                    return -1;
                case 13:
                    l -= 12;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(int*)(p + (l + 8)) == *(int*)(pf + 8) && p[l + 12] == pf[12]) return l;
                    } 
                    return -1;
                case 14:
                    l -= 13;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(int*)(p + (l + 8)) == *(int*)(pf + 8) && *(short*)(p + (l + 12)) == *(short*)(pf + 12)) return l;
                    } 
                    return -1;
                case 15:
                    l -= 14;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(int*)(p + (l + 8)) == *(int*)(pf + 8) && *(short*)(p + (l + 12)) == *(short*)(pf + 12) && p[l + 14] == pf[14]) return l;
                    } 
                    return -1;
                case 16:
                    l -= 15;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(long*)(p + (l + 8)) == *(long*)(pf + 8)) return l;
                    } 
                    return -1;
            }
            l -= lf - 1;
            while (--l != -1)
            {
                if (*(long*)(p + l) == *(long*)pf && *(long*)(p + (l + 8)) == *(long*)(pf + 8) && Comp.Equal(p + (l + 16), pf + 16, lf - 16)) return l;
            }
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(char* p, int l, char* pf, int lf, bool noCopStack)
        {
            if (lf < 2 || lf >= l)
            {
                if (lf == 1) return LastIndexOf(p, l, *pf);
                if (lf == l && lf > 0 && Comp.Equal(p, pf, l)) return 0;
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }
            switch (lf)
            {
                case 2:
                    l--;
                    while (--l != -1)
                    {
                        if (*(int*)(p + l) == *(int*)pf) return l;
                    } 
                    return -1;
                case 3:
                    l -= 2;
                    while (--l != -1)
                    {
                        if (*(int*)(p + l) == *(int*)pf && p[l + 2] == pf[2]) return l;
                    } 
                    return -1;
                case 4:
                    l -= 3;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf) return l;
                    } 
                    return -1;
                case 5:
                    l -= 4;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && p[l + 4] == pf[4]) return l;
                    } 
                    return -1;
                case 6:
                    l -= 5;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(int*)(p + (l + 4)) == *(int*)(pf + 4)) return l;
                    } 
                    return -1;
                case 7:
                    l -= 6;
                    while (--l != -1)
                    {
                        if (*(long*)(p + l) == *(long*)pf && *(int*)(p + (l + 4)) == *(int*)(pf + 4) && p[l + 6] == pf[6]) return l;
                    }
                    return -1;
                case 8:
                    l -= 7;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4)
                            ) return l;
                    } 
                    return -1;
                case 9:
                    l -= 8;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                            p[l + 8] == pf[8]
                            ) return l;
                    } 
                    return -1;
                case 10:
                    l -= 9;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                            *(int*)(p + (l + 8)) == *(int*)(pf + 8)
                            ) return l;
                    }
                    return -1;
                case 11:
                    l -= 10;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                            *(int*)(p + (l + 8)) == *(int*)(pf + 8) &&
                            p[l + 10] == pf[10]
                            ) return l;
                    } 
                    return -1;
                case 12:
                    l -= 11;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (l + 8)) == *(long*)(pf + 8)
                            ) return l;
                    }
                    return -1;
                case 13:
                    l -= 12;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (l + 8)) == *(long*)(pf + 8) &&
                            p[l + 12] == pf[12]
                            ) return l;
                    } 
                    return -1;
                case 14:
                    l -= 13;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (l + 8)) == *(long*)(pf + 8) &&
                            *(int*)(p + (l + 12)) == *(int*)(pf + 12)
                            ) return l;
                    } 
                    return -1;
                case 15:
                    l -= 14;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (l + 8)) == *(long*)(pf + 8) &&
                            *(int*)(p + (l + 12)) == *(int*)(pf + 12) &&
                            p[l + 14] == pf[14]
                            ) return l;
                    }
                    return -1;
                case 16:
                    l -= 15;
                    while (--l != -1)
                    {
                        if (
                            *(long*)(p + l) == *(long*)pf &&
                            *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                            *(long*)(p + (l + 8)) == *(long*)(pf + 8) &&
                            *(long*)(p + (l + 12)) == *(long*)(pf + 12)
                            ) return l;
                    }
                    return -1;
            }
            l -= lf - 1;
            while (--l != -1)
            {
                if (
                    *(long*)(p + l) == *(long*)pf &&
                    *(long*)(p + (l + 4)) == *(long*)(pf + 4) &&
                    *(long*)(p + (l + 8)) == *(long*)(pf + 8) &&
                    *(long*)(p + (l + 12)) == *(long*)(pf + 12) &&
                    Comp.Equal(p + (l + 16), pf + 16, lf - 16)
                    ) return l;
            } 
            return -1;
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(sbyte* p, int l, sbyte* pf, int lf, bool noCopStack)
        {
            return LastIndexOf((byte*)p, l, (byte*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(ushort* p, int l, ushort* pf, int lf, bool noCopStack)
        {
            return LastIndexOf((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca elementos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin del elemento relativa a p.</returns>
        public static int LastIndexOf(short* p, int l, short* pf, int lf, bool noCopStack)
        {
            return LastIndexOf((char*)p, l, (char*)pf, lf, noCopStack);
        }

        #endregion

        #region IndexOfAny

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(byte* p, int l, byte* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return -1;

            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(p, l, *pf);
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            byte c;
            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                        return a;
                } while (++b != lf);
            } while (++a != l);
            return -1;
        }


        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(char* p, int l, char* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return -1;

            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(p, l, *pf);
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            char c;
            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                        return a;
                } while (++b != lf);
            } while (++a != l);
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(int* p, int l, int* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return -1;

            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(p, l, *pf);
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackInts && &lf > General.MinimumStack)
            {
                int* pcops = stackalloc int[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            int c;
            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                        return a;
                } while (++b != lf);
            } while (++a != l);
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(long* p, int l, long* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return -1;

            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(p, l, *pf);
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackLongs && &lf > General.MinimumStack)
            {
                long* pcops = stackalloc long[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            long c;
            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                        return a;
                } while (++b != lf);
            } while (++a != l);
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(sbyte* p, int l, sbyte* pf, int lf, bool noCopStack)
        {
            return IndexOfAny((byte*)p, l, (byte*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(ushort* p, int l, ushort* pf, int lf, bool noCopStack)
        {
            return IndexOfAny((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(short* p, int l, short* pf, int lf, bool noCopStack)
        {
            return IndexOfAny((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(uint* p, int l, uint* pf, int lf, bool noCopStack)
        {
            return IndexOfAny((int*)p, l, (int*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin.</returns>
        public static int IndexOfAny(ulong* p, int l, ulong* pf, int lf, bool noCopStack)
        {
            return IndexOfAny((long*)p, l, (long*)pf, lf, noCopStack);
        }

        #endregion

        #region LastIndexOfAny

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(byte* p, int l, byte* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return -1;

            if (lf < 2)
            {
                if (lf == 1)
                    return LastIndexOf(p, l, *pf);
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            byte c;
            while (--l != -1)
            {
                b = 0;
                c = p[l];
                do
                {
                    if (c == pf[b])
                        return l;
                } while (++b != lf);
            }
            return -1;
        }


        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(char* p, int l, char* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return -1;

            if (lf < 2)
            {
                if (lf == 1)
                    return LastIndexOf(p, l, *pf);
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            char c;
            while (--l != -1)
            {
                b = 0;
                c = p[l];
                do
                {
                    if (c == pf[b])
                        return l;
                } while (++b != lf);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(int* p, int l, int* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return -1;

            if (lf < 2)
            {
                if (lf == 1)
                    return LastIndexOf(p, l, *pf);
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackInts && &lf > General.MinimumStack)
            {
                int* pcops = stackalloc int[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int c;
            while (--l != -1)
            {
                b = 0;
                c = p[l];
                do
                {
                    if (c == pf[b])
                        return l;
                } while (++b != lf);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(long* p, int l, long* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return -1;

            if (lf < 2)
            {
                if (lf == 1)
                    return LastIndexOf(p, l, *pf);
                return -1;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackLongs && &lf > General.MinimumStack)
            {
                long* pcops = stackalloc long[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            long c;
            while (--l != -1)
            {
                b = 0;
                c = p[l];
                do
                {
                    if (c == pf[b])
                        return l;
                } while (++b != lf);
            }
            return -1;
        }


        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(sbyte* p, int l, sbyte* pf, int lf, bool noCopStack)
        {
            return LastIndexOfAny((byte*)p, l, (byte*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(ushort* p, int l, ushort* pf, int lf, bool noCopStack)
        {
            return LastIndexOfAny((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(short* p, int l, short* pf, int lf, bool noCopStack)
        {
            return LastIndexOfAny((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(uint* p, int l, uint* pf, int lf, bool noCopStack)
        {
            return LastIndexOfAny((int*)p, l, (int*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos del puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int LastIndexOfAny(ulong* p, int l, ulong* pf, int lf, bool noCopStack)
        {
            return LastIndexOfAny((long*)p, l, (long*)pf, lf, noCopStack);
        }


        #endregion

        #region BinarySearch

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(byte* p, int l, byte f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                if (p[t] <= f)
                {
                    if (p[t] == f)
                        return t;
                    i = t + 1;
                    continue;
                }
                l = t - 1;
            }
            return ~i;
        }

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(sbyte* p, int l, sbyte f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                if (p[t] <= f)
                {
                    if (p[t] == f)
                        return t;
                    i = t + 1;
                    continue;
                }
                l = t - 1;
            }
            return ~i;
        }

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(char* p, int l, char f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                if (p[t] <= f)
                {
                    if (p[t] == f)
                        return t;
                    i = t + 1;
                    continue;
                }
                l = t - 1;
            }
            return ~i;
        }

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(ushort* p, int l, ushort f)
        {
            return BinarySearch((char*)p, l, (char)f);
        }

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(short* p, int l, short f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                if (p[t] <= f)
                {
                    if (p[t] == f)
                        return t;
                    i = t + 1;
                    continue;
                }
                l = t - 1;
            }
            return ~i;
        }

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(uint* p, int l, uint f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                if (p[t] <= f)
                {
                    if (p[t] == f)
                        return t;
                    i = t + 1;
                    continue;
                }
                l = t - 1;
            }
            return ~i;
        }


        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(int* p, int l, int f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                if (p[t] <= f)
                {
                    if (p[t] == f)
                        return t;
                    i = t + 1;
                    continue;
                }
                l = t - 1;
            }
            return ~i;
        }

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(ulong* p, int l, ulong f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                if (p[t] <= f)
                {
                    if (p[t] == f)
                        return t;
                    i = t + 1;
                    continue;
                }
                l = t - 1;
            }
            return ~i;
        }

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(PByte* p, int l, PByte f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                switch (p[t].CompareTo(f))
                {
                    case -1:
                    i = t + 1;
                    break;
                    case 0:
                    return t;
                    default:
                    l = t - 1;
                    break;
                }
            }
            return ~i;
        }

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(PChar* p, int l, PChar f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                switch (p[t].CompareTo(f))
                {
                    case -1:
                    i = t + 1;
                    break;
                    case 0:
                    return t;
                    default:
                    l = t - 1;
                    break;
                }
            }
            return ~i;
        }

  
        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// El array debe estar ordenado en forma ascendente.
        /// Si existen varias ocurrencias, no necesariamente retornar la ubicacin de la primera.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public static int BinarySearch(long* p, int l, long f)
        {
            l--;
            int i = 0, t;
            while (i <= l)
            {
                t = i + ((l - i) >> 1);
                if (p[t] <= f)
                {
                    if (p[t] == f)
                        return t;
                    i = t + 1;
                    continue;
                }
                l = t - 1;
            }
            return ~i;
        }

        
        #endregion

        #region Count

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(byte* p, int l, byte f)
        {
            if (l < 1) return 0;
            int tot = 0;
            int a = 0;
            do
            {
                if (p[a] == f) tot++;
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(char* p, int l, char f)
        {
            if (l < 1) return 0;
            int tot = 0;
            int a = 0;
            do
            {
                if (p[a] == f) tot++;
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(int* p, int l, int f)
        {
            if (l < 1) return 0;
            int tot = 0;
            int a = 0;
            do
            {
                if (p[a] == f) tot++;
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(long* p, int l, long f)
        {
            if (l < 1) return 0;
            int tot = 0;
            int a = 0;
            do
            {
                if (p[a] == f) tot++;
            } while (++a != l);
            return tot;
        }


 

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(sbyte* p, int l, sbyte f)
        {
            return Count((byte*)p, l, (byte)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(ushort* p, int l, ushort f)
        {
            return Count((char*)p, l, (char)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(short* p, int l, short f)
        {
            return Count((char*)p, l, (char)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(uint* p, int l, uint f)
        {
            return Count((int*)p, l, (int)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos o 0.</returns>
        public static int Count(ulong* p, int l, ulong f)
        {
            return Count((long*)p, l, (long)f);
        }

        /// <summary>
		/// Cuenta la cantidad de cadenas de elementos en memoria.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo de bsqueda.</param>
		/// <param name="pf">Cadena de elementos a buscar.</param>
		/// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
		/// <returns>Cantidad o 0.</returns>
        public static int Count(byte* p, int l, byte* pf, int lf, bool noCopStack)
        {
            if (lf < 2 || lf >= l)
            {
                if (lf == 1) return Count(p, l, *pf);
                if (lf == l && lf > 0 && Comp.Equal(p, pf, l)) return 1;
                return 0;
            }
            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }
            int tot = 0;
            int ubi;
            while (true) 
            {
                ubi = IndexOf(p, l, pf, lf, true);
                if (ubi != -1)
                {
                    tot++;
                    ubi += lf;
                    l -= ubi;
                    if (l >= lf) 
                    {
                        p += ubi;
                        continue;
                    }
                }
                return tot;
            } 
        }

        /// <summary>
		/// Cuenta la cantidad de cadenas de elementos en memoria.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo de bsqueda.</param>
		/// <param name="pf">Cadena de elementos a buscar.</param>
		/// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
		/// <returns>Cantidad o 0.</returns>
        public static int Count(sbyte* p, int l, sbyte* pf, int lf, bool noCopStack)
        {
            return Count((byte*)p, l, (byte*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de cadenas de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int Count(char* p, int l, char* pf, int lf, bool noCopStack)
        {
            if (lf < 2 || lf >= l)
            {
                if (lf == 1) return Count(p, l, *pf);
                if (lf == l && lf > 0 && Comp.Equal(p, pf, l)) return 1;
                return 0;
            }
            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }
            int tot = 0;
            int ubi;
            while (true)
            {
                ubi = IndexOf(p, l, pf, lf, true);
                if (ubi != -1)
                {
                    tot++;
                    ubi += lf;
                    l -= ubi;
                    if (l >= lf)
                    {
                        p += ubi;
                        continue;
                    }
                }
                return tot;
            }
        }

        /// <summary>
        /// Cuenta la cantidad de cadenas de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int Count(ushort* p, int l, ushort* pf, int lf, bool noCopStack)
        {
            return Count((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de cadenas de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int Count(short* p, int l, short* pf, int lf, bool noCopStack)
        {
            return Count((char*)p, l, (char*)pf, lf, noCopStack);
        }


        #endregion

        #region CountAny

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(byte* p, int l, byte* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return 0;
            if (lf < 2)
            {
                if (lf == 1)
                    return Count(p, l, *pf);
                return 0;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            int tot = 0;
            byte c;
            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);
            return tot;
        }


        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(char* p, int l, char* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return 0;
            if (lf < 2)
            {
                if (lf == 1)
                    return Count(p, l, *pf);
                return 0;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            int tot = 0;
            char c;
            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(int* p, int l, int* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return 0;
            if (lf < 2)
            {
                if (lf == 1)
                    return Count(p, l, *pf);
                return 0;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackInts && &lf > General.MinimumStack)
            {
                int* pcops = stackalloc int[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            int tot = 0;
            int c;
            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(long* p, int l, long* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return 0;
            if (lf < 2)
            {
                if (lf == 1)
                    return Count(p, l, *pf);
                return 0;
            }

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackLongs && &lf > General.MinimumStack)
            {
                long* pcops = stackalloc long[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            int tot = 0;
            long c;
            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);
            return tot;
        }


        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(sbyte* p, int l, sbyte* pf, int lf, bool noCopStack)
        {
            return CountAny((byte*)p, l, (byte*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(ushort* p, int l, ushort* pf, int lf, bool noCopStack)
        {
            return CountAny((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(short* p, int l, short* pf, int lf, bool noCopStack)
        {
            return CountAny((char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(uint* p, int l, uint* pf, int lf, bool noCopStack)
        {
            return CountAny((int*)p, l, (int*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad o 0.</returns>
        public static int CountAny(ulong* p, int l, ulong* pf, int lf, bool noCopStack)
        {
            return CountAny((long*)p, l, (long*)pf, lf, noCopStack);
        }

        #endregion

        #region Locations

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, byte* p, int l, byte f)
        {
            if (l < 1)
                return ret;
            int a = 0;
            if (ret==null)
                ret = new Ints();
            if (sum!=0)
            {
                do
                {
                    if (p[a] == f)
                        ret.Append(a+sum);
                } while (++a != l);
            }
            else
            {
                do
                {
                    if (p[a] == f)
                        ret.Append(a);
                } while (++a != l);
            }
            return ret;
        }


        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, char* p, int l, char f)
        {
            if (l < 1)
                return ret;
            int a = 0;
            if (ret==null)
                ret = new Ints();
            if (sum!=0)
            {
                do
                {
                    if (p[a] == f)
                        ret.Append(a+sum);
                } while (++a != l);
            }
            else
            {
                do
                {
                    if (p[a] == f)
                        ret.Append(a);
                } while (++a != l);
            }
            return ret;
        }


        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, int* p, int l, int f)
        {
            if (l < 1)
                return ret;
            int a = 0;
            if (ret==null)
                ret = new Ints();
            if (sum!=0)
            {
                do
                {
                    if (p[a] == f)
                        ret.Append(a+sum);
                } while (++a != l);
            }
            else
            {
                do
                {
                    if (p[a] == f)
                        ret.Append(a);
                } while (++a != l);
            }
            return ret;
        }


        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, long* p, int l, long f)
        {
            if (l < 1)
                return ret;
            int a = 0;
            if (ret==null)
                ret = new Ints();
            if (sum!=0)
            {
                do
                {
                    if (p[a] == f)
                        ret.Append(a+sum);
                } while (++a != l);
            }
            else
            {
                do
                {
                    if (p[a] == f)
                        ret.Append(a);
                } while (++a != l);
            }
            return ret;
        }


        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, sbyte* p, int l, sbyte f)
        {
            return Locations(ret, sum, (byte*)p, l, (byte)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, short* p, int l, short f)
        {
            return Locations(ret, sum, (char*)p, l, (char)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, ushort* p, int l, ushort f)
        {
            return Locations(ret, sum, (char*)p, l, (char)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, uint* p, int l, uint f)
        {
            return Locations(ret, sum, (int*)p, l, (int)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, ulong* p, int l, ulong f)
        {
            return Locations(ret, sum, (long*)p, l, (long)f);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, byte* p, int l, byte* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return ret;
            if (lf < 2)
            {
                if (lf == 1)
                    return Locations(ret, sum, p, l, *pf);
                return ret;
            }

            if (ret==null)
                ret = new Ints();

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            byte c;
            if (sum!=0)
            {
                do
                {
                    b = 0;
                    c = p[a];
                    do
                    {
                        if (c == pf[b])
                        {
                            ret.Append(a+sum);
                            break;
                        }
                    } while (++b != lf);
                } while (++a != l);
            }
            else
            {
                do
                {
                    b = 0;
                    c = p[a];
                    do
                    {
                        if (c == pf[b])
                        {
                            ret.Append(a);
                            break;
                        }
                    } while (++b != lf);
                } while (++a != l);
            }
            return ret;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, char* p, int l, char* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return ret;
            if (lf < 2)
            {
                if (lf == 1)
                    return Locations(ret, sum, p, l, *pf);
                return ret;
            }

            if (ret==null)
                ret = new Ints();

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            char c;
            if (sum!=0)
            {
                do
                {
                    b = 0;
                    c = p[a];
                    do
                    {
                        if (c == pf[b])
                        {
                            ret.Append(a+sum);
                            break;
                        }
                    } while (++b != lf);
                } while (++a != l);
            }
            else
            {
                do
                {
                    b = 0;
                    c = p[a];
                    do
                    {
                        if (c == pf[b])
                        {
                            ret.Append(a);
                            break;
                        }
                    } while (++b != lf);
                } while (++a != l);
            }
            return ret;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, int* p, int l, int* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return ret;
            if (lf < 2)
            {
                if (lf == 1)
                    return Locations(ret, sum, p, l, *pf);
                return ret;
            }

            if (ret==null)
                ret = new Ints();

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                int* pcops = stackalloc int[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            int c;
            if (sum!=0)
            {
                do
                {
                    b = 0;
                    c = p[a];
                    do
                    {
                        if (c == pf[b])
                        {
                            ret.Append(a+sum);
                            break;
                        }
                    } while (++b != lf);
                } while (++a != l);
            }
            else
            {
                do
                {
                    b = 0;
                    c = p[a];
                    do
                    {
                        if (c == pf[b])
                        {
                            ret.Append(a);
                            break;
                        }
                    } while (++b != lf);
                } while (++a != l);
            }
            return ret;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, long* p, int l, long* pf, int lf, bool noCopStack)
        {
            if (l < 1)
                return ret;
            if (lf < 2)
            {
                if (lf == 1)
                    return Locations(ret, sum, p, l, *pf);
                return ret;
            }

            if (ret==null)
                ret = new Ints();

            if (!noCopStack && pf > General.MaximumStack && l > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                long* pcops = stackalloc long[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int b;
            int a = 0;
            long c;
            if (sum!=0)
            {
                do
                {
                    b = 0;
                    c = p[a];
                    do
                    {
                        if (c == pf[b])
                        {
                            ret.Append(a+sum);
                            break;
                        }
                    } while (++b != lf);
                } while (++a != l);
            }
            else
            {
                do
                {
                    b = 0;
                    c = p[a];
                    do
                    {
                        if (c == pf[b])
                        {
                            ret.Append(a);
                            break;
                        }
                    } while (++b != lf);
                } while (++a != l);
            }
            return ret;
        }

 
        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, sbyte* p, int l, sbyte* pf, int lf, bool noCopStack)
        {
            return LocationsAny(ret, sum, (byte*)p, l, (byte*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, short* p, int l, short* pf, int lf, bool noCopStack)
        {
            return LocationsAny(ret, sum, (char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, ushort* p, int l, ushort* pf, int lf, bool noCopStack)
        {
            return LocationsAny(ret, sum, (char*)p, l, (char*)pf, lf, noCopStack);
        }

        
        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, uint* p, int l, uint* pf, int lf, bool noCopStack)
        {
            return LocationsAny(ret, sum, (int*)p, l, (int*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos a buscar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints LocationsAny(Ints ret, int sum, ulong* p, int l, ulong* pf, int lf, bool noCopStack)
        {
            return LocationsAny(ret, sum, (long*)p, l, (long*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, byte* p, int l, byte* pf, int lf, bool noCopStack)
        {
            if (lf < 2 || lf >= l)
            {
                if (lf == 1)
                    return Locations(ret,sum,p, l, *pf);
                if (l>0)
                {
                    if (lf == l && Comp.Equal(p, pf, l))
                    {
                        if (ret==null)
                            ret=new Ints(1);
                        ret.Append(sum);
                    }
                    else if (ret==null)
                        ret=new Ints();
                }
                return ret;
            }
            if (ret==null)
                ret=new Ints();

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int ubi=l;
            if (sum!=0)
            {
                l+=sum;
                p-=sum;
            }
otra:
            ubi = IndexOf(p + sum, ubi, pf, lf, true);
            if (ubi != -1)
            {
                ret.Append(sum+=ubi);
                sum += lf;
                ubi = l - sum;
                if (lf <= ubi)
                    goto otra;
            }

            return ret;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, char* p, int l, char* pf, int lf, bool noCopStack)
        {
            if (lf < 2 || lf >= l)
            {
                if (lf == 1)
                    return Locations(ret,sum,p, l, *pf);
                if (l>0)
                {
                    if (lf == l && Comp.Equal(p, pf, l))
                    {
                        if (ret==null)
                            ret=new Ints(1);
                        ret.Append(sum);
                    }
                    else if (ret==null)
                        ret=new Ints();
                }
                return ret;
            }
            if (ret==null)
                ret=new Ints();

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int ubi=l;
            if (sum!=0)
            {
                l+=sum;
                p-=sum;
            }
            otra:
                ubi = IndexOf(p + sum, ubi, pf, lf, true);
            if (ubi != -1)
            {
                ret.Append(sum+=ubi);
                sum += lf;
                ubi = l - sum;
                if (lf <= ubi)
                    goto otra;
            }

            return ret;
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, sbyte* p, int l, sbyte* pf, int lf, bool noCopStack)
        {
            return Locations(ret, sum, (byte*)p, l, (byte*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, short* p, int l, short* pf, int lf, bool noCopStack)
        {
            return Locations(ret, sum, (char*)p, l, (char*)pf, lf, noCopStack);
        }

        /// <summary>
        /// Cuenta la cantidad de elementos en memoria y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo del puntero es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Ints nuevo o actualizado donde se almacenarn las ubicaciones. Si el Ints de entrada es nulo, la salida tambin podra serla si el largo del puntero es menor a 1.</returns>
        public static Ints Locations(Ints ret, int sum, ushort* p, int l, ushort* pf, int lf, bool noCopStack)
        {
            return Locations(ret, sum, (char*)p, l, (char*)pf, lf, noCopStack);
        }

        #endregion


        #region Remove

        /// <summary>
        /// Copia memoria filtrando cadenas de elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a eliminar.</param>
        /// <param name="lf">Largo de la cadena de elementos a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, byte* po, byte* pc, byte* pd, int l, out int lFinal, byte* pf, int lf, bool clean, bool noCopStack)
        {
            if (lf >= l || lf < 1)
            {
                if (lf == l)
                {
                    if (pc == null)
                        pc = po;
                    if ((lf == 1 && *pc == *pf) || (lf > 1 && Comp.Equal(pc, pf, l)))
                    {
                        lFinal = 0;
                        if (clean && alwaysCopy)
                            General.Clean(pd, l);
                        return 1;
                    }
                }
                if (po == pd)
                    lFinal = l;
                else if (alwaysCopy && l > 0)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                    lFinal = 0;
                return 0;
            }

            if (pd > po && pd - po < l)
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD,"pc,pd");
                General.Move(po, pd, l);
                po = pd;
            }
            else if (pc == null)
                pc = po;

            int tot = 0, ipo=0, tc;
            lFinal = 0;

            if (lf == 1)
            {
                int a = 0;
                byte f = *pf;
                do
                {
                    if (pc[a] == f)
                    {
                        tot++;
                        tc = a - ipo;
                        if (tc != 0)
                        {
                            General.Copy(po + ipo, pd + lFinal, tc);
                            lFinal += tc;
                        }
                        ipo = a + 1;
                    }
                } while (++a != l);
            }
            else 
            {
                if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
                {
                    byte* pcops = stackalloc byte[lf];
                    General.Copy(pf, pcops, lf);
                    pf = pcops;
                }

                tc = l;
otra:
                tc = IndexOf(pc + ipo, tc, pf, lf, true);
                if (tc != -1)
                {
                    tot++;
                    if (tc != 0)
                    {
                        General.Copy(po + ipo, pd + lFinal, tc);
                        lFinal += tc;
                    }
                    ipo += lf;
                    tc = l - ipo;
                    if (lf <= tc)
                        goto otra;
                }
            }

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando cadenas de elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a eliminar.</param>
        /// <param name="lf">Largo de la cadena de elementos a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, sbyte* po, sbyte* pc, sbyte* pd, int l, out int lFinal, sbyte* pf, int lf, bool clean, bool noCopStack)
        {
            return Remove(alwaysCopy, (byte*)po, (byte*)pc, (byte*)pd, l, out lFinal, (byte*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, byte* po, byte* pc, byte* pd, int l, out int lFinal, byte* pf, int lf, bool clean, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Remove(alwaysCopy, po, pc, pd, l, out lFinal, pf, lf, clean,noCopStack);
                if (po == pd)
                {
                    lFinal = l;
                }
                else if (alwaysCopy && l > 0)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                    lFinal = 0;
                return 0;
            }

            if (pd > po && pd - po < l)
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD,"pc,pd");
                General.Move(po, pd, l);
                po = pd;
            }
            else if (pc == null)
                pc = po;

            int tot = 0;
            int ipo = 0;
            int tc;
            lFinal = 0;
            int a = 0;
            int b;
            byte c;

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            do
            {
                b = 0;
                c = pc[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        tc = a - ipo;
                        if (tc != 0)
                        {
                            General.Copy(po + ipo, pd + lFinal, tc);
                            lFinal += tc;
                        }
                        ipo = a + 1;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, sbyte* po, sbyte* pc, sbyte* pd, int l, out int lFinal, sbyte* pf, int lf, bool clean, bool noCopStack)
        {
            return RemoveAny(alwaysCopy, (byte*)po, (byte*)pc, (byte*)pd, l, out lFinal, (byte*)pf, lf, clean, noCopStack);
        }
        
        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Si tot y lf son mayor a 0, debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - (tot*lf)).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, byte* po, int l, byte* pd, int lf, bool clean)
        {
            if (lf < 1 || tot < 1)
            {
                return l;
            }
            if (l < 1)
            {
                throw new ArgumentOutOfRangeException("l",l,Errors.MINOR_ONE);
            }
            int lFinal = l - (tot * lf);
            if (lFinal == 0)
            {
                if (clean)
                    General.Clean(pd, l);
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            int a = 0, ipo = 0, tc, difipo;
            lFinal = 0;
            do
            {
                tc = pCont[a];
                if (tc != ipo)
                {
                    difipo = tc - ipo;
                    General.Copy(po + ipo, pd + lFinal, difipo);
                    lFinal += difipo;
                }
                ipo = tc + lf;
            } while (++a != tot);
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean)
                General.Clean(pd + lFinal, l - lFinal);
            return lFinal;
        }

                /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Si tot y lf son mayor a 0, debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - (tot*lf)).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, sbyte* po, int l, sbyte* pd, int lf, bool clean)
        {
            return RemoveLocations(pCont, tot, (byte*)po, l, (byte*)pd, lf, clean);
        }



        /// <summary>
        /// Copia memoria filtrando cadenas de elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a eliminar.</param>
        /// <param name="lf">Largo de la cadena de elementos a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, char* po, char* pc, char* pd, int l, out int lFinal, char* pf, int lf, bool clean, bool noCopStack)
        {
            if (lf >= l || lf < 1)
            {
                if (lf == l)
                {
                    if (pc == null)
                        pc = po;
                    if ((lf == 1 && *pc == *pf) || (lf > 1 && Comp.Equal(pc, pf, l)))
                    {
                        lFinal = 0;
                        if (clean && alwaysCopy)
                            General.Clean(pd, l);
                        return 1;
                    }
                }
                if (po == pd)
                    lFinal = l;
                else if (alwaysCopy && l > 0)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                    lFinal = 0;
                return 0;
            }

            if (pd > po && pd - po < l)
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD, "pc,pd");
                General.Move(po, pd, l);
                po = pd;
            }
            else if (pc == null)
                pc = po;

            int tot = 0, ipo = 0, tc;
            lFinal = 0;

            if (lf == 1)
            {
                int a = 0;
                char f = *pf;
                do
                {
                    if (pc[a] == f)
                    {
                        tot++;
                        tc = a - ipo;
                        if (tc != 0)
                        {
                            General.Copy(po + ipo, pd + lFinal, tc);
                            lFinal += tc;
                        }
                        ipo = a + 1;
                    }
                } while (++a != l);
            }
            else
            {
                if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
                {
                    char* pcops = stackalloc char[lf];
                    General.Copy(pf, pcops, lf);
                    pf = pcops;
                }

                tc = l;
otra:
                tc = IndexOf(pc + ipo, tc, pf, lf, true);
                if (tc != -1)
                {
                    tot++;
                    if (tc != 0)
                    {
                        General.Copy(po + ipo, pd + lFinal, tc);
                        lFinal += tc;
                    }
                    ipo += lf;
                    tc = l - ipo;
                    if (lf <= tc)
                        goto otra;
                }
            }

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando cadenas de elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a eliminar.</param>
        /// <param name="lf">Largo de la cadena de elementos a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, short* po, short* pc, short* pd, int l, out int lFinal, short* pf, int lf, bool clean, bool noCopStack)
        {
            return Remove(alwaysCopy, (char*)po, (char*)pc, (char*)pd, l, out lFinal, (char*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria filtrando cadenas de elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a eliminar.</param>
        /// <param name="lf">Largo de la cadena de elementos a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, ushort* po, ushort* pc, ushort* pd, int l, out int lFinal, ushort* pf, int lf, bool clean, bool noCopStack)
        {
            return Remove(alwaysCopy, (char*)po, (char*)pc, (char*)pd, l, out lFinal, (char*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, char* po, char* pc, char* pd, int l, out int lFinal, char* pf, int lf, bool clean, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Remove(alwaysCopy, po, pc, pd, l, out lFinal, pf, lf, clean, noCopStack);
                if (po == pd)
                {
                    lFinal = l;
                }
                else if (alwaysCopy && l > 0)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                    lFinal = 0;
                return 0;
            }

            if (pd > po && pd - po < l)
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD, "pc,pd");
                General.Move(po, pd, l);
                po = pd;
            }
            else if (pc == null)
                pc = po;

            int tot = 0;
            int ipo = 0;
            int tc;
            lFinal = 0;
            int a = 0;
            int b;
            char c;

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            do
            {
                b = 0;
                c = pc[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        tc = a - ipo;
                        if (tc != 0)
                        {
                            General.Copy(po + ipo, pd + lFinal, tc);
                            lFinal += tc;
                        }
                        ipo = a + 1;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, short* po, short* pc, short* pd, int l, out int lFinal, short* pf, int lf, bool clean, bool noCopStack)
        {
            return RemoveAny(alwaysCopy, (char*)po, (char*)pc, (char*)pd, l, out lFinal, (char*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, ushort* po, ushort* pc, ushort* pd, int l, out int lFinal, ushort* pf, int lf, bool clean, bool noCopStack)
        {
            return RemoveAny(alwaysCopy, (char*)po, (char*)pc, (char*)pd, l, out lFinal, (char*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Si tot y lf son mayor a 0, debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - (tot*lf)).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, char* po, int l, char* pd, int lf, bool clean)
        {
            if (lf < 1 || tot < 1)
            {
                return l;
            }
            if (l < 1)
            {
                throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE);
            }
            int lFinal = l - (tot * lf);
            if (lFinal == 0)
            {
                if (clean)
                    General.Clean(pd, l);
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            int a = 0, ipo = 0, tc, difipo;
            lFinal = 0;
            do
            {
                tc = pCont[a];
                if (tc != ipo)
                {
                    difipo = tc - ipo;
                    General.Copy(po + ipo, pd + lFinal, difipo);
                    lFinal += difipo;
                }
                ipo = tc + lf;
            } while (++a != tot);
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean)
                General.Clean(pd + lFinal, l - lFinal);
            return lFinal;
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Si tot y lf son mayor a 0, debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - (tot*lf)).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, short* po, int l, short* pd, int lf, bool clean)
        {
            return RemoveLocations(pCont, tot, (char*)po, l, (char*)pd, lf, clean);
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Si tot y lf son mayor a 0, debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - (tot*lf)).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, ushort* po, int l, ushort* pd, int lf, bool clean)
        {
            return RemoveLocations(pCont, tot, (char*)po, l, (char*)pd, lf, clean);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, byte* po, byte* pd, int l, out int lFinal, byte f, bool clean)
        {
            if (l < 2)
            {
                if (l < 1)
                {
                    lFinal = 0;
                    return 0;
                }
                else if (*po == f)
                {
                    lFinal = 0;
                    if (clean)
                        *pd = 0;
                    return 1;
                }
                else if (alwaysCopy)
                    *pd = *po;
                lFinal = 1;
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int tot = 0, ipo = 0, a = 0, tc;
            lFinal = 0;
            do
            {
                if (po[a] == f)
                {
                    tot++;
                    tc = a - ipo;
                    if (tc != 0)
                    {
                        General.Copy(po + ipo, pd + lFinal, tc);
                        lFinal += tc;
                    }
                    ipo = a + 1;
                }
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, sbyte* po, sbyte* pd, int l, out int lFinal, sbyte f, bool clean)
        {
            return Remove(alwaysCopy, (byte*)po, (byte*)pd, l, out lFinal, (byte)f, clean);
        }
        
        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, char* po, char* pd, int l, out int lFinal, char f, bool clean)
        {
            if (l < 2)
            {
                if (l < 1)
                {
                    lFinal = 0;
                    return 0;
                }
                else if (*po == f)
                {
                    lFinal = 0;
                    if (clean)
                        *pd = '\0';
                    return 1;
                }
                else if (alwaysCopy)
                    *pd = *po;
                lFinal = 1;
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int tot = 0, ipo = 0, a = 0, tc;
            lFinal = 0;
            do
            {
                if (po[a] == f)
                {
                    tot++;
                    tc = a - ipo;
                    if (tc != 0)
                    {
                        General.Copy(po + ipo, pd + lFinal, tc);
                        lFinal += tc;
                    }
                    ipo = a + 1;
                }
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, short* po, short* pd, int l, out int lFinal, short f, bool clean)
        {
            return Remove(alwaysCopy, (char*)po, (char*)pd, l, out lFinal, (char)f, clean);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, ushort* po, ushort* pd, int l, out int lFinal, ushort f, bool clean)
        {
            return Remove(alwaysCopy, (char*)po, (char*)pd, l, out lFinal, (char)f, clean);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, int* po, int* pd, int l, out int lFinal, int f, bool clean)
        {
            if (l < 2)
            {
                if (l < 1)
                {
                    lFinal = 0;
                    return 0;
                }

                else if (*po == f)
                {
                    lFinal = 0;
                    if (clean)
                        *pd = 0;
                    return 1;
                }
                else if (alwaysCopy)
                    *pd = *po;
                lFinal = 1;
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int tot = 0, ipo = 0, a = 0, tc;
            lFinal = 0;
            do
            {
                if (po[a] == f)
                {
                    tot++;
                    tc = a - ipo;
                    if (tc != 0)
                    {
                        General.Copy(po + ipo, pd + lFinal, tc);
                        lFinal += tc;
                    }
                    ipo = a + 1;
                }
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, uint* po, uint* pd, int l, out int lFinal, uint f, bool clean)
        {
            return Remove(alwaysCopy, (int*)po, (int*)pd, l, out lFinal, (int)f, clean);
        }


        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, long* po, long* pd, int l, out int lFinal, long f, bool clean)
        {
            if (l < 2)
            {
                if (l < 1)
                {
                    lFinal = 0;
                    return 0;
                }
                else if (*po == f)
                {
                    lFinal = 0;
                    if (clean)
                        *pd = 0;
                    return 1;
                }
                else if (alwaysCopy)
                    *pd = *po;
                lFinal = 1;
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int tot = 0, ipo = 0, a = 0, tc;
            lFinal = 0;
            do
            {
                if (po[a] == f)
                {
                    tot++;
                    tc = a - ipo;
                    if (tc != 0)
                    {
                        General.Copy(po + ipo, pd + lFinal, tc);
                        lFinal += tc;
                    }
                    ipo = a + 1;
                }
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }


        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(bool alwaysCopy, ulong* po, ulong* pd, int l, out int lFinal, ulong f, bool clean)
        {
            return Remove(alwaysCopy, (long*)po, (long*)pd, l, out lFinal, (long)f, clean);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, byte* po, byte* pd, int l, out int lFinal, byte* pf, int lf, bool clean, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Remove(alwaysCopy, po, pd, l, out lFinal, *pf, clean);
                if (po == pd)
                {
                    lFinal = l;
                }
                else if (alwaysCopy && l > 0)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                    lFinal = 0;
                return 0;
            }

            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int tot = 0;
            int ipo = 0;
            int tc;
            lFinal = 0;
            int a = 0;
            int b;
            byte c;

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            do
            {
                b = 0;
                c = po[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        tc = a - ipo;
                        if (tc != 0)
                        {
                            General.Copy(po + ipo, pd + lFinal, tc);
                            lFinal += tc;
                        }
                        ipo = a + 1;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

                /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, sbyte* po, sbyte* pd, int l, out int lFinal, sbyte* pf, int lf, bool clean, bool noCopStack)
        {
            return RemoveAny(alwaysCopy, (byte*)po, (byte*)pd, l, out lFinal, (byte*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, char* po, char* pd, int l, out int lFinal, char* pf, int lf, bool clean, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Remove(alwaysCopy, po, pd, l, out lFinal, *pf, clean);
                if (po == pd)
                {
                    lFinal = l;
                }
                else if (alwaysCopy && l > 0)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                    lFinal = 0;
                return 0;
            }

            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int tot = 0;
            int ipo = 0;
            int tc;
            lFinal = 0;
            int a = 0;
            int b;
            char c;

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            do
            {
                b = 0;
                c = po[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        tc = a - ipo;
                        if (tc != 0)
                        {
                            General.Copy(po + ipo, pd + lFinal, tc);
                            lFinal += tc;
                        }
                        ipo = a + 1;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, short* po, short* pd, int l, out int lFinal, short* pf, int lf, bool clean, bool noCopStack)
        {
            return RemoveAny(alwaysCopy, (char*)po, (char*)pd, l, out lFinal, (char*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, ushort* po, ushort* pd, int l, out int lFinal, ushort* pf, int lf, bool clean, bool noCopStack)
        {
            return RemoveAny(alwaysCopy, (char*)po, (char*)pd, l, out lFinal, (char*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, int* po, int* pd, int l, out int lFinal, int* pf, int lf, bool clean, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Remove(alwaysCopy, po, pd, l, out lFinal, *pf, clean);
                if (po == pd)
                {
                    lFinal = l;
                }
                else if (alwaysCopy && l > 0)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                    lFinal = 0;
                return 0;
            }

            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int tot = 0;
            int ipo = 0;
            int tc;
            lFinal = 0;
            int a = 0;
            int b;
            int c;

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackInts && &lf > General.MinimumStack)
            {
                int* pcops = stackalloc int[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            do
            {
                b = 0;
                c = po[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        tc = a - ipo;
                        if (tc != 0)
                        {
                            General.Copy(po + ipo, pd + lFinal, tc);
                            lFinal += tc;
                        }
                        ipo = a + 1;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, uint* po, uint* pd, int l, out int lFinal, uint* pf, int lf, bool clean, bool noCopStack)
        {
            return RemoveAny(alwaysCopy, (int*)po, (int*)pd, l, out lFinal, (int*)pf, lf, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, long* po, long* pd, int l, out int lFinal, long* pf, int lf, bool clean, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Remove(alwaysCopy, po, pd, l, out lFinal, *pf, clean);
                if (po == pd)
                {
                    lFinal = l;
                }
                else if (alwaysCopy && l > 0)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                    lFinal = 0;
                return 0;
            }

            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int tot = 0;
            int ipo = 0;
            int tc;
            lFinal = 0;
            int a = 0;
            int b;
            long c;

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackLongs && &lf > General.MinimumStack)
            {
                long* pcops = stackalloc long[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            do
            {
                b = 0;
                c = po[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        tc = a - ipo;
                        if (tc != 0)
                        {
                            General.Copy(po + ipo, pd + lFinal, tc);
                            lFinal += tc;
                        }
                        ipo = a + 1;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(bool alwaysCopy, ulong* po, ulong* pd, int l, out int lFinal, ulong* pf, int lf, bool clean, bool noCopStack)
        {
            return RemoveAny(alwaysCopy, (long*)po, (long*)pd, l, out lFinal, (long*)pf, lf, clean, noCopStack);
        }


        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, byte* po, int l, byte* pd, bool clean)
        {
            if (tot < 1)
            {
                return l;
            }
            if (l < tot)
            {
                throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE);
            }
            int lFinal = l - tot;
            if (lFinal == 0)
            {
                if (clean)
                    General.Clean(pd, l);
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            int a = 0, ipo = 0, tc, difipo;
            lFinal = 0;
            do
            {
                tc = pCont[a];
                if (tc != ipo)
                {
                    difipo = tc - ipo;
                    General.Copy(po + ipo, pd + lFinal, difipo);
                    lFinal += difipo;
                }
                ipo = tc + 1;
            } while (++a != tot);
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean)
                General.Clean(pd + lFinal, l - lFinal);
            return lFinal;
        }

                /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, sbyte* po, int l, sbyte* pd, bool clean) 
        {
            return RemoveLocations(pCont, tot, (byte*)po, l, (byte*)pd, clean);
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, char* po, int l, char* pd, bool clean)
        {
            if (tot < 1)
            {
                return l;
            }
            if (l < tot)
            {
                throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE);
            }
            int lFinal = l - tot;
            if (lFinal == 0)
            {
                if (clean)
                    General.Clean(pd, l);
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            int a = 0, ipo = 0, tc, difipo;
            lFinal = 0;
            do
            {
                tc = pCont[a];
                if (tc != ipo)
                {
                    difipo = tc - ipo;
                    General.Copy(po + ipo, pd + lFinal, difipo);
                    lFinal += difipo;
                }
                ipo = tc + 1;
            } while (++a != tot);
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean)
                General.Clean(pd + lFinal, l - lFinal);
            return lFinal;
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, short* po, int l, short* pd, bool clean)
        {
            return RemoveLocations(pCont, tot, (char*)po, l, (char*)pd, clean);
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, ushort* po, int l, ushort* pd, bool clean)
        {
            return RemoveLocations(pCont, tot, (char*)po, l, (char*)pd, clean);
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, int* po, int l, int* pd, bool clean)
        {
            if (tot < 1)
            {
                return l;
            }
            if (l < tot)
            {
                throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE);
            }
            int lFinal = l - tot;
            if (lFinal == 0)
            {
                if (clean)
                    General.Clean(pd, l);
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            int a = 0, ipo = 0, tc, difipo;
            lFinal = 0;
            do
            {
                tc = pCont[a];
                if (tc != ipo)
                {
                    difipo = tc - ipo;
                    General.Copy(po + ipo, pd + lFinal, difipo);
                    lFinal += difipo;
                }
                ipo = tc + 1;
            } while (++a != tot);
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean)
                General.Clean(pd + lFinal, l - lFinal);
            return lFinal;
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, uint* po, int l, uint* pd, bool clean)
        {
            return RemoveLocations(pCont, tot, (int*)po, l, (int*)pd, clean);
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, long* po, int l, long* pd, bool clean)
        {
            if (tot < 1)
            {
                return l;
            }
            if (l < tot)
            {
                throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE);
            }
            int lFinal = l - tot;
            if (lFinal == 0)
            {
                if (clean)
                    General.Clean(pd, l);
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            int a = 0, ipo = 0, tc, difipo;
            lFinal = 0;
            do
            {
                tc = pCont[a];
                if (tc != ipo)
                {
                    difipo = tc - ipo;
                    General.Copy(po + ipo, pd + lFinal, difipo);
                    lFinal += difipo;
                }
                ipo = tc + 1;
            } while (++a != tot);
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean)
                General.Clean(pd + lFinal, l - lFinal);
            return lFinal;
        }

        /// <summary>
        /// Copia memoria eliminando elementos
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser nicas y ascendentes (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser igual o mayor a tot.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l - tot).
        /// </returns>
        public static int RemoveLocations(int* pCont, int tot, ulong* po, int l, ulong* pd, bool clean)
        {
            return RemoveLocations(pCont, tot, (long*)po, l, (long*)pd, clean);
        }

        #endregion

        #region Exist

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(byte* p, int l, byte f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(sbyte* p, int l, sbyte f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(char* p, int l, char f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(short* p, int l, short f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(ushort* p, int l, ushort f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(int* p, int l, int f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(uint* p, int l, uint f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(long* p, int l, long f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        /// <summary>
        /// Verifica la existencia de un elemento en el puntero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(ulong* p, int l, ulong f)
        {
            int a = 0;
            while (a < l) { if (p[a] == f) return true; a++; }
            return false;
        }

        
        #endregion

        #region Reservar Locations

        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, byte* po, int l, byte* pd, int lf, int lr, bool clean)
        {
            if (tot < 1)
                return l;
            if (l < 1)
                throw new ArgumentOutOfRangeException("l",l,Errors.MINOR_ONE);
            Mem.Heap heap = null;
            if (lr == lf)
            {
                if (po == pd)
                    return l;
                if (lr < 17 || (pd > po && pd - po < l))
                {
                    General.Move(po, pd, l);
                    return l;
                }
            }
            else if (lr > lf)
            {
                if (pd == po || (pd > po && pd - po < l) || (pd < po && po - pd < (tot * (lr - lf) + l)))
                {
                    if (l < General.MaxStackBytes && &lf>General.MinimumStack)
                    {
                        byte* ptmp = stackalloc byte[l];
                        General.Copy(po, ptmp, l);
                        po = ptmp;
                    }
                    else
                    {
                        heap = new Heap(l, false);
                        General.Copy(po, (byte*)heap.Pointer, l);
                        po = (byte*)heap.Pointer;
                    }
                }
            }
            else if (pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int a = 0, ipo = 0, tc, difipo;
            int lFinal = 0;
            if (lr != lf)
            {
                do
                {
                    tc = pCont[a];
                    if (tc != ipo)
                    {
                        difipo = tc - ipo;
                        General.Copy(po + ipo, pd + lFinal, difipo);
                        lFinal += difipo;
                    }
                    pCont[a] = lFinal;
                    lFinal += lr;
                    ipo = tc + lf;
                } while (++a != tot);
            }
            else
            {
                do
                {
                    tc = pCont[a];
                    if (tc != ipo)
                    {
                        difipo = tc - ipo;
                        General.Copy(po + ipo, pd + ipo, difipo);
                    }
                    ipo = tc + lr;
                } while (++a != tot);
                lFinal = ipo;
            }
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean && l > lFinal)
                General.Clean(pd + lFinal, l - lFinal);
            if (heap != null)
                heap.Dispose();
            return lFinal;
        }

        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, sbyte* po, int l, sbyte* pd, int lf, int lr, bool clean)
        {
            return ReserveLocations(pCont, tot, (byte*)po, l, (byte*)pd, lf, lr, clean);
        }

        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, char* po, int l, char* pd, int lf, int lr, bool clean)
        {
            if (tot < 1)
                return l;
            if (l < 1)
                throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE);
            Mem.Heap heap = null;
            if (lr == lf)
            {
                if (po == pd)
                    return l;
                if (lr < 17 || (pd > po && pd - po < l))
                {
                    General.Move(po, pd, l);
                    return l;
                }
            }
            else if (lr > lf)
            {
                if (pd == po || (pd > po && pd - po < l) || (pd < po && po - pd < (tot * (lr - lf) + l)))
                {
                    if (l < General.MaxStackChars && &lf > General.MinimumStack)
                    {
                        char* ptmp = stackalloc char[l];
                        General.Copy(po, ptmp, l);
                        po = ptmp;
                    }
                    else
                    {
                        heap = new Heap(l<<1, false);
                        General.Copy(po, (char*)heap.Pointer, l);
                        po = (char*)heap.Pointer;
                    }
                }
            }
            else if (pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int a = 0, ipo = 0, tc, difipo;
            int lFinal = 0;
            if (lr != lf)
            {
                do
                {
                    tc = pCont[a];
                    if (tc != ipo)
                    {
                        difipo = tc - ipo;
                        General.Copy(po + ipo, pd + lFinal, difipo);
                        lFinal += difipo;
                    }
                    pCont[a] = lFinal;
                    lFinal += lr;
                    ipo = tc + lf;
                } while (++a != tot);
            }
            else
            {
                do
                {
                    tc = pCont[a];
                    if (tc != ipo)
                    {
                        difipo = tc - ipo;
                        General.Copy(po + ipo, pd + ipo, difipo);
                    }
                    ipo = tc + lr;
                } while (++a != tot);
                lFinal = ipo;
            }
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean && l > lFinal)
                General.Clean(pd + lFinal, l - lFinal);
            if (heap != null)
                heap.Dispose();
            return lFinal;
        }

        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, short* po, int l, short* pd, int lf, int lr, bool clean)
        {
            return ReserveLocations(pCont, tot, (char*)po, l, (char*)pd, lf, lr, clean);
        }

        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, ushort* po, int l, ushort* pd, int lf, int lr, bool clean)
        {
            return ReserveLocations(pCont, tot, (char*)po, l, (char*)pd, lf, lr, clean);
        }

        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, int* po, int l, int* pd, int lf, int lr, bool clean)
        {
            if (tot < 1)
                return l;
            if (l < 1)
                throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE);
            Mem.Heap heap = null;
            if (lr == lf)
            {
                if (po == pd)
                    return l;
                if (lr < 17 || (pd > po && pd - po < l))
                {
                    General.Move(po, pd, l);
                    return l;
                }
            }
            else if (lr > lf)
            {
                if (pd == po || (pd > po && pd - po < l) || (pd < po && po - pd < (tot * (lr - lf) + l)))
                {
                    if (l < General.MaxStackInts && &lf > General.MinimumStack)
                    {
                        int* ptmp = stackalloc int[l];
                        General.Copy(po, ptmp, l);
                        po = ptmp;
                    }
                    else
                    {
                        heap = new Heap(l<<2, false);
                        General.Copy(po, (int*)heap.Pointer, l);
                        po = (int*)heap.Pointer;
                    }
                }
            }
            else if (pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int a = 0, ipo = 0, tc, difipo;
            int lFinal = 0;
            if (lr != lf)
            {
                do
                {
                    tc = pCont[a];
                    if (tc != ipo)
                    {
                        difipo = tc - ipo;
                        General.Copy(po + ipo, pd + lFinal, difipo);
                        lFinal += difipo;
                    }
                    pCont[a] = lFinal;
                    lFinal += lr;
                    ipo = tc + lf;
                } while (++a != tot);
            }
            else
            {
                do
                {
                    tc = pCont[a];
                    if (tc != ipo)
                    {
                        difipo = tc - ipo;
                        General.Copy(po + ipo, pd + ipo, difipo);
                    }
                    ipo = tc + lr;
                } while (++a != tot);
                lFinal = ipo;
            }
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean && l > lFinal)
                General.Clean(pd + lFinal, l - lFinal);
            if (heap != null)
                heap.Dispose();
            return lFinal;
        }

        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, uint* po, int l, uint* pd, int lf, int lr, bool clean)
        {
            return ReserveLocations(pCont, tot, (int*)po, l, (int*)pd, lf, lr, clean);
        }


        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, long* po, int l, long* pd, int lf, int lr, bool clean)
        {
            if (tot < 1)
                return l;
            if (l < 1)
                throw new ArgumentOutOfRangeException("l", l, Errors.MINOR_ONE);
            Mem.Heap heap = null;
            if (lr == lf)
            {
                if (po == pd)
                    return l;
                if (lr < 17 || (pd > po && pd - po < l))
                {
                    General.Move(po, pd, l);
                    return l;
                }
            }
            else if (lr > lf)
            {
                if (pd == po || (pd > po && pd - po < l) || (pd < po && po - pd < (tot * (lr - lf) + l)))
                {
                    if (l < General.MaxStackLongs && &lf > General.MinimumStack)
                    {
                        long* ptmp = stackalloc long[l];
                        General.Copy(po, ptmp, l);
                        po = ptmp;
                    }
                    else
                    {
                        heap = new Heap(l<<3, false);
                        General.Copy(po, (long*)heap.Pointer, l);
                        po = (long*)heap.Pointer;
                    }
                }
            }
            else if (pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }

            int a = 0, ipo = 0, tc, difipo;
            int lFinal = 0;
            if (lr != lf)
            {
                do
                {
                    tc = pCont[a];
                    if (tc != ipo)
                    {
                        difipo = tc - ipo;
                        General.Copy(po + ipo, pd + lFinal, difipo);
                        lFinal += difipo;
                    }
                    pCont[a] = lFinal;
                    lFinal += lr;
                    ipo = tc + lf;
                } while (++a != tot);
            }
            else
            {
                do
                {
                    tc = pCont[a];
                    if (tc != ipo)
                    {
                        difipo = tc - ipo;
                        General.Copy(po + ipo, pd + ipo, difipo);
                    }
                    ipo = tc + lr;
                } while (++a != tot);
                lFinal = ipo;
            }
            tc = l - ipo;
            if (tc != 0)
            {
                General.Copy(po + ipo, pd + lFinal, tc);
                lFinal += tc;
            }
            if (clean && l > lFinal)
                General.Clean(pd + lFinal, l - lFinal);
            if (heap != null)
                heap.Dispose();
            return lFinal;
        }

        /// <summary>
        /// Copia memoria reservando espacio para ubicaciones de reemplazo
        /// indicados en el puntero de ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// Si lr es diferente a lf, el puntero de ubicaciones es modificado con las ubicaciones finales.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de insercin (salto sobre destino).
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReserveLocations(int* pCont, int tot, ulong* po, int l, ulong* pd, int lf, int lr, bool clean)
        {
            return ReserveLocations(pCont, tot, (long*)po, l, (long*)pd, lf, lr, clean);
        }


        #endregion

        #region Replace

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(byte* p, int l, byte f, byte r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }
        
        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(sbyte* p, int l, sbyte f, sbyte r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(char* p, int l, char f, char r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(short* p, int l, short f, short r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(ushort* p, int l, ushort f, ushort r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(int* p, int l, int f, int r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(uint* p, int l, uint f, uint r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(long* p, int l, long f, long r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(ulong* p, int l, ulong f, ulong r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                if (p[a] == f)
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;
        }


        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, byte* po, byte* pd, int l, byte f, byte r)
        {

            if (pd == po)
                return Replace(pd, l, f, r);
            if (l < 1)
                return 0;

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return Replace(pd, l, f, r);
            }

            int ubi = IndexOf(po, l, f);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            return Replace(pd + ubi, l - ubi, f, r) + 1;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, sbyte* po, sbyte* pd, int l, sbyte f, sbyte r)
        {
            return Replace(alwaysCopy, (byte*)po, (byte*)pd, l, (byte)f, (byte)r);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, char* po, char* pd, int l, char f, char r)
        {

            if (pd == po)
                return Replace(pd, l, f, r);
            if (l < 1)
                return 0;

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return Replace(pd, l, f, r);
            }

            int ubi = IndexOf(po, l, f);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            return Replace(pd + ubi, l - ubi, f, r) + 1;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, short* po, short* pd, int l, short f, short r)
        {
            return Replace(alwaysCopy, (char*)po, (char*)pd, l, (char)f, (char)r);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, ushort* po, ushort* pd, int l, ushort f, ushort r)
        {
            return Replace(alwaysCopy, (char*)po, (char*)pd, l, (char)f, (char)r);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, int* po, int* pd, int l, int f, int r)
        {

            if (pd == po)
                return Replace(pd, l, f, r);
            if (l < 1)
                return 0;

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return Replace(pd, l, f, r);
            }

            int ubi = IndexOf(po, l, f);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            return Replace(pd + ubi, l - ubi, f, r) + 1;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, uint* po, uint* pd, int l, uint f, uint r)
        {
            return Replace(alwaysCopy, (int*)po, (int*)pd, l, (int)f, (int)r);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, long* po, long* pd, int l, long f, long r)
        {

            if (pd == po)
                return Replace(pd, l, f, r);
            if (l < 1)
                return 0;

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return Replace(pd, l, f, r);
            }

            int ubi = IndexOf(po, l, f);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            return Replace(pd + ubi, l - ubi, f, r) + 1;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="f">Elemento a buscar.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(bool alwaysCopy, ulong* po, ulong* pd, int l, ulong f, ulong r)
        {
            return Replace(alwaysCopy, (long*)po, (long*)pd, l, (long)f, (long)r);
        }

        #endregion

        #region Replace Equal

        /// <summary>
        /// Reemplaza cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(byte* p, byte* pc, int l, byte* pf, byte* pr, int lfr, bool noCopStack)
        {
            if (lfr < 1 || lfr >= l)
            {
                if (lfr == l)
                {
                    if (pc == null)
                        pc = p;
                    if (l == 1)
                    {
                        if (*pc == *pf)
                        {
                            *p = *pr;
                            return 1;
                        }
                    }
                    else if (lfr > 1 && Comp.Equal(pc, pf, l))
                    {
                        General.Copy(pr, p, l);
                        return 1;
                    }
                }
                return 0;
            }

            if (pc == null)
                pc = p;

            int tot = 0, a = 0;
            if (lfr == 1)
            {
                byte f = *pf;
                byte r = *pr;
                do
                {
                    if (pc[a] == f)
                    {
                        tot++;
                        p[a] = r;
                    }
                } while (++a != l);
                return tot;
            }

            if (!noCopStack && l - lfr > 127 && lfr < General.MaxStackBytes && &lfr>General.MinimumStack)
            {
                if (pf > General.MaximumStack)
                {
                    byte* pcops = stackalloc byte[lfr];
                    General.Copy(pf, pcops, lfr);
                    pf = pcops;
                }
                if (pr > General.MaximumStack && l - lfr > 1023)
                {
                    byte* pcops2 = stackalloc byte[lfr];
                    General.Copy(pr, pcops2, lfr);
                    pr = pcops2;
                }
            }
            int ubi;
otra:
            ubi = IndexOf(pc + a, l - a, pf, lfr, true);
            if (ubi != -1)
            {
                tot++;
                a += ubi;
                General.Copy(pr, p + a, lfr);
                a += lfr;
                if (l - a >= lfr)
                    goto otra;
            }
            return tot;
        }

        /// <summary>
        /// Reemplaza cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(sbyte* p, sbyte* pc, int l, sbyte* pf, sbyte* pr, int lfr, bool noCopStack)
        {
            return ReplaceEqual((byte*)p, (byte*)pc, l, (byte*)pf, (byte*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Reemplaza cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(char* p, char* pc, int l, char* pf, char* pr, int lfr, bool noCopStack)
        {
            if (lfr < 1 || lfr >= l)
            {
                if (lfr == l)
                {
                    if (pc == null)
                        pc = p;
                    if (l == 1)
                    {
                        if (*pc == *pf)
                        {
                            *p = *pr;
                            return 1;
                        }
                    }
                    else if (lfr > 1 && Comp.Equal(pc, pf, l))
                    {
                        General.Copy(pr, p, l);
                        return 1;
                    }
                }
                return 0;
            }

            if (pc == null)
                pc = p;

            int tot = 0, a = 0;
            if (lfr == 1)
            {
                char f = *pf;
                char r = *pr;
                do
                {
                    if (pc[a] == f)
                    {
                        tot++;
                        p[a] = r;
                    }
                } while (++a != l);
                return tot;
            }

            if (!noCopStack && l - lfr > 127 && lfr < General.MaxStackChars && &lfr > General.MinimumStack)
            {
                if (pf > General.MaximumStack)
                {
                    char* pcops = stackalloc char[lfr];
                    General.Copy(pf, pcops, lfr);
                    pf = pcops;
                }
                if (pr > General.MaximumStack && l - lfr > 1023)
                {
                    char* pcops2 = stackalloc char[lfr];
                    General.Copy(pr, pcops2, lfr);
                    pr = pcops2;
                }
            }
            int ubi;
otra:
            ubi = IndexOf(pc + a, l - a, pf, lfr, true);
            if (ubi != -1)
            {
                tot++;
                a += ubi;
                General.Copy(pr, p + a, lfr);
                a += lfr;
                if (l - a >= lfr)
                    goto otra;
            }
            return tot;
        }

        /// <summary>
        /// Reemplaza cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(short* p, short* pc, int l, short* pf, short* pr, int lfr, bool noCopStack)
        {
            return ReplaceEqual((char*)p, (char*)pc, l, (char*)pf, (char*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Reemplaza cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(ushort* p, ushort* pc, int l, ushort* pf, ushort* pr, int lfr, bool noCopStack)
        {
            return ReplaceEqual((char*)p, (char*)pc, l, (char*)pf, (char*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd, siendo diferente a po: Se lanza una excepcin (no es permitido).
        /// En el resto de las situaciones tambin deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(bool alwaysCopy, byte* po, byte* pc, byte* pd, int l, byte* pf, byte* pr, int lfr, bool noCopStack)
        {
            if (pd == po)
                return ReplaceEqual(pd, pc, l, pf, pr, lfr, noCopStack);

            if (lfr < 1 || lfr >= l)
            {
                if (lfr == l)
                {
                    if (pc == null)
                        pc = po;

                    if (l == 1)
                    {
                        if (*pc == *pf)
                        {
                            *pd = *pr;
                            return 1;
                        }
                    }
                    else if (lfr > 1 && Comp.Equal(pc, pf, l))
                    {
                        General.Copy(pr, pd, l);
                        return 1;
                    }
                }
                if (alwaysCopy && l > 0)
                    General.Move(po, pd, l);
                return 0;
            }

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD,"pc,pd");
                General.Move(po, pd, l);
                return ReplaceEqual(pd, pc, l, pf, pr, lfr, noCopStack);
            }

            if (pc == null)
                pc = po;
            else if (pc == pd)
                throw new ArgumentException(Errors.PC_EQUAL_PD, "pc,pd");

            int ubi;
            if (lfr == 1)
            {
                ubi = IndexOf(pc, l, *pf);
                if (ubi == -1)
                    return 0;
                General.Copy(po, pd, l);
                pd[ubi] = *pr;
                if (++ubi == l)
                    return 1;
                if (pc == po)
                {
                    pd += ubi;
                    return ReplaceEqual(pd, pd, l - ubi, pf, pr, lfr, true) + 1;
                }
                return ReplaceEqual(pd + ubi, pc + ubi, l - ubi, pf, pr, lfr, true) + 1;
            }
            if (!noCopStack)
            {
                if (pf > General.MaximumStack && l - lfr > 127 && lfr < General.MaxStackBytes && &lfr > General.MinimumStack)
                {
                    byte* pcops = stackalloc byte[lfr];
                    General.Copy(pf, pcops, lfr);
                    pf = pcops;
                }
                else
                    noCopStack = true;
            }

            ubi = IndexOf(pc, l, pf, lfr,true);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);

            if ((ubi += lfr) == l)
            {
                General.Copy(pr, pd + ubi, lfr);
                return 1;
            }
            if (!noCopStack && l - lfr > 1023)
            {
                byte* pcops2 = stackalloc byte[lfr];
                General.Copy(pr, pcops2, lfr);
                pr = pcops2;
            }
            General.Copy(pr, pd + ubi, lfr);
            if (pc == po)
            {
                pd += ubi;
                return ReplaceEqual(pd, pd, l - ubi, pf, pr, lfr,true) + 1;
            }
            return ReplaceEqual(pd + ubi, pc + ubi, l - ubi, pf, pr, lfr,true) + 1;
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd, siendo diferente a po: Se lanza una excepcin (no es permitido).
        /// En el resto de las situaciones tambin deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(bool alwaysCopy, sbyte* po, sbyte* pc, sbyte* pd, int l, sbyte* pf, sbyte* pr, int lfr, bool noCopStack)
        {
            return ReplaceEqual(alwaysCopy, (byte*)po, (byte*)pc, (byte*)pd, l, (byte*)pf, (byte*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd, siendo diferente a po: Se lanza una excepcin (no es permitido).
        /// En el resto de las situaciones tambin deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(bool alwaysCopy, char* po, char* pc, char* pd, int l, char* pf, char* pr, int lfr, bool noCopStack)
        {
            if (pd == po)
                return ReplaceEqual(pd, pc, l, pf, pr, lfr, noCopStack);

            if (lfr < 1 || lfr >= l)
            {
                if (lfr == l)
                {
                    if (pc == null)
                        pc = po;

                    if (l == 1)
                    {
                        if (*pc == *pf)
                        {
                            *pd = *pr;
                            return 1;
                        }
                    }
                    else if (lfr > 1 && Comp.Equal(pc, pf, l))
                    {
                        General.Copy(pr, pd, l);
                        return 1;
                    }
                }
                if (alwaysCopy && l > 0)
                    General.Move(po, pd, l);
                return 0;
            }

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD, "pc,pd");
                General.Move(po, pd, l);
                return ReplaceEqual(pd, pc, l, pf, pr, lfr, noCopStack);
            }

            if (pc == null)
                pc = po;
            else if (pc == pd)
                throw new ArgumentException(Errors.PC_EQUAL_PD, "pc,pd");

            int ubi;
            if (lfr == 1)
            {
                ubi = IndexOf(pc, l, *pf);
                if (ubi == -1)
                    return 0;
                General.Copy(po, pd, l);
                pd[ubi] = *pr;
                if (++ubi == l)
                    return 1;
                if (pc == po)
                {
                    pd += ubi;
                    return ReplaceEqual(pd, pd, l - ubi, pf, pr, lfr, true) + 1;
                }
                return ReplaceEqual(pd + ubi, pc + ubi, l - ubi, pf, pr, lfr, true) + 1;
            }
            if (!noCopStack)
            {
                if (pf > General.MaximumStack && l - lfr > 127 && lfr < General.MaxStackChars && &lfr > General.MinimumStack)
                {
                    char* pcops = stackalloc char[lfr];
                    General.Copy(pf, pcops, lfr);
                    pf = pcops;
                }
                else
                    noCopStack = true;
            }

            ubi = IndexOf(pc, l, pf, lfr, true);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);

            if ((ubi += lfr) == l)
            {
                General.Copy(pr, pd + ubi, lfr);
                return 1;
            }
            if (!noCopStack && l - lfr > 1023)
            {
                char* pcops2 = stackalloc char[lfr];
                General.Copy(pr, pcops2, lfr);
                pr = pcops2;
            }
            General.Copy(pr, pd + ubi, lfr);
            if (pc == po)
            {
                pd += ubi;
                return ReplaceEqual(pd, pd, l - ubi, pf, pr, lfr, true) + 1;
            }
            return ReplaceEqual(pd + ubi, pc + ubi, l - ubi, pf, pr, lfr, true) + 1;
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd, siendo diferente a po: Se lanza una excepcin (no es permitido).
        /// En el resto de las situaciones tambin deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(bool alwaysCopy, short* po, short* pc, short* pd, int l, short* pf, short* pr, int lfr, bool noCopStack)
        {
            return ReplaceEqual(alwaysCopy, (char*)po, (char*)pc, (char*)pd, l, (char*)pf, (char*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd, siendo diferente a po: Se lanza una excepcin (no es permitido).
        /// En el resto de las situaciones tambin deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceEqual(bool alwaysCopy, ushort* po, ushort* pc, ushort* pd, int l, ushort* pf, ushort* pr, int lfr, bool noCopStack)
        {
            return ReplaceEqual(alwaysCopy, (char*)po, (char*)pc, (char*)pd, l, (char*)pf, (char*)pr, lfr, noCopStack);
        }

        #endregion

        #region ReplaceAny

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, byte* po, byte* pd, int l, byte* pf, int lf, byte r, bool noCopStack)
        {
            if (pd == po)
                return ReplaceAny(pd, l, pf, lf, r, noCopStack);

            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Replace(alwaysCopy, po, pd, l, *pf, r);
                if (alwaysCopy && l > 0)
                    General.Move(po, pd, l);
                return 0;
            }
            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return ReplaceAny(pd, l, pf, lf, r, noCopStack);
            }

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int ubi = IndexOfAny(po, l, pf, lf,true);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            pd += ubi;
            return ReplaceAny(pd, l - ubi, pf, lf, r,true) + 1;

        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, byte* po, byte* pc, byte* pd, int l, byte* pf, int lf, byte r, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                {
                    byte tmp=*pf;
                    return ReplaceEqual(alwaysCopy, po, pc, pd, l, &tmp, &r, 1, true);
                }
                if (alwaysCopy && l > 0)
                    General.Move(po, pd, l);
                return 0;
            }
            if (alwaysCopy || (pd > po && pd - po < l))
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD, "pc,pd");
                General.Move(po, pd, l);
                po = pd;
            }
            else if (pc == null)
                pc = po;

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }
            
            int ubi = IndexOfAny(pc, l, pf, lf,true);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            int tot=0;
            do
            {
                tot++;
                pd[ubi] = r;
                if (++ubi == l)
                    return tot;
                pd += ubi;
                ubi = IndexOfAny(pc+=ubi, l-=ubi, pf, lf, true);
            }while(ubi != -1);
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, sbyte* po, sbyte* pc, sbyte* pd, int l, sbyte* pf, int lf, sbyte r, bool noCopStack)
        {
            return ReplaceAny(alwaysCopy,(byte*)po,(byte*)pc,(byte*)pd,l,(byte*)pf,lf,(byte)r,noCopStack);
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(byte* p, int l, byte* pf, int lf, byte r, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Replace(p, l, *pf, r);
                return 0;
            }
            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackBytes && &lf > General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int tot = 0;
            int a = 0;
            int b;
            byte c;

            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        p[a] = r;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, sbyte* po, sbyte* pd, int l, sbyte* pf, int lf, sbyte r, bool noCopStack)
        {
            return ReplaceAny(alwaysCopy, (byte*)po, (byte*)pd, l, (byte*)pf, lf, (byte)r, noCopStack);
        }


        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(sbyte* p, int l, sbyte* pf, int lf, sbyte r, bool noCopStack)
        {
            return ReplaceAny((byte*)p, l, (byte*)pf, lf, (byte)r, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, char* po, char* pd, int l, char* pf, int lf, char r, bool noCopStack)
        {
            if (pd == po)
                return ReplaceAny(pd, l, pf, lf, r, noCopStack);

            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Replace(alwaysCopy, po, pd, l, *pf, r);
                if (alwaysCopy && l > 0)
                    General.Move(po, pd, l);
                return 0;
            }
            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return ReplaceAny(pd, l, pf, lf, r, noCopStack);
            }

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int ubi = IndexOfAny(po, l, pf, lf, true);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            pd += ubi;
            return ReplaceAny(pd, l - ubi, pf, lf, r, true) + 1;

        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, char* po, char* pc, char* pd, int l, char* pf, int lf, char r, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                {
                    char tmp=*pf;
                    return ReplaceEqual(alwaysCopy, po, pc, pd, l, &tmp, &r, 1, true);
                }
                if (alwaysCopy && l > 0)
                    General.Move(po, pd, l);
                return 0;
            }
            if (alwaysCopy || (pd > po && pd - po < l))
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD, "pc,pd");
                General.Move(po, pd, l);
                po = pd;
            }
            else if (pc == null)
                pc = po;

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf>General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }
            
            int ubi = IndexOfAny(pc, l, pf, lf,true);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            int tot=0;
            do
            {
                tot++;
                pd[ubi] = r;
                if (++ubi == l)
                    return tot;
                pd += ubi;
                ubi = IndexOfAny(pc+=ubi, l-=ubi, pf, lf, true);
            }while(ubi != -1);
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, short* po, short* pc, short* pd, int l, short* pf, int lf, short r, bool noCopStack)
        {
            return ReplaceAny(alwaysCopy,(char*)po,(char*)pc,(char*)pd,l,(char*)pf,lf,(char)r,noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, ushort* po, ushort* pc, ushort* pd, int l, ushort* pf, int lf, ushort r, bool noCopStack)
        {
            return ReplaceAny(alwaysCopy,(char*)po,(char*)pc,(char*)pd,l,(char*)pf,lf,(char)r,noCopStack);
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(char* p, int l, char* pf, int lf, char r, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Replace(p, l, *pf, r);
                return 0;
            }
            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                char* pcops = stackalloc char[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int tot = 0;
            int a = 0;
            int b;
            char c;

            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        p[a] = r;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, short* po, short* pd, int l, short* pf, int lf, short r, bool noCopStack)
        {
            return ReplaceAny(alwaysCopy, (char*)po, (char*)pd, l, (char*)pf, lf, (char)r, noCopStack);
        }


        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(short* p, int l, short* pf, int lf, short r, bool noCopStack)
        {
            return ReplaceAny((char*)p, l, (char*)pf, lf, (char)r, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, ushort* po, ushort* pd, int l, ushort* pf, int lf, ushort r, bool noCopStack)
        {
            return ReplaceAny(alwaysCopy, (char*)po, (char*)pd, l, (char*)pf, lf, (char)r, noCopStack);
        }


        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(ushort* p, int l, ushort* pf, int lf, ushort r, bool noCopStack)
        {
            return ReplaceAny((char*)p, l, (char*)pf, lf, (char)r, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, int* po, int* pd, int l, int* pf, int lf, int r, bool noCopStack)
        {
            if (pd == po)
                return ReplaceAny(pd, l, pf, lf, r, noCopStack);

            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Replace(alwaysCopy, po, pd, l, *pf, r);
                if (alwaysCopy && l > 0)
                    General.Move(po, pd, l);
                return 0;
            }
            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return ReplaceAny(pd, l, pf, lf, r, noCopStack);
            }

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackInts && &lf > General.MinimumStack)
            {
                int* pcops = stackalloc int[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int ubi = IndexOfAny(po, l, pf, lf, true);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            pd += ubi;
            return ReplaceAny(pd, l - ubi, pf, lf, r, true) + 1;

        }


        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(int* p, int l, int* pf, int lf, int r, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Replace(p, l, *pf, r);
                return 0;
            }
            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackInts && &lf > General.MinimumStack)
            {
                int* pcops = stackalloc int[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int tot = 0;
            int a = 0;
            int b;
            int c;

            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        p[a] = r;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, uint* po, uint* pd, int l, uint* pf, int lf, uint r, bool noCopStack)
        {
            return ReplaceAny(alwaysCopy, (int*)po, (int*)pd, l, (int*)pf, lf, (int)r, noCopStack);
        }


        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(uint* p, int l, uint* pf, int lf, uint r, bool noCopStack)
        {
            return ReplaceAny((int*)p, l, (int*)pf, lf, (int)r, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, long* po, long* pd, int l, long* pf, int lf, long r, bool noCopStack)
        {
            if (pd == po)
                return ReplaceAny(pd, l, pf, lf, r, noCopStack);

            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Replace(alwaysCopy, po, pd, l, *pf, r);
                if (alwaysCopy && l > 0)
                    General.Move(po, pd, l);
                return 0;
            }
            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return ReplaceAny(pd, l, pf, lf, r, noCopStack);
            }

            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackLongs && &lf > General.MinimumStack)
            {
                long* pcops = stackalloc long[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int ubi = IndexOfAny(po, l, pf, lf, true);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            pd += ubi;
            return ReplaceAny(pd, l - ubi, pf, lf, r, true) + 1;

        }


        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(long* p, int l, long* pf, int lf, long r, bool noCopStack)
        {
            if (lf < 2 || l < 1)
            {
                if (lf == 1)
                    return Replace(p, l, *pf, r);
                return 0;
            }
            if (!noCopStack && pf > General.MaximumStack && l - lf > 127 && lf < General.MaxStackLongs && &lf > General.MinimumStack)
            {
                long* pcops = stackalloc long[lf];
                General.Copy(pf, pcops, lf);
                pf = pcops;
            }

            int tot = 0;
            int a = 0;
            int b;
            long c;

            do
            {
                b = 0;
                c = p[a];
                do
                {
                    if (c == pf[b])
                    {
                        tot++;
                        p[a] = r;
                        break;
                    }
                } while (++b != lf);
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(bool alwaysCopy, ulong* po, ulong* pd, int l, ulong* pf, int lf, ulong r, bool noCopStack)
        {
            return ReplaceAny(alwaysCopy, (long*)po, (long*)pd, l, (long*)pf, lf, (long)r, noCopStack);
        }

        /// <summary>
        /// Reemplaza elementos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de bsqueda.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(ulong* p, int l, ulong* pf, int lf, ulong r, bool noCopStack)
        {
            return ReplaceAny((long*)p, l, (long*)pf, lf, (long)r, noCopStack);
        }


        #endregion

        #region Replace por Menor

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos por otra cadena de menor tamao.
        /// Es una funcin directa, por lo que lr debera estar verificado correctamente 
        /// (siempre debe ser menor a lf).
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lf">
        /// Largo de la cadena de elementos a buscar.
        /// Si es menor o igual a lr se genera una excepcin.
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de elementos de reemplazo.
        /// Si es mayor o igual a lf se genera una excepcin.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan todas estas condiciones: lr es 0, alwaysCopy es false, lFinal es 0 y po-pd son diferentes,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceMinor(bool alwaysCopy, byte* po, byte* pc, byte* pd, int l, out int lFinal, byte* pf, int lf, byte* pr, int lr, bool clean,bool noCopStack)
        {
            if (lf >= l || lr < 1 || lr >= lf)
            {
                if (lr >= lf) throw new ArgumentException(Errors.LR_NO_MINOR_LF,"lf,lr");
                if (lr < 1) return Remove(alwaysCopy, po, pc, pd, l, out lFinal, pf, lf, clean, noCopStack);
                if (lf == l)
                {
                    if (pc == null)
                        pc = po;
                    if (Comp.Equal(pc, pf, l))
                    {
                        lFinal = lr;
                        if (lr != 1)
                            General.Copy(pr, pd, lr);
                        else
                            *pd = *pr;
                        if (clean)
                            General.Clean(pd + lr, lf - lr);
                        return 1;
                    }
                }
                if (po == pd)
                {
                    lFinal = l;
                }
                else if (alwaysCopy)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                {
                    lFinal = 0;
                }
                return 0;
            }

            if (pd > po && pd - po < l)
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD,"pc,pd");
                General.Move(po, pd, l);
                po = pd;
            }
            else if (pc == null)
                pc = po;


            if (!noCopStack && l - lf > 127 && lf < General.MaxStackBytes && &lf>General.MinimumStack)
            {
                if (pf > General.MaximumStack)
                {
                    byte* pcops = stackalloc byte[lf];
                    General.Copy(pf, pcops, lf);
                    pf = pcops;
                }
                if (lr != 1 && pr > General.MaximumStack && l - lf > 1023)
                {
                    byte* pcops2 = stackalloc byte[lr];
                    General.Copy(pr, pcops2, lr);
                    pr = pcops2;
                }
            }

            int tot = 0, ipo = 0, tc = l;
            lFinal = 0;
            byte r = *pr;
otra:
            tc = IndexOf(pc + ipo, tc, pf, lf, true);
            if (tc != -1)
            {
                tot++;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                    if (lr == 1)
                    {
                        pd[lFinal++] = r;
                    }
                    else 
                    {
                        General.Copy(pr, pd + lFinal, lr);
                        lFinal += lr;
                    }
                }
                ipo += lf;
                tc = l - ipo;
                if (lf <= tc)
                    goto otra;
            }
            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean)
                    General.Clean(pd + lFinal, l - lFinal);
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos por otra cadena de menor tamao.
        /// Es una funcin directa, por lo que lr debera estar verificado correctamente 
        /// (siempre debe ser menor a lf).
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lf">
        /// Largo de la cadena de elementos a buscar.
        /// Si es menor o igual a lr se genera una excepcin.
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de elementos de reemplazo.
        /// Si es mayor o igual a lf se genera una excepcin.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan todas estas condiciones: lr es 0, alwaysCopy es false, lFinal es 0 y po-pd son diferentes,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceMinor(bool alwaysCopy, sbyte* po, sbyte* pc, sbyte* pd, int l, out int lFinal, sbyte* pf, int lf, sbyte* pr, int lr, bool clean, bool noCopStack)
        {
            return ReplaceMinor(alwaysCopy, (byte*)po, (byte*)pc, (byte*)pd, l, out lFinal, (byte*)pf, lf, (byte*)pr, lr, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos por otra cadena de menor tamao.
        /// Es una funcin directa, por lo que lr debera estar verificado correctamente 
        /// (siempre debe ser menor a lf).
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lf">
        /// Largo de la cadena de elementos a buscar.
        /// Si es menor o igual a lr se genera una excepcin.
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de elementos de reemplazo.
        /// Si es mayor o igual a lf se genera una excepcin.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan todas estas condiciones: lr es 0, alwaysCopy es false, lFinal es 0 y po-pd son diferentes,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceMinor(bool alwaysCopy, char* po, char* pc, char* pd, int l, out int lFinal, char* pf, int lf, char* pr, int lr, bool clean, bool noCopStack)
        {
            if (lf >= l || lr < 1 || lr >= lf)
            {
                if (lr >= lf) throw new ArgumentException(Errors.LR_NO_MINOR_LF, "lf,lr");
                if (lr < 1) return Remove(alwaysCopy, po, pc, pd, l, out lFinal, pf, lf, clean, noCopStack);
                if (lf == l)
                {
                    if (pc == null)
                        pc = po;
                    if (Comp.Equal(pc, pf, l))
                    {
                        lFinal = lr;
                        if (lr != 1)
                            General.Copy(pr, pd, lr);
                        else
                            *pd = *pr;
                        if (clean)
                            General.Clean(pd + lr, lf - lr);
                        return 1;
                    }
                }
                if (po == pd)
                {
                    lFinal = l;
                }
                else if (alwaysCopy)
                {
                    General.Move(po, pd, l);
                    lFinal = l;
                }
                else
                {
                    lFinal = 0;
                }
                return 0;
            }

            if (pd > po && pd - po < l)
            {
                if (pc == null || po == pc)
                    pc = pd;
                else if (pc == pd)
                    throw new ArgumentException(Errors.PC_EQUAL_PD, "pc,pd");
                General.Move(po, pd, l);
                po = pd;
            }
            else if (pc == null)
                pc = po;


            if (!noCopStack && l - lf > 127 && lf < General.MaxStackChars && &lf > General.MinimumStack)
            {
                if (pf > General.MaximumStack)
                {
                    char* pcops = stackalloc char[lf];
                    General.Copy(pf, pcops, lf);
                    pf = pcops;
                }
                if (lr != 1 && pr > General.MaximumStack && l - lf > 1023)
                {
                    char* pcops2 = stackalloc char[lr];
                    General.Copy(pr, pcops2, lr);
                    pr = pcops2;
                }
            }

            int tot = 0, ipo = 0, tc = l;
            lFinal = 0;
            char r = *pr;
otra:
            tc = IndexOf(pc + ipo, tc, pf, lf, true);
            if (tc != -1)
            {
                tot++;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                    if (lr == 1)
                    {
                        pd[lFinal++] = r;
                    }
                    else
                    {
                        General.Copy(pr, pd + lFinal, lr);
                        lFinal += lr;
                    }
                }
                ipo += lf;
                tc = l - ipo;
                if (lf <= tc)
                    goto otra;
            }
            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean)
                    General.Clean(pd + lFinal, l - lFinal);
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }


        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos por otra cadena de menor tamao.
        /// Es una funcin directa, por lo que lr debera estar verificado correctamente 
        /// (siempre debe ser menor a lf).
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lf">
        /// Largo de la cadena de elementos a buscar.
        /// Si es menor o igual a lr se genera una excepcin.
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de elementos de reemplazo.
        /// Si es mayor o igual a lf se genera una excepcin.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan todas estas condiciones: lr es 0, alwaysCopy es false, lFinal es 0 y po-pd son diferentes,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceMinor(bool alwaysCopy, short* po, short* pc, short* pd, int l, out int lFinal, short* pf, int lf, short* pr, int lr, bool clean, bool noCopStack)
        {
            return ReplaceMinor(alwaysCopy, (char*)po, (char*)pc, (char*)pd, l, out lFinal, (char*)pf, lf, (char*)pr, lr, clean, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos por otra cadena de menor tamao.
        /// Es una funcin directa, por lo que lr debera estar verificado correctamente 
        /// (siempre debe ser menor a lf).
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pc">
        /// Puntero de comparacin para operaciones donde se ignora May/Min u otras donde
        /// el puntero de bsqueda es independiente al de origen.
        /// Si es null o igual a po: Usar el puntero de origen (bsqueda normal).
        /// Si es igual a pd: Buscar sobre el puntero destino y copiar areas no coincidentes desde po.
        /// En este caso, pd no debe estar solapado con po!
        /// En el resto de las situaciones deber encontrarse en un area diferente
        /// a po y pd dado que no verifica ni corrige solapados.
        /// </param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lf">
        /// Largo de la cadena de elementos a buscar.
        /// Si es menor o igual a lr se genera una excepcin.
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de elementos de reemplazo.
        /// Si es mayor o igual a lf se genera una excepcin.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan todas estas condiciones: lr es 0, alwaysCopy es false, lFinal es 0 y po-pd son diferentes,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack las cadenas de bsqueda y reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceMinor(bool alwaysCopy, ushort* po, ushort* pc, ushort* pd, int l, out int lFinal, ushort* pf, int lf, ushort* pr, int lr, bool clean, bool noCopStack)
        {
            return ReplaceMinor(alwaysCopy, (char*)po, (char*)pc, (char*)pd, l, out lFinal, (char*)pf, lf, (char*)pr, lr, clean, noCopStack);
        }

        #endregion

        #region Replace Locations

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, byte* p, byte r)
        {
            if (tot < 1)
                return;
            int a = 0;
            do
            {
                p[pCont[a]] = r;
            } while (++a != tot);
            return;
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, byte* p, byte* pr, int lfr, bool noCopStack)
        {
            if (lfr < 2)
            {
                if (lfr == 1)
                    ReplaceLocations(pCont, tot, p, *pr);
                return;
            }
            
            if (tot < 1)
                return;

            if (!noCopStack && tot > 15 && pr > General.MaximumStack && lfr < General.MaxStackBytes && &lfr>General.MinimumStack)
            {
                byte* pcops = stackalloc byte[lfr];
                General.Copy(pr, pcops, lfr);
                pr = pcops;
            }

            int a = 0;
            do
            {
                General.Copy(pr, p + pCont[a], lfr);
            } while (++a != tot);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, byte* po, int l, byte* pd, int lf, byte* pr, int lr, bool clean, bool noCopStack)
        {
            int lFinal = ReserveLocations(pCont, tot, po, l, pd, lf, lr, clean);
            if (lFinal > 0)
            {
                ReplaceLocations(pCont, tot, pd, pr, lr, noCopStack);
            }
            return lFinal;
        }

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, sbyte* p, sbyte r)
        {
            ReplaceLocations(pCont, tot, (byte*)p, (byte)r);
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, sbyte* p, sbyte* pr, int lfr, bool noCopStack)
        {
            ReplaceLocations(pCont, tot, (byte*)p, (byte*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, sbyte* po, int l, sbyte* pd, int lf, sbyte* pr, int lr, bool clean, bool noCopStack)
        {
            return ReplaceLocations(pCont, tot, (byte*)po, l, (byte*)pd, lf, (byte*)pr, lr, clean, noCopStack);
        }

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, char* p, char r)
        {
            if (tot < 1)
                return;
            int a = 0;
            do
            {
                p[pCont[a]] = r;
            } while (++a != tot);
            return;
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, char* p, char* pr, int lfr, bool noCopStack)
        {
            if (lfr < 2)
            {
                if (lfr == 1)
                    ReplaceLocations(pCont, tot, p, *pr);
                return;
            }

            if (tot < 1)
                return;

            if (!noCopStack && tot > 15 && pr > General.MaximumStack && lfr < General.MaxStackChars && &lfr > General.MinimumStack)
            {
                char* pcops = stackalloc char[lfr];
                General.Copy(pr, pcops, lfr);
                pr = pcops;
            }

            int a = 0;
            do
            {
                General.Copy(pr, p + pCont[a], lfr);
            } while (++a != tot);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, char* po, int l, char* pd, int lf, char* pr, int lr, bool clean, bool noCopStack)
        {
            int lFinal = ReserveLocations(pCont, tot, po, l, pd, lf, lr, clean);
            if (lFinal > 0)
            {
                ReplaceLocations(pCont, tot, pd, pr, lr, noCopStack);
            }
            return lFinal;
        }

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, short* p, short r)
        {
            ReplaceLocations(pCont, tot, (char*)p, (char)r);
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, short* p, short* pr, int lfr, bool noCopStack)
        {
            ReplaceLocations(pCont, tot, (char*)p, (char*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, short* po, int l, short* pd, int lf, short* pr, int lr, bool clean, bool noCopStack)
        {
            return ReplaceLocations(pCont, tot, (char*)po, l, (char*)pd, lf, (char*)pr, lr, clean, noCopStack);
        }

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, ushort* p, ushort r)
        {
            ReplaceLocations(pCont, tot, (char*)p, (char)r);
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, ushort* p, ushort* pr, int lfr, bool noCopStack)
        {
            ReplaceLocations(pCont, tot, (char*)p, (char*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, ushort* po, int l, ushort* pd, int lf, ushort* pr, int lr, bool clean, bool noCopStack)
        {
            return ReplaceLocations(pCont, tot, (char*)po, l, (char*)pd, lf, (char*)pr, lr, clean, noCopStack);
        }

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, int* p, int r)
        {
            if (tot < 1)
                return;
            int a = 0;
            do
            {
                p[pCont[a]] = r;
            } while (++a != tot);
            return;
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, int* p, int* pr, int lfr, bool noCopStack)
        {
            if (lfr < 2)
            {
                if (lfr == 1)
                    ReplaceLocations(pCont, tot, p, *pr);
                return;
            }

            if (tot < 1)
                return;

            if (!noCopStack && tot > 15 && pr > General.MaximumStack && lfr < General.MaxStackInts && &lfr > General.MinimumStack)
            {
                int* pcops = stackalloc int[lfr];
                General.Copy(pr, pcops, lfr);
                pr = pcops;
            }

            int a = 0;
            do
            {
                General.Copy(pr, p + pCont[a], lfr);
            } while (++a != tot);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, int* po, int l, int* pd, int lf, int* pr, int lr, bool clean, bool noCopStack)
        {
            int lFinal = ReserveLocations(pCont, tot, po, l, pd, lf, lr, clean);
            if (lFinal > 0)
            {
                ReplaceLocations(pCont, tot, pd, pr, lr, noCopStack);
            }
            return lFinal;
        }

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, uint* p, uint r)
        {
            ReplaceLocations(pCont, tot, (int*)p, (int)r);
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, uint* p, uint* pr, int lfr, bool noCopStack)
        {
            ReplaceLocations(pCont, tot, (int*)p, (int*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, uint* po, int l, uint* pd, int lf, uint* pr, int lr, bool clean, bool noCopStack)
        {
            return ReplaceLocations(pCont, tot, (int*)po, l, (int*)pd, lf, (int*)pr, lr, clean, noCopStack);
        }

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, long* p, long r)
        {
            if (tot < 1)
                return;
            int a = 0;
            do
            {
                p[pCont[a]] = r;
            } while (++a != tot);
            return;
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, long* p, long* pr, int lfr, bool noCopStack)
        {
            if (lfr < 2)
            {
                if (lfr == 1)
                    ReplaceLocations(pCont, tot, p, *pr);
                return;
            }

            if (tot < 1)
                return;

            if (!noCopStack && tot > 15 && pr > General.MaximumStack && lfr < General.MaxStackLongs && &lfr > General.MinimumStack)
            {
                long* pcops = stackalloc long[lfr];
                General.Copy(pr, pcops, lfr);
                pr = pcops;
            }

            int a = 0;
            do
            {
                General.Copy(pr, p + pCont[a], lfr);
            } while (++a != tot);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, long* po, int l, long* pd, int lf, long* pr, int lr, bool clean, bool noCopStack)
        {
            int lFinal = ReserveLocations(pCont, tot, po, l, pd, lf, lr, clean);
            if (lFinal > 0)
            {
                ReplaceLocations(pCont, tot, pd, pr, lr, noCopStack);
            }
            return lFinal;
        }

        /// <summary>
        /// Reemplaza elementos en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: En este caso, las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="r">
        /// Reemplazo.
        /// </param>
        public static void ReplaceLocations(int* pCont, int tot, ulong* p, ulong r)
        {
            ReplaceLocations(pCont, tot, (long*)p, (long)r);
        }

        /// <summary>
        /// Reemplaza elementos de igual tamao en las ubicaciones indicadas.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones pueden estar desordenadas.
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="p">
        /// Puntero Origen/Destino.
        /// </param>
        /// <param name="pr">
        /// Puntero de reemplazos.
        /// </param>
        /// <param name="lfr">
        /// Largo del Puntero de reemplazo.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        public static void ReplaceLocations(int* pCont, int tot, ulong* p, ulong* pr, int lfr, bool noCopStack)
        {
            ReplaceLocations(pCont, tot, (long*)p, (long*)pr, lfr, noCopStack);
        }

        /// <summary>
        /// Copia memoria reemplazando, insertando o eliminando ubicaciones.
        /// </summary>
        /// <param name="pCont">
        /// Puntero de ubicaciones: Las ubicaciones deben ser ascendentes y no solaparse (no verifica).
        /// </param>
        /// <param name="tot">
        /// Total de elementos en el puntero de ubicaciones.
        /// </param>
        /// <param name="po">
        /// Puntero de origen.
        /// </param>
        /// <param name="l">
        /// Largo del puntero de origen. Debe ser mayor a 0.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe ser del largo suficiente para contener la cadena final.
        /// </param>
        /// <param name="lf">
        /// Largo de la cadena de eliminacin (salto sobre origen).
        /// </param>
        /// <param name="pr">Puntero de reemplazos.</param>
        /// <param name="lr">
        /// Largo del puntero de reemplazo.
        /// </param>
        /// <param name="clean">
        /// Si true y lr es menor a lf, rellena con ceros el area 
        /// en pd equivalente al numero de elementos eliminados a partir del largo final.
        /// </param>
        /// <param name="noCopStack">No intentar copiar al Stack la cadena de reemplazo.</param>
        /// <returns>
        /// Largo final del puntero (es calculado en base a l + (tot*(lr-lf))).
        /// </returns>
        public static int ReplaceLocations(int* pCont, int tot, ulong* po, int l, ulong* pd, int lf, ulong* pr, int lr, bool clean, bool noCopStack)
        {
            return ReplaceLocations(pCont, tot, (long*)po, l, (long*)pd, lf, (long*)pr, lr, clean, noCopStack);
        }


        #endregion

        #region Diferente Cero

        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0.</returns>
        public static int IndexOfNoZero(byte* p, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p != 0) return 0;
                return -1;
            }
            int l1 = l;
            if (l > 15)
            {
                do
                {
                    if (*((int*)p) != 0)
                    {
                        if (*((short*)p) != 0)
                        {
                            if (*p != 0) return l1 - l;
                            return l1 - l + 1;
                        }
                        if (p[2] != 0) return l1 - l + 2;
                        return l1 - l + 3;
                    }

                    if (*((int*)(p + 4)) != 0)
                    {
                        if (*((short*)(p + 4)) != 0)
                        {
                            if (p[4] != 0) return l1 - l + 4;
                            return l1 - l + 5;
                        }
                        if (p[6] != 0) return l1 - l + 6;
                        return l1 - l + 7;
                    }

                    if (*((int*)(p + 8)) != 0)
                    {
                        if (*((short*)(p + 8)) != 0)
                        {
                            if (p[8] != 0) return l1 - l + 8;
                            return l1 - l + 9;
                        }
                        if (p[10] != 0) return l1 - l + 10;
                        return l1 - l + 11;
                    }
                    if (*((int*)(p + 12)) != 0)
                    {
                        if (*((short*)(p + 12)) != 0)
                        {
                            if (p[12] != 0) return l1 - l + 12;
                            return l1 - l + 13;
                        }
                        if (p[14] != 0) return l1 - l + 14;
                        return l1 - l + 15;
                    }

                    p += 16;
                }
                while ((l -= 16) > 15);
            }
            if (l == 0)
            {
                return -1;
            }
            if ((l & 8) != 0)
            {
                if (*((int*)p) != 0)
                {
                    if (*((short*)p) != 0)
                    {
                        if (*p != 0) return l1 - l;
                        return l1 - l + 1;
                    }
                    if (p[2] != 0) return l1 - l + 2;
                    return l1 - l + 3;
                }

                if (*((int*)(p + 4)) != 0)
                {
                    if (*((short*)(p + 4)) != 0)
                    {
                        if (p[4] != 0) return l1 - l + 4;
                        return l1 - l + 5;
                    }
                    if (p[6] != 0) return l1 - l + 6;
                    return l1 - l + 7;
                }
                p += 8;
                l -= 8;
            }
            if ((l & 4) != 0)
            {
                if (*((int*)p) != 0)
                {
                    if (*((short*)p) != 0)
                    {
                        if (*p != 0) return l1 - l;
                        return l1 - l + 1;
                    }
                    if (p[2] != 0) return l1 - l + 2;
                    return l1 - l + 3;
                }
                p += 4;
                l -= 4;
            }
            if ((l & 2) != 0)
            {
                if (*((short*)p) != 0)
                {
                    if (*p != 0) return l1 - l;
                    return l1 - l + 1;
                }
                p += 2;
                l -= 2;
            }
            if (l == 1 && *p != 0) return l1 - 1;
            return -1;
        }


        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0) o -1 si todos los elementos son iguales a 0.</returns>
        public static int LastIndexOfNoZero(byte* p, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p != 0) return 0;
                return -1;
            }
            switch (l & 15)
            {
                case 0:
                    p += l;
                    break;
                case 1:
                    if (*(p += --l) != 0) return l;
                    break;
                case 2:
                    if (*((short*)(p += (l -= 2))) != 0)
                    {
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (l == 0) return -1;
                    break;
                case 3:
                    if (*(p += --l) != 0) return l;
                    if (*((short*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if ((l -= 2) == 0) return -1;
                    break;
                case 4:
                    if (*((int*)(p += (l -= 4))) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l + 3;
                            return l + 2;
                        }
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (l == 0) return -1;
                    break;
                case 5:
                    if (*(p += --l) != 0) return l;
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 1;
                            return l - 2;
                        }
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if ((l -= 4) == 0) return -1;
                    break;
                case 6:
                    if (*((short*)(p += (l -= 2))) != 0)
                    {
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 1;
                            return l - 2;
                        }
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if ((l -= 4) == 0) return -1;
                    break;
                case 7:
                    if (*(p += --l) != 0) return l;
                    if (*((short*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 3;
                            return l - 4;
                        }
                        if (p[1] != 0) return l - 5;
                        return l - 6;
                    }
                    if ((l -= 6) == 0) return -1;
                    break;
                case 8:
                    if (*((int*)(p += (l -= 4))) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l + 3;
                            return l + 2;
                        }
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 1;
                            return l - 2;
                        }
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if ((l -= 4) == 0) return -1;
                    break;
                case 9:
                    if (*(p += --l) != 0) return l;
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 1;
                            return l - 2;
                        }
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 5;
                            return l - 6;
                        }
                        if (p[1] != 0) return l - 7;
                        return l - 8;
                    }
                    if ((l -= 8) == 0) return -1;
                    break;
                case 10:
                    if (*((short*)(p += (l -= 2))) != 0)
                    {
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 1;
                            return l - 2;
                        }
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 5;
                            return l - 6;
                        }
                        if (p[1] != 0) return l - 7;
                        return l - 8;
                    }
                    if ((l -= 8) == 0) return -1;
                    break;
                case 11:
                    if (*(p += --l) != 0) return l;
                    if (*((short*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 3;
                            return l - 4;
                        }
                        if (p[1] != 0) return l - 5;
                        return l - 6;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 7;
                            return l - 8;
                        }
                        if (p[1] != 0) return l - 9;
                        return l - 10;
                    }
                    if ((l -= 10) == 0) return -1;
                    break;
                case 12:
                    if (*((int*)(p += (l -= 4))) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l + 3;
                            return l + 2;
                        }
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 1;
                            return l - 2;
                        }
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 5;
                            return l - 6;
                        }
                        if (p[1] != 0) return l - 7;
                        return l - 8;
                    }
                    if ((l -= 8) == 0) return -1;
                    break;
                case 13:
                    if (*(p += --l) != 0) return l;
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 1;
                            return l - 2;
                        }
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 5;
                            return l - 6;
                        }
                        if (p[1] != 0) return l - 7;
                        return l - 8;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 9;
                            return l - 10;
                        }
                        if (p[1] != 0) return l - 11;
                        return l - 12;
                    }
                    if ((l -= 12) == 0) return -1;
                    break;
                case 14:
                    if (*((short*)(p += (l -= 2))) != 0)
                    {
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 1;
                            return l - 2;
                        }
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 5;
                            return l - 6;
                        }
                        if (p[1] != 0) return l - 7;
                        return l - 8;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 9;
                            return l - 10;
                        }
                        if (p[1] != 0) return l - 11;
                        return l - 12;
                    }
                    if ((l -= 12) == 0) return -1;
                    break;
                case 15:
                    if (*(p += --l) != 0) return l;
                    if (*((short*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 3;
                            return l - 4;
                        }
                        if (p[1] != 0) return l - 5;
                        return l - 6;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 7;
                            return l - 8;
                        }
                        if (p[1] != 0) return l - 9;
                        return l - 10;
                    }
                    if (*((int*)(p -= 4)) != 0)
                    {
                        if (*((short*)(p + 2)) != 0)
                        {
                            if (p[3] != 0) return l - 11;
                            return l - 12;
                        }
                        if (p[1] != 0) return l - 13;
                        return l - 14;
                    }
                    if ((l -= 14) == 0) return -1;
                    break;
            }
            do
            {
                if (*((int*)(p - 4)) != 0)
                {
                    if (*((short*)(p - 2)) != 0)
                    {
                        if (p[-1] != 0) return l - 1;
                        return l - 2;
                    }
                    if (p[-3] != 0) return l - 3;
                    return l - 4;
                }
                if (*((int*)(p - 8)) != 0)
                {
                    if (*((short*)(p - 6)) != 0)
                    {
                        if (p[-5] != 0) return l - 5;
                        return l - 6;
                    }
                    if (p[-7] != 0) return l - 7;
                    return l - 8;
                }
                if (*((int*)(p - 12)) != 0)
                {
                    if (*((short*)(p - 10)) != 0)
                    {
                        if (p[-9] != 0) return l - 9;
                        return l - 10;
                    }
                    if (p[-11] != 0) return l - 11;
                    return l - 12;
                }
                if (*((int*)(p - 16)) != 0)
                {
                    if (*((short*)(p - 14)) != 0)
                    {
                        if (p[-13] != 0) return l - 13;
                        return l - 14;
                    }
                    if (p[-15] != 0) return l - 15;
                    return l - 16;
                }
                if ((l -= 16) == 0) return -1;
                p -= 16;
            } while (true);
        }


        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0.</returns>
        public static int IndexOfNoZero(short* p, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p != 0) return 0;
                return -1;
            }
            int l1 = l;
            if (l > 15)
            {
                do
                {
                    if (*((int*)p) != 0)
                    {
                        if (*p != 0) return l1 - l;
                        return l1 - l + 1;
                    }
                    if (*((int*)(p + 2)) != 0)
                    {
                        if (p[2] != 0) return l1 - l + 2;
                        return l1 - l + 3;
                    }
                    if (*((int*)(p + 4)) != 0)
                    {
                        if (p[4] != 0) return l1 - l + 4;
                        return l1 - l + 5;
                    }
                    if (*((int*)(p + 6)) != 0)
                    {
                        if (p[6] != 0) return l1 - l + 6;
                        return l1 - l + 7;
                    }
                    if (*((int*)(p + 8)) != 0)
                    {
                        if (p[8] != 0) return l1 - l + 8;
                        return l1 - l + 9;
                    }
                    if (*((int*)(p + 10)) != 0)
                    {
                        if (p[10] != 0) return l1 - l + 10;
                        return l1 - l + 11;
                    }
                    if (*((int*)(p + 12)) != 0)
                    {
                        if (p[12] != 0) return l1 - l + 12;
                        return l1 - l + 13;
                    }
                    if (*((int*)(p + 14)) != 0)
                    {
                        if (p[14] != 0) return l1 - l + 14;
                        return l1 - l + 15;
                    }
                    p += 16;
                }
                while ((l -= 16) > 15);
            }
            if (l == 0)
            {
                return -1;
            }

            if ((l & 8) != 0)
            {
                if (*((int*)p) != 0)
                {
                    if (*p != 0) return l1 - l;
                    return l1 - l + 1;
                }
                if (*((int*)(p + 2)) != 0)
                {
                    if (p[2] != 0) return l1 - l + 2;
                    return l1 - l + 3;
                }
                if (*((int*)(p + 4)) != 0)
                {
                    if (p[4] != 0) return l1 - l + 4;
                    return l1 - l + 5;
                }
                if (*((int*)(p + 6)) != 0)
                {
                    if (p[6] != 0) return l1 - l + 6;
                    return l1 - l + 7;
                }
                p += 8;
                l -= 8;
            }

            if ((l & 4) != 0)
            {
                if (*((int*)p) != 0)
                {
                    if (*p != 0) return l1 - l;
                    return l1 - l + 1;
                }
                if (*((int*)(p + 2)) != 0)
                {
                    if (p[2] != 0) return l1 - l + 2;
                    return l1 - l + 3;
                }
                p += 4;
                l -= 4;
            }

            if ((l & 2) != 0)
            {
                if (*((int*)p) != 0)
                {
                    if (*p != 0) return l1 - l;
                    return l1 - l + 1;
                }
                p += 2;
                l -= 2;
            }
            if (l == 1 && *p != 0) return l1 - 1;
            return -1;

        }

        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0) o -1 si todos los elementos son iguales a 0.</returns>
        public static int LastIndexOfNoZero(short* p, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p != 0) return 0;
                return -1;
            }
            switch (l & 7)
            {
                case 0:
                    p += l;
                    break;
                case 1:
                    if (*(p += --l) != 0) return l;
                    break;
                case 2:
                    if (*((int*)(p += (l -= 2))) != 0)
                    {
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (l == 0) return -1;
                    break;
                case 3:
                    if (*(p += --l) != 0) return l;
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if ((l -= 2) == 0) return -1;
                    break;
                case 4:
                    if (*((int*)(p += (l -= 2))) != 0)
                    {
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if ((l -= 2) == 0) return -1;
                    break;
                case 5:
                    if (*(p += --l) != 0) return l;
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if ((l -= 4) == 0) return -1;
                    break;
                case 6:
                    if (*((int*)(p += (l -= 2))) != 0)
                    {
                        if (p[1] != 0) return l + 1;
                        return l;
                    }
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if ((l -= 4) == 0) return -1;
                    break;
                case 7:
                    if (*(p += --l) != 0) return l;
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 1;
                        return l - 2;
                    }
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 3;
                        return l - 4;
                    }
                    if (*((int*)(p -= 2)) != 0)
                    {
                        if (p[1] != 0) return l - 5;
                        return l - 6;
                    }
                    if ((l -= 6) == 0) return -1;
                    break;
            }
            do
            {
                if (*((int*)(p - 2)) != 0)
                {
                    if (p[-1] != 0) return l - 1;
                    return l - 2;
                }

                if (*((int*)(p - 4)) != 0)
                {
                    if (p[-3] != 0) return l - 3;
                    return l - 4;
                }

                if (*((int*)(p - 6)) != 0)
                {
                    if (p[-5] != 0) return l - 5;
                    return l - 6;
                }

                if (*((int*)(p - 8)) != 0)
                {
                    if (p[-7] != 0) return l - 7;
                    return l - 8;
                }

                if ((l -= 8) == 0) return -1;
                p -= 8;
            } while (true);
        }


        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0.</returns>
        public static int IndexOfNoZero(int* p, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p != 0) return 0;
                return -1;
            }
            int l1 = l;
            if (l > 15)
            {
                do
                {
                    if (*p != 0)
                        return l1 - l;
                    if (p[1] != 0)
                        return l1 - l + 1;
                    if (p[2] != 0)
                        return l1 - l + 2;
                    if (p[3] != 0)
                        return l1 - l + 3;
                    if (p[4] != 0)
                        return l1 - l + 4;
                    if (p[5] != 0)
                        return l1 - l + 5;
                    if (p[6] != 0)
                        return l1 - l + 6;
                    if (p[7] != 0)
                        return l1 - l + 7;
                    if (p[8] != 0)
                        return l1 - l + 8;
                    if (p[9] != 0)
                        return l1 - l + 9;
                    if (p[10] != 0)
                        return l1 - l + 10;
                    if (p[11] != 0)
                        return l1 - l + 11;
                    if (p[12] != 0)
                        return l1 - l + 12;
                    if (p[13] != 0)
                        return l1 - l + 13;
                    if (p[14] != 0)
                        return l1 - l + 14;
                    if (p[15] != 0)
                        return l1 - l + 15;
                    p += 16;
                }
                while ((l -= 16) > 15);
            }
            if (l == 0)
            {
                return -1;
            }
            if ((l & 8) != 0)
            {
                if (*p != 0)
                    return l1 - l;
                if (p[1] != 0)
                    return l1 - l + 1;
                if (p[2] != 0)
                    return l1 - l + 2;
                if (p[3] != 0)
                    return l1 - l + 3;
                if (p[4] != 0)
                    return l1 - l + 4;
                if (p[5] != 0)
                    return l1 - l + 5;
                if (p[6] != 0)
                    return l1 - l + 6;
                if (p[7] != 0)
                    return l1 - l + 7;
                p += 8;
                l -= 8;
            }

            if ((l & 4) != 0)
            {
                if (*p != 0)
                    return l1 - l;
                if (p[1] != 0)
                    return l1 - l + 1;
                if (p[2] != 0)
                    return l1 - l + 2;
                if (p[3] != 0)
                    return l1 - l + 3;
                p += 4;
                l -= 4;
            }

            if ((l & 2) != 0)
            {
                if (*p != 0)
                    return l1 - l;
                if (p[1] != 0)
                    return l1 - l + 1;
                p += 2;
                l -= 2;
            }
            if (l == 1 && *p != 0) 
                return l1 - 1;
            return -1;
        }

        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0) o -1 si todos los elementos son iguales a 0.</returns>
        public static int LastIndexOfNoZero(int* p, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p != 0) return 0;
                return -1;
            }
            switch (l & 7)
            {
                case 0:
                    p += l;
                    break;
                case 1:
                    if (*(p += --l) != 0) return l;
                    break;
                case 2:
                    p += l-=2;
                    if (p[1] != 0) return l + 1;
                    if (*p != 0) return l;
                    if (l == 0) return -1;
                    break;
                case 3:
                    p += l -= 3;
                    if (p[2] != 0) return l + 2;
                    if (p[1] != 0) return l + 1;
                    if (*p != 0) return l;
                    if (l == 0) return -1;
                    break;
                case 4:
                    p += l -= 4;
                    if (p[3] != 0) return l + 3;
                    if (p[2] != 0) return l + 2;
                    if (p[1] != 0) return l + 1;
                    if (*p != 0) return l;
                    if (l == 0) return -1;
                    break;
                case 5:
                    p += l -= 5;
                    if (p[4] != 0) return l + 4;
                    if (p[3] != 0) return l + 3;
                    if (p[2] != 0) return l + 2;
                    if (p[1] != 0) return l + 1;
                    if (*p != 0) return l;
                    if (l == 0) return -1;
                    break;
                case 6:
                    p += l -= 6;
                    if (p[5] != 0) return l + 5;
                    if (p[4] != 0) return l + 4;
                    if (p[3] != 0) return l + 3;
                    if (p[2] != 0) return l + 2;
                    if (p[1] != 0) return l + 1;
                    if (*p != 0) return l;
                    if (l == 0) return -1;
                    break;
                case 7:
                    p += l -= 7;
                    if (p[6] != 0) return l + 6;
                    if (p[5] != 0) return l + 5;
                    if (p[4] != 0) return l + 4;
                    if (p[3] != 0) return l + 3;
                    if (p[2] != 0) return l + 2;
                    if (p[1] != 0) return l + 1;
                    if (*p != 0) return l;
                    if (l == 0) return -1;
                    break;
            }

            do
            {
                if (p[-1] != 0) return l - 1;
                if (p[-2] != 0) return l - 2;
                if (p[-3] != 0) return l - 3;
                if (p[-4] != 0) return l - 4;
                if (p[-5] != 0) return l - 5;
                if (p[-6] != 0) return l - 6;
                if (p[-7] != 0) return l - 7;
                if (*(p -= 8) != 0) return l - 8;

            } while ((l -= 8) != 0);
            return -1;
        }

        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0L.</returns>
        public static int IndexOfNoZero(long* p, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p != 0L) return 0;
                return -1;
            }
            int l1 = l;
            if (l > 15)
            {
                do
                {
                    if (*p != 0L)
                        return l1 - l;
                    if (p[1] != 0L)
                        return l1 - l + 1;
                    if (p[2] != 0L)
                        return l1 - l + 2;
                    if (p[3] != 0L)
                        return l1 - l + 3;
                    if (p[4] != 0L)
                        return l1 - l + 4;
                    if (p[5] != 0L)
                        return l1 - l + 5;
                    if (p[6] != 0L)
                        return l1 - l + 6;
                    if (p[7] != 0L)
                        return l1 - l + 7;
                    if (p[8] != 0L)
                        return l1 - l + 8;
                    if (p[9] != 0L)
                        return l1 - l + 9;
                    if (p[10] != 0L)
                        return l1 - l + 10;
                    if (p[11] != 0L)
                        return l1 - l + 11;
                    if (p[12] != 0L)
                        return l1 - l + 12;
                    if (p[13] != 0L)
                        return l1 - l + 13;
                    if (p[14] != 0L)
                        return l1 - l + 14;
                    if (p[15] != 0L)
                        return l1 - l + 15;
                    p += 16;
                }
                while ((l -= 16) > 15);
            }
            if (l == 0)
            {
                return -1;
            }
            if ((l & 8) != 0)
            {
                if (*p != 0L)
                    return l1 - l;
                if (p[1] != 0L)
                    return l1 - l + 1;
                if (p[2] != 0L)
                    return l1 - l + 2;
                if (p[3] != 0L)
                    return l1 - l + 3;
                if (p[4] != 0L)
                    return l1 - l + 4;
                if (p[5] != 0L)
                    return l1 - l + 5;
                if (p[6] != 0L)
                    return l1 - l + 6;
                if (p[7] != 0L)
                    return l1 - l + 7;
                p += 8;
                l -= 8;
            }

            if ((l & 4) != 0)
            {
                if (*p != 0L)
                    return l1 - l;
                if (p[1] != 0L)
                    return l1 - l + 1;
                if (p[2] != 0L)
                    return l1 - l + 2;
                if (p[3] != 0L)
                    return l1 - l + 3;
                p += 4;
                l -= 4;
            }

            if ((l & 2) != 0)
            {
                if (*p != 0L)
                    return l1 - l;
                if (p[1] != 0L)
                    return l1 - l + 1;
                p += 2;
                l -= 2;
            }
            if (l == 1 && *p != 0L)
                return l1 - 1;
            return -1;
        }

        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0L) o -1 si todos los elementos son iguales a 0L.</returns>
        public static int LastIndexOfNoZero(long* p, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p != 0L) return 0;
                return -1;
            }
            switch (l & 7)
            {
                case 0:
                    p += l;
                    break;
                case 1:
                    if (*(p += --l) != 0L) return l;
                    break;
                case 2:
                    p += l -= 2;
                    if (p[1] != 0L) return l + 1;
                    if (*p != 0L) return l;
                    if (l == 0) return -1;
                    break;
                case 3:
                    p += l -= 3;
                    if (p[2] != 0L) return l + 2;
                    if (p[1] != 0L) return l + 1;
                    if (*p != 0L) return l;
                    if (l == 0) return -1;
                    break;
                case 4:
                    p += l -= 4;
                    if (p[3] != 0L) return l + 3;
                    if (p[2] != 0L) return l + 2;
                    if (p[1] != 0L) return l + 1;
                    if (*p != 0L) return l;
                    if (l == 0) return -1;
                    break;
                case 5:
                    p += l -= 5;
                    if (p[4] != 0L) return l + 4;
                    if (p[3] != 0L) return l + 3;
                    if (p[2] != 0L) return l + 2;
                    if (p[1] != 0L) return l + 1;
                    if (*p != 0L) return l;
                    if (l == 0) return -1;
                    break;
                case 6:
                    p += l -= 6;
                    if (p[5] != 0L) return l + 5;
                    if (p[4] != 0L) return l + 4;
                    if (p[3] != 0L) return l + 3;
                    if (p[2] != 0L) return l + 2;
                    if (p[1] != 0L) return l + 1;
                    if (*p != 0L) return l;
                    if (l == 0) return -1;
                    break;
                case 7:
                    p += l -= 7;
                    if (p[6] != 0L) return l + 6;
                    if (p[5] != 0L) return l + 5;
                    if (p[4] != 0L) return l + 4;
                    if (p[3] != 0L) return l + 3;
                    if (p[2] != 0L) return l + 2;
                    if (p[1] != 0L) return l + 1;
                    if (*p != 0L) return l;
                    if (l == 0) return -1;
                    break;
            }

            do
            {
                if (p[-1] != 0L) return l - 1;
                if (p[-2] != 0L) return l - 2;
                if (p[-3] != 0L) return l - 3;
                if (p[-4] != 0L) return l - 4;
                if (p[-5] != 0L) return l - 5;
                if (p[-6] != 0L) return l - 6;
                if (p[-7] != 0L) return l - 7;
                if (*(p -= 8) != 0L) return l - 8;

            } while ((l -= 8) != 0);
            return -1;
        }

        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0.</returns>
        public static int IndexOfNoZero(sbyte* p, int l)
        {
            return IndexOfNoZero((byte*)p, l);
        }


        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0) o -1 si todos los elementos son iguales a 0.</returns>
        public static int LastIndexOfNoZero(sbyte* p, int l)
        {
            return LastIndexOfNoZero((byte*)p, l);
        }

        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0.</returns>
        public static int IndexOfNoZero(char* p, int l)
        {
            return IndexOfNoZero((short*)p, l);
        }


        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0) o -1 si todos los elementos son iguales a 0.</returns>
        public static int LastIndexOfNoZero(char* p, int l)
        {
            return LastIndexOfNoZero((short*)p, l);
        }

        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0.</returns>
        public static int IndexOfNoZero(ushort* p, int l)
        {
            return IndexOfNoZero((short*)p, l);
        }


        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0) o -1 si todos los elementos son iguales a 0.</returns>
        public static int LastIndexOfNoZero(ushort* p, int l)
        {
            return LastIndexOfNoZero((short*)p, l);
        }

        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0.</returns>
        public static int IndexOfNoZero(uint* p, int l)
        {
            return IndexOfNoZero((int*)p, l);
        }


        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0) o -1 si todos los elementos son iguales a 0.</returns>
        public static int LastIndexOfNoZero(uint* p, int l)
        {
            return LastIndexOfNoZero((int*)p, l);
        }

        /// <summary>
        /// Busca en memoria el primer elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Primer elemento diferente a cero o -1 si todos los elementos son 0.</returns>
        public static int IndexOfNoZero(ulong* p, int l)
        {
            return IndexOfNoZero((long*)p, l);
        }


        /// <summary>
        /// Busca en memoria el ltimo elemento diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ultimo elemento diferente a cero (ubicacin base 0) o -1 si todos los elementos son iguales a 0.</returns>
        public static int LastIndexOfNoZero(ulong* p, int l)
        {
            return LastIndexOfNoZero((long*)p, l);
        }


        #endregion

        #region Equal

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(byte* p1, byte* p2, int l)
        {
            if (p1 != p2) 
            {
                if (l > 63)
                {
                    do
                    {
                        if (*(long*)p1 != *(long*)p2 ||
                            *(long*)(p1 + 8) != *(long*)(p2 + 8) ||
                            *(long*)(p1 + 16) != *(long*)(p2 + 16) ||
                            *(long*)(p1 + 24) != *(long*)(p2 + 24) ||
                            *(long*)(p1 + 32) != *(long*)(p2 + 32) ||
                            *(long*)(p1 + 40) != *(long*)(p2 + 40) ||
                            *(long*)(p1 + 48) != *(long*)(p2 + 48) ||
                            *(long*)(p1 + 56) != *(long*)(p2 + 56)
                            ) return false;
                        p1 += 64;
                        p2 += 64;
                    }
                    while ((l -= 64) > 63);
                }
                if (l > 0)
                {
                    if ((l & 32) != 0)
                    {
                        if (*(long*)p1 != *(long*)p2 || 
                            *(long*)(p1 + 8) != *(long*)(p2 + 8) ||
                            *(long*)(p1 + 16) != *(long*)(p2 + 16) ||
                            *(long*)(p1 + 24) != *(long*)(p2 + 24)
                            ) return false;
                        p1 += 32;
                        p2 += 32;
                    }
                    if ((l & 16) != 0)
                    {
                        if (*(long*)p1 != *(long*)p2 || *(long*)(p1 + 8) != *(long*)(p2 + 8)) return false;
                        p1 += 16;
                        p2 += 16;
                    }
                    if ((l & 8) != 0)
                    {
                        if (*(long*)p1 != *(long*)p2) return false;
                        p1 += 8;
                        p2 += 8;
                    }
                    if ((l & 4) != 0)
                    {
                        if (*(int*)p1 != *(int*)p2) return false;
                        p1 += 4;
                        p2 += 4;
                    }
                    if ((l & 2) != 0)
                    {
                        if (*(short*)p1 != *(short*)p2) return false;
                        p1 += 2;
                        p2 += 2;
                    }
                    if ((l & 1) != 0 && *p1 != *p2) return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(sbyte* p1, sbyte* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(ushort* p1, ushort* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l << 1);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(short* p1, short* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l << 1);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(char* p1, char* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l << 1);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(uint* p1, uint* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l << 2);
        }


        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(int* p1, int* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l << 2);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(ulong* p1, ulong* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l << 3);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(PByte* p1, PByte* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l * General.MultPByte);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(PChar* p1, PChar* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l * General.MultPByte);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(long* p1, long* p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l << 3);
        }

        /// <summary>
        /// Compara memoria y retorna true si los bloques son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(void** p1, void** p2, int l)
        {
            return Equal((byte*)p1, (byte*)p2, l << General.DespVoid);
        }

        #endregion

        #region Compare

        private static int compDif(byte* p1, byte* p2, int l)
        {
            int a = 0;
            while (p1[a] == p2[a]) a++;
            return a;
        }

        private static int compDif(char* p1, char* p2, int l)
        {
            int a = 0;
            while (p1[a] == p2[a]) a++;
            return a;
        }

        
        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(byte* p1, byte* p2, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p1 != *p2) return 0;
                return -1;
            }
            int l1 = l;
            if (l1 > 31)
            {
                if (p1 == p2)
                    return -1;
                do
                {
                    if (*((long*)p1) != *((long*)p2))
                        return l - l1 + compDif(p1, p2, 8);

                    if (*((long*)(p1 + 8)) != *((long*)(p2 + 8)))
                        return l - l1 + 8 + compDif(p1 + 8, p2 + 8, 8);

                    if (*((long*)(p1 + 16)) != *((long*)(p2 + 16)))
                        return l - l1 + 16 + compDif(p1 + 16, p2 + 16, 8);

                    if (*((long*)(p1 + 24)) != *((long*)(p2 + 24)))
                        return l - l1 + 24 + compDif(p1 + 24, p2 + 24, 8);

                    p1 += 32;
                    p2 += 32;
                }
                while ((l1 -= 32) > 31);
            }
            if (l1 == 0)
                return -1;

            if ((l1 & 16) != 0)
            {
                if (*((long*)p1) != *((long*)p2))
                    return l - l1 + compDif(p1, p2, 8);

                if (*((long*)(p1 + 8)) != *((long*)(p2 + 8)))
                    return l - l1 + 8 + compDif(p1 + 8, p2 + 8, 8);
                p1 += 16;
                p2 += 16;
                l1 -= 16;
            }

            if ((l1 & 8) != 0)
            {
                if (*((long*)p1) != *((long*)p2))
                    return l - l1 + compDif(p1, p2, 8);

                p1 += 8;
                p2 += 8;
                l1 -= 8;
            }

            if ((l1 & 4) != 0)
            {
                if (*((int*)p1) != *((int*)p2))
                    return l - l1 + compDif(p1, p2, 4);

                p1 += 4;
                p2 += 4;
                l1 -= 4;
            }

            if ((l1 & 2) != 0)
            {
                if (*((short*)p1) != *((short*)p2))
                    return l - l1 + compDif(p1, p2, 2);

                p1 += 2;
                p2 += 2;
                l1 -= 2;
            }

            if (l1 == 1 && *p1 != *p2) return l - 1;
            return -1;
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(char* p1, char* p2, int l)
        {
            if (l < 2)
            {
                if (l == 1 && *p1 != *p2) return 0;
                return -1;
            }
            int l1 = l;
            if (l1 > 15)
            {
                if (p1 == p2)
                    return -1;
                do
                {
                    if (*((long*)p1) != *((long*)p2))
                        return l - l1 + compDif(p1, p2, 4);

                    if (*((long*)(p1 + 4)) != *((long*)(p2 + 4)))
                        return l - l1 + 4 + compDif(p1 + 4, p2 + 4, 4);

                    if (*((long*)(p1 + 8)) != *((long*)(p2 + 8)))
                        return l - l1 + 8 + compDif(p1 + 8, p2 + 8, 4);

                    if (*((long*)(p1 + 12)) != *((long*)(p2 + 12)))
                        return l - l1 + 12 + compDif(p1 + 12, p2 + 12, 4);

                    p1 += 16;
                    p2 += 16;
                }
                while ((l1 -= 16) > 15);
            }
            if (l1 == 0)
                return -1;

            if ((l1 & 8) != 0)
            {
                if (*((long*)p1) != *((long*)p2))
                    return l - l1 + compDif(p1, p2, 4);

                if (*((long*)(p1 + 4)) != *((long*)(p2 + 4)))
                    return l - l1 + 4 + compDif(p1 + 4, p2 + 4, 4);
                p1 += 8;
                p2 += 8;
                l1 -= 8;
            }

            if ((l1 & 4) != 0)
            {
                if (*((long*)p1) != *((long*)p2))
                    return l - l1 + compDif(p1, p2, 4);

                p1 += 4;
                p2 += 4;
                l1 -= 4;
            }

            if ((l1 & 2) != 0)
            {
                if (*((int*)p1) != *((int*)p2))
                    return l - l1 + compDif(p1, p2, 2);

                p1 += 2;
                p2 += 2;
                l1 -= 2;
            }

            if (l1 == 1 && *p1 != *p2) return l - 1;
            return -1;
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(int* p1, int* p2, int l)
        {
            int a = 0;
            while (a < l)
            {
                if (p1[a] != p2[a])
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(long* p1, long* p2, int l)
        {
            int a = 0;
            while (a < l)
            {
                if (p1[a] != p2[a])
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(PByte* p1, PByte* p2, int l)
        {
            int a = 0;
            while (a < l)
            {
                if (p1[a] != p2[a])
                    return a;
                a++;
            }
            return -1;
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(sbyte* p1, sbyte* p2, int l)
        {
            return Compare((byte*)p1, (byte*)p2, l);
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(short* p1, short* p2, int l)
        {
            return Compare((char*)p1, (char*)p2, l);
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(ushort* p1, ushort* p2, int l)
        {
            return Compare((char*)p1, (char*)p2, l);
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(uint* p1, uint* p2, int l)
        {
            return Compare((int*)p1, (int*)p2, l);
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(ulong* p1, ulong* p2, int l)
        {
            return Compare((long*)p1, (long*)p2, l);
        }

        /// <summary>
        /// Compara memoria y retorna la ubicacin del primer elemento diferente
        /// o -1 si son iguales.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 si son iguales o primer elemento diferente (base 0).</returns>
        public static int Compare(PChar* p1, PChar* p2, int l)
        {
            return Compare((PByte*)p1, (PByte*)p2, l);
        }

        #endregion

        #region NoZero

        /// <summary>
        /// Retorna true si en la cadena hay un valor diferente a cero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si existe un elemento diferente a cero.</returns>
        public static bool NoZero(byte*p, int l)
        {
            if (l > 1)
            {
                if (l > 31)
                {
                    if (General.FrameWork64)
                    {
                        do
                        {
                            if (*((long*)p) != 0L || *((long*)(p + 8)) != 0L || *((long*)(p + 16)) != 0L || *((long*)(p + 24)) != 0L)
                                return true;
                            p += 32;
                        }
                        while ((l -= 32) > 31);
                    }
                    else
                    {
                        do
                        {
                            if (*((int*)p) != 0 || *((int*)(p + 4)) != 0 || *((int*)(p + 8)) != 0 || *((int*)(p + 12)) != 0 || *((int*)(p + 16)) != 0 || *((int*)(p + 20)) != 0 || *((int*)(p + 24)) != 0 || *((int*)(p + 28)) != 0)
                                return true;
                            p += 32;
                        }
                        while ((l -= 32) > 31);
                    }
                    if (l == 0)
                        return false;
                }
                if ((l & 16) != 0)
                {
                    if (*((int*)p) != 0 || *((int*)(p + 4)) != 0 || *((int*)(p + 8)) != 0 || *((int*)(p + 12)) != 0)
                        return true;
                    p += 16;
                    l -= 16;
                }
                if ((l & 8) != 0)
                {
                    if (*((int*)p) != 0 || *((int*)(p + 4)) != 0)
                        return true;
                    p += 8;
                    l -= 8;
                }
                if ((l & 4) != 0)
                {
                    if (*((int*)p) != 0)
                        return true;
                    p += 4;
                    l -= 4;
                }
                if ((l & 2) != 0)
                {
                    if (*((short*)p) != 0)
                        return true;
                    p += 2;
                    l -= 2;
                }
            }
            return l == 1 && *p != 0;
        }

        #endregion

        #region StrLen

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(byte*p)
        {
            int a=0;
            while(p[a]!=0)
                a++;
            return a;
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(sbyte*p)
        {
            return StrLen((byte*)p);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(char*p)
        {
            int a=0;
            while(p[a]!='\0')
                a++;
            return a;
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(short*p)
        {
            return StrLen((char*)p);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(ushort*p)
        {
            return StrLen((char*)p);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(int*p)
        {
            int a=0;
            while(p[a]!=0)
                a++;
            return a;
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(uint*p)
        {
            return StrLen((int*)p);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(long*p)
        {
            int a=0;
            while(p[a]!=0L)
                a++;
            return a;
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado.</returns>
        public static int StrLen(ulong*p)
        {
            return StrLen((long*)p);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(byte*p, int max)
        {
            if (max<1)
                return 0;
            int a=0;
            while(p[a]!=0 && ++a!=max);
            return a;
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(sbyte*p, int max)
        {
            return StrLen((byte*)p,max);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(char*p, int max)
        {
            if (max<1)
                return 0;
            int a=0;
            while(p[a]!='\0' && ++a!=max);
            return a;
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(short*p, int max)
        {
            return StrLen((char*)p,max);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(ushort*p, int max)
        {
            return StrLen((char*)p,max);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(int*p, int max)
        {
            if (max<1)
                return 0;
            int a=0;
            while(p[a]!=0 && ++a!=max);
            return a;
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(uint*p, int max)
        {
            return StrLen((int*)p,max);
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(long*p, int max)
        {
            if (max<1)
                return 0;
            int a=0;
            while(p[a]!=0L && ++a!=max);
            return a;
        }

        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="max">Lmite a la bsqueda.</param>
        /// <returns>La ubicacin del primer cero encontrado o max.</returns>
        public static int StrLen(ulong*p, int max)
        {
            return StrLen((long*)p,max);
        }


        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="sof">Tamao de cada elemento.</param>
         /// <param name="p">Puntero.</param>
        /// <returns>La ubicacin del primer cero encontrado (en unidades de elementos).</returns>
        public static int StrLen(int sof, void*p)
        {
            switch(sof)
            {
                case 0:
                    throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
                case 1:
                    return StrLen((byte*)p);
                case 2:
                    return StrLen((char*)p);
                case 4:
                    return StrLen((int*)p);
                case 8:
                    return StrLen((long*)p);
            }
            if (sof<0)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.NEGATIVE);
            int a=0;
            while(!NoZero((byte*)p+a,sof))
                a+=sof;
            return a/sof;
        }


        /// <summary>
        /// Obtiene largo de cadena.
        /// </summary>
        /// <param name="sof">Tamao de cada elemento.</param>
        /// <param name="p">Puntero.</param>
        /// <param name="maxItems">Lmite a la bsqueda (en unidades de elementos).</param>
        /// <returns>La ubicacin del primer cero encontrado (en unidades de elementos) o maxItems.</returns>
        public static int StrLen(int sof, void*p, int maxItems)
        {
            switch(sof)
            {
                case 0:
                    throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
                case 1:
                    return StrLen((byte*)p,maxItems);
                case 2:
                    return StrLen((char*)p,maxItems);
                case 4:
                    return StrLen((int*)p,maxItems);
                case 8:
                    return StrLen((long*)p,maxItems);
            }
            if (sof<0)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.NEGATIVE);
            if (maxItems<1)
                return 0;
            maxItems*=sof;
            int a=0;
            while(!NoZero((byte*)p+a,sof) && (a+=sof)<maxItems);
            return a/sof;
        }

        #endregion

    }
}
